//===========================================================================
//
// Regions part of the old ISC Clunkdraw routines.  Separate because you
// don't always want or need them.  These are necessary for the Window Manager
// and the rounded shapes, but not for basic drawing.
//
//===========================================================================

unit clunkregions;

{$mode objfpc}{$H+}

interface

   uses Clunkdraw;

   procedure ClipRectSect(r : Rect);
   procedure ClipRectUnion(r : Rect);
   procedure ClipRectDiff(r : Rect);
   procedure ClipRectXor(r : Rect);
   procedure SetClipSect(rgn : RgnHandle);
   procedure SetClipUnion(rgn : RgnHandle);
   procedure SetClipDiff(rgn : RgnHandle);
   procedure SetClipXor(rgn : RgnHandle);
   procedure MapRgn(rgn : RgnHandle; srcRect, dstRect : Rect);
   procedure SectRgn(srcRgnA, srcRgnB, dstRgn : RgnHandle);
   procedure UnionRgn(srcRgnA, srcRgnB, dstRgn : RgnHandle);
   procedure DiffRgn(srcRgnA, srcRgnB, dstRgn : RgnHandle);
   procedure XorRgn(srcRgnA, srcRgnB, dstRgn : RgnHandle);
   procedure InsetRgn(rgn : RgnHandle; dh, dv : smallInt);

   procedure FrameRgn(rgn : RgnHandle);
   function  BitMapToRegion(rgn : RgnHandle; var sbm : BitMap) : integer;

implementation

const
   EndMarker = $7FFF; // End-of-region/polygon-data marker value.

procedure ClipRectSect(r : Rect);
var
   tp : GrafPtr;
   temp : RgnHandle;
begin
   tp := thePort;
   temp := NewRgn();
   RectRgn(temp, r);
   SectRgn(tp^.clipRgn, temp, tp^.clipRgn);
   DisposeRgn(temp);
end;

procedure ClipRectUnion(r : Rect);
var
   tp : GrafPtr;
   temp : RgnHandle;
begin
   tp := thePort;
   temp := NewRgn();
   RectRgn(temp, r);
   UnionRgn(tp^.clipRgn, temp, tp^.clipRgn);
   DisposeRgn(temp);
end;

procedure ClipRectDiff(r : Rect);
var
   tp : GrafPtr;
   temp : RgnHandle;
begin
   tp := thePort;
   temp := NewRgn();
   RectRgn(temp, r);
   DiffRgn(tp^.clipRgn, temp, tp^.clipRgn);
   DisposeRgn(temp);
end;

procedure ClipRectXor(r : Rect);
var
   tp : GrafPtr;
   temp : RgnHandle;
begin
   tp := thePort;
   temp := NewRgn();
   RectRgn(temp, r);
   XorRgn(tp^.clipRgn, temp, tp^.clipRgn);
   DisposeRgn(temp);
end;

procedure SetClipSect(rgn : RgnHandle);
var
   tp : GrafPtr;
begin
   tp := thePort;
   SectRgn(tp^.clipRgn, rgn, tp^.clipRgn);
end;

procedure SetClipUnion(rgn : RgnHandle);
var
   tp : GrafPtr;
begin
   tp := thePort;
   UnionRgn(tp^.clipRgn, rgn, tp^.clipRgn);
end;

procedure SetClipDiff(rgn : RgnHandle);
var
   tp : GrafPtr;
begin
   tp := thePort;
   DiffRgn(tp^.clipRgn, rgn, tp^.clipRgn);
end;

procedure SetClipXor(rgn : RgnHandle);
var
   tp : GrafPtr;
begin
   tp := thePort;
   XorRgn(tp^.clipRgn, rgn, tp^.clipRgn);
end;

//
// CopyWorkRgn converts a region into a more verbose form that's better suited
// for use by the region merging routines.  It does it by copying it into a
// new place (although this isn't the most efficient way to do it).
//
// The new form is the same as the original, but it's always in scanline form
// (even if it was originally in rectangular form), and there's an extra null
// scanline on the front (which will be filled in later to make both regions
// involved in region math start on the same scanline) and an extra null
// scanline on the end to force expiry of the region at the end during the
// region math routines.
//
// The region's size field _must_ be accurate for this to work correctly.
//
procedure CopyWorkRgn(srcRgn : RgnHandle; ps : PsmallInt);
var
   ii : integer;
   q : PsmallInt;
   rp : ^Rect;
begin
   if srcRgn <> nil then begin          // Not if it isn't real.
      ii := srcRgn^^.rgnSize;
      if ii = sizeof(RegionHdr) then begin // Was it rectangular?
         ps^ := 0; inc(ps);     // Filled in by who called us.
         ps^ := EndMarker;  inc(ps);

         rp := @srcRgn^^.rgnBBox;
         ps^ := rp^.top;    inc(ps); // Construct a scanline version of a
         ps^ := rp^.left;   inc(ps); // simple rectangular region that's the
         ps^ := rp^.right;  inc(ps); // same as the region's bounding box.
         ps^ := EndMarker;  inc(ps);
         ps^ := rp^.bottom; inc(ps);
         ps^ := EndMarker;  inc(ps);
         ps^ := EndMarker;
      end else begin            // For complex regions, we use up the former
                                // region header to make room for additions.
         ps^ := 0; inc(ps);     // Filled in by who called us.
         ps^ := EndMarker;  inc(ps);

         ii -= sizeof(RegionHdr); // We don't want the region header.
         q := @srcRgn^^.rgnData;
         system.Move(q^, ps^, ii); // Copy the region's data.
         ps += ii div 2;
         dec(ps);               // Add a dummy (empty) scanline.
         ps^ := srcRgn^^.rgnBBox.bottom; inc(ps);
         ps^ := EndMarker;  inc(ps);
         ps^ := EndMarker;
      end;
   end;
end;

// Check for duplicate output scanline.  Discard duplicate scanlines.
procedure checkdups(var anchor, lastout, outptr : PsmallInt);
var
   ii, jj : integer;
   p, q : ^smallInt;
begin
   ii := anchor - lastout;      // Length of previous scanline.
   jj := outptr - anchor;       // Length of this scanline
   if ii <> jj then begin       // Lines differ in length,
      lastout := anchor;        // keep this output line.
      anchor := outptr;
   end else begin               // Same length, are they the same?
      p := anchor + 1;          // (Skip Y bytes, they're not the same
      q := lastout + 1;         //  and we don't care.)
      dec(ii);
      while ii > 0 do begin     // Check all the X byte markers.
         dec(ii);
         if p^ <> q^ then begin // Any differences?
            lastout := anchor;  // Yes, keep the output line.
            anchor := outptr;
            break;
         end;
         inc(p); inc(q);
      end;
   end;
   outptr := anchor;
end;


// Clean up the output region.
procedure wrapup(lasso : Rect; outbuf : PsmallInt; var urect : Rect; outptr : PsmallInt; dstRgn : RgnHandle);
var
   scanline : integer;
   scratchRp : RgnPtr;
begin
   scratchRp := RgnPtr(outbuf);
   lasso.top := scratchRp^.rgnData[0];
   lasso.bottom := urect.bottom;
   if (outptr[-1] = EndMarker) and (outptr[-3] = EndMarker) then begin // Is the
      outptr -= 2;              // last scanline empty?  Yes, trim it off.
      lasso.bottom := outptr^;
   end;
   outptr^ := EndMarker;        // Put on END_OF_REGION tag.
   inc(outptr);
   scanline := PByte(outptr) - PByte(outbuf); // Calculate region's
   if scanline >= 20 then begin    // size (in bytes).
      if scanline = 20 then begin // Consists of a single scanline, must be
         RectRgn(dstRgn, lasso); // only a rectangle then.
      end else begin            // Is a for-real complex region.
         scratchRp^.rgnSize := scanline; // Put in size.
         scratchRp^.rgnBBox := lasso;  // Put in bounding Rect.
         CopyRgn(@scratchRp, dstRgn); // Copy to its final resting place!
      end;
   end else begin               // Is an empty result (no valid scanlines).
      SetEmptyRgn(dstRgn);
   end;
end;

// Scale a region upwards or downwards in size.  Currently does not
// eliminate scanlines that collapse into nothingness.  Needs to call
// the checkdups function, and arrange for any necessary shrinking.
//
procedure MapRgn(rgn : RgnHandle; srcRect, dstRect : Rect);
var pt : Point;
    p : ^smallInt;
begin
   if (rgn = nil) or (rgn^ = nil) then exit;
   if rgn^^.rgnSize <> sizeof(RegionHdr) then begin
      p := @rgn^^.rgnData;
      while p^ <> EndMarker do begin // Do each scanline of the region.
         pt.v := p^;
         MapPt(pt, srcRect, dstRect);
         p^ := pt.v;
         inc(p);
         while p^ <> EndMarker do begin // Do each X pair of the scanline.
            pt.h := p^;
            MapPt(pt, srcRect, dstRect);
            p^ := pt.h;
            inc(p);
            pt.h := p^;
            MapPt(pt, srcRect, dstRect);
            p^ := pt.h;
            inc(p);
         end;
         inc(p);                // Bump to next Y coordinate.
      end;
   end;
   MapRect(rgn^^.rgnBBox, srcRect, dstRect);
end;

//
// Regarding getting working space for combining regions, in the worst
// case scenario we could need nearly 4x the space of the largest input
// region.  In the X direction you can end up with nearly 2x the segments,
// if B splits every segment of A.  However, in the Y direction a run can
// be split, and require coming back, for 3x the scanlines, and the middle
// scanline is 2x the size that it was.  Something like:
//
//                          (A: DE
// (A: DE) xor (B: FG)  ==>  B: DFGE
//                           C: DE)
//
// (3 wrds) xor (3 wrds) => (11 words)
//
// So, an expansion factor (XF) of 4 is called for.
const XF = 4;

      A_BIT    = 2;        // Used for running the inner loops.
      B_BIT    = 1;
      AOFFBOFF = 0;
      AOFFBON  = B_BIT;
      AONBOFF  = A_BIT;
      AONBON   = A_BIT or B_BIT;
//
// SectRgn calculates the intersection of two regions and places the
// intersection in a third region.  This does _not_ create the destination
// region; space must already have been allocated for it.  The destination
// region can be one of the source regions, if desired.
//
// If the regions do not intersect, or one of the regions is empty, the
// destination is set to the empty region defined by the rectangle (0,0)(0,0).
//
procedure SectRgn(srcRgnA, srcRgnB, dstRgn : RgnHandle);
var
   urect, lasso : Rect;
   scanline, penstate, start : integer;
   pA, pB, outptr, curlineA, curlineB, lastout, anchor : ^smallInt;
   holdingA, holdingB : Boolean;
   outbuf, workA, workB : ^smallInt;
   recta, rectb : ^Rect;
   Aval : smallInt;
begin
   if EmptyRgn(srcRgnA) or EmptyRgn(srcRgnB) then begin // Sources empty?
      SetEmptyRgn(dstRgn);      // Yes, result is empty.
      exit;
   end;
   if not SectRect(srcRgnA^^.rgnBBox, srcRgnB^^.rgnBBox, urect) then begin
      SetEmptyRgn(dstRgn);      // If the regions don't intersect, the
      exit;                     // result is empty.
   end;
   scanline := srcRgnA^^.rgnSize; // Temp var usage.
   penstate := srcRgnB^^.rgnSize; // "
   workA := GetMem(scanline + 5 * sizeof(smallInt));
   workB := GetMem(penstate + 5 * sizeof(smallInt));
   if penstate > scanline then  // Buffer thrice the size of largest input.
      scanline := penstate;
   outbuf := GetMem(XF * scanline + 100); // See above.
   CopyWorkRgn(srcRgnA, workA);
   CopyWorkRgn(srcRgnB, workB);
   pA := @workA[2];             // Skip over room for dummy scanline.
   pB := @workB[2];
   SetRect(lasso, 32767, 32767, -32768, -32768); // No first coordinates.

   recta := @srcRgnA^^.rgnBBox;
   rectb := @srcRgnB^^.rgnBBox;
   if recta^.top > rectb^.top then begin // They don't start in the same place,
      dec(pA); pA^ := EndMarker;        // add a dummy scanline so they do.
      dec(pA); pA^ := rectb^.top;
   end;
   if rectb^.top > recta^.top then begin
      dec(pB); pB^ := EndMarker;
      dec(pB); pB^ := recta^.top;
   end;
   curlineA := pA;              // Drop the current input scanline anchors.
   curlineB := pB;

   scanline := urect.top;
   outptr := @RgnPtr(outbuf)^.rgnData;
   anchor := outptr;
   lastout := @RgnPtr(outbuf)^.rgnBBox.bottom;
   lastout[0] := scanline;      // Put out a dummy empty scanline so we can
   lastout[1] := EndMarker;     // compare 1st output scanline to something.
   while scanline < urect.bottom do begin
      penstate := AOFFBOFF;     // Neither region is 'ON'.
      holdingA := pA^ = EndMarker;
      if not holdingA then      // More of this region left?
         inc(pA);               // Skip to X coordinate pairs.
      holdingB := pB^ = EndMarker;
      if not holdingB then
         inc(pB);
      outptr^ := scanline; inc(outptr); // Output a Y coordinate.
      // Loop across scanline.
      start := lasso.left;      // (To avoid compiler warning; not necessary.)
      while (pA^ <> EndMarker) or (pB^ <> EndMarker) do begin
         Aval := pA^;
         if Aval < pB^ then begin // A's changing.
            if penstate = AOFFBON then begin // Result off & going on.
               start := Aval;
            end;
            if penstate = AONBON then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if Aval > lasso.right then
                  lasso.right := Aval;
               outptr^ := start; inc(outptr);
               outptr^ := Aval;  inc(outptr);
            end;
            penstate := penstate xor AONBOFF;
            inc(pA);
         end else if pB^ < Aval then begin // B's changing.
            if penstate = AONBOFF then begin // Result off & going on.
               start := pB^;
            end;
            if penstate = AONBON then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if pB^ > lasso.right then
                  lasso.right := pB^;
               outptr^ := start; inc(outptr);
               outptr^ := pB^;   inc(outptr);
            end;
            penstate := penstate xor AOFFBON;
            inc(pB);
         end else begin         // A & B both changing.
            if penstate = AOFFBOFF then begin // Result off & going on.
               start := Aval;
            end;
            if penstate = AONBON then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if Aval > lasso.right then
                  lasso.right := Aval;
               outptr^ := start; inc(outptr);
               outptr^ := Aval;  inc(outptr);
            end;
            penstate := penstate xor AONBON;
            inc(pA);
            inc(pB);
         end;
      end;
      outptr^ := EndMarker; inc(outptr); // Mark end of X coordinate pairs.
      checkdups(anchor, lastout, outptr); // Check for dup line
      if not holdingA then
         inc(pA);               // Bump to the next Y coordinate in lists.
      if not holdingB then
         inc(pB);
      if pA^ <= pB^ then begin  // Current A line expired?
         curlineA := pA;        // Yes, use next A scanline.
         scanline := pA^;
      end;
      if pB^ <= pA^ then begin  // Current B line expired?
         curlineB := pB;        // Yes, use next B scanline.
         scanline := pB^;
      end;
      pA := curlineA;
      pB := curlineB;
   end;
   wrapup(lasso, outbuf, urect, outptr, dstRgn);
   FreeMem(outbuf);
   FreeMem(workB);
   FreeMem(workA);
end;

//
// UnionRgn calculates the union of two regions and places the union in a
// third region.  This does _not_ create the destination region; space must
// already have been allocated for it.  The destination region can be one of
// the source regions, if desired.
//
// If both regions are empty, the destination is set to the empty region
// defined by the rectangle (0,0)(0,0).
//
procedure UnionRgn(srcRgnA, srcRgnB, dstRgn : RgnHandle);
var
   urect, lasso : Rect;
   scanline, penstate, start : integer;
   pA, pB, outptr, curlineA, curlineB, lastout, anchor : ^smallInt;
   holdingA, holdingB : Boolean;
   outbuf, workA, workB : ^smallInt;
   recta, rectb : ^Rect;
   Aval : smallInt;
begin
   if EmptyRgn(srcRgnA) then begin      // One of the sources empty?
      CopyRgn(srcRgnB, dstRgn);         // Yes, result is the other one.
      exit;
   end;
   if EmptyRgn(srcRgnB) then begin
      CopyRgn(srcRgnA, dstRgn);
      exit;
   end;
   UnionRect(srcRgnA^^.rgnBBox, srcRgnB^^.rgnBBox, urect);
   scanline := srcRgnA^^.rgnSize; // Temp var usage.
   penstate := srcRgnB^^.rgnSize; // "
   workA := GetMem(scanline + 5 * sizeof(smallInt));
   workB := GetMem(penstate + 5 * sizeof(smallInt));
   if penstate > scanline then  // Buffer thrice the size of largest input.
      scanline := penstate;
   outbuf := GetMem(XF * scanline + 100); // See above.
   CopyWorkRgn(srcRgnA, workA);
   CopyWorkRgn(srcRgnB, workB);
   pA := @workA[2];             // Skip over room for dummy scanline.
   pB := @workB[2];
   SetRect(lasso, 32767, 32767, -32768, -32768); // No first coordinates.

   recta := @srcRgnA^^.rgnBBox;
   rectb := @srcRgnB^^.rgnBBox;
   if recta^.top > rectb^.top then begin // They don't start in the same place,
      dec(pA); pA^ := EndMarker;        // add a dummy scanline so they do.
      dec(pA); pA^ := rectb^.top;
   end;
   if rectb^.top > recta^.top then begin
      dec(pB); pB^ := EndMarker;
      dec(pB); pB^ := recta^.top;
   end;
   curlineA := pA;              // Drop the current input scanline anchors.
   curlineB := pB;

   scanline := urect.top;
   outptr := @RgnPtr(outbuf)^.rgnData;
   anchor := outptr;
   lastout := @RgnPtr(outbuf)^.rgnBBox.bottom;
   lastout[0] := scanline;      // Put out a dummy empty scanline so we can
   lastout[1] := EndMarker;     // compare 1st output scanline to something.
   while scanline < urect.bottom do begin
      penstate := AOFFBOFF;     // Neither region is 'ON'.
      holdingA := pA^ = EndMarker;
      if not holdingA then      // More of this region left?
         inc(pA);               // Skip to X coordinate pairs.
      holdingB := pB^ = EndMarker;
      if not holdingB then
         inc(pB);
      outptr^ := scanline; inc(outptr); // Output a Y coordinate.
      // Loop across scanline.
      start := lasso.left;      // (To avoid compiler warning; not necessary.)
      while (pA^ <> EndMarker) or (pB^ <> EndMarker) do begin
         Aval := pA^;
         if Aval < pB^ then begin // A's changing.
            if penstate = AOFFBOFF then begin // Result off & going on.
               start := Aval;
            end;
            if penstate = AONBOFF then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if Aval > lasso.right then
                  lasso.right := Aval;
               outptr^ := start; inc(outptr);
               outptr^ := Aval;  inc(outptr);
            end;
            penstate := penstate xor AONBOFF;
            inc(pA);
         end else if pB^ < Aval then begin // B's changing.
            if penstate = AOFFBOFF then begin // Result off & going on.
               start := pB^;
            end;
            if penstate = AOFFBON then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if pB^ > lasso.right then
                  lasso.right := pB^;
               outptr^ := start; inc(outptr);
               outptr^ := pB^;   inc(outptr);
            end;
            penstate := penstate xor AOFFBON;
            inc(pB);
         end else begin         // A & B both changing.
            if penstate = AOFFBOFF then begin // Result off & going on.
               start := Aval;
            end;
            if penstate = AONBON then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if Aval > lasso.right then
                  lasso.right := Aval;
               outptr^ := start; inc(outptr);
               outptr^ := Aval;  inc(outptr);
            end;
            penstate := penstate xor AONBON;
            inc(pA);
            inc(pB);
         end;
      end;
      outptr^ := EndMarker; inc(outptr); // Mark end of X coordinate pairs.
      checkdups(anchor, lastout, outptr); // Check for dup line
      if not holdingA then
         inc(pA);               // Bump to the next Y coordinate in lists.
      if not holdingB then
         inc(pB);
      if pA^ <= pB^ then begin  // Current A line expired?
         curlineA := pA;        // Yes, use next A scanline.
         scanline := pA^;
      end;
      if pB^ <= pA^ then begin  // Current B line expired?
         curlineB := pB;        // Yes, use next B scanline.
         scanline := pB^;
      end;
      pA := curlineA;
      pB := curlineB;
   end;
   wrapup(lasso, outbuf, urect, outptr, dstRgn);
   FreeMem(outbuf);
   FreeMem(workB);
   FreeMem(workA);
end;

//
// DiffRgn subtracts srcRgnB from srcRgnA and places the difference in a third
// region.  This does _not_ create the destination region; space must already
// have been allocated for it.  The destination region can be one of the
// source regions, if desired.
//
// If the first source region is empty, the destination is set to the empty
// region defined by the rectangle (0,0)(0,0).
//
procedure DiffRgn(srcRgnA, srcRgnB, dstRgn : RgnHandle);
var
   urect, lasso : Rect;
   scanline, penstate, start : integer;
   pA, pB, outptr, curlineA, curlineB, lastout, anchor : ^smallInt;
   holdingA, holdingB : Boolean;
   outbuf, workA, workB : ^smallInt;
   recta, rectb : ^Rect;
   Aval : smallInt;
begin
   if EmptyRgn(srcRgnA) then begin // Source A empty?
      SetEmptyRgn(dstRgn);      // Yes, result is empty.
      exit;
   end;
   if EmptyRgn(srcRgnB) or      // If subtracting nothing, or if the regions
      not SectRect(srcRgnA^^.rgnBBox, srcRgnB^^.rgnBBox, urect) then begin
      CopyRgn(srcRgnA, dstRgn); // don't intersect, the result is then
      exit;                     // only source A.
   end;
   urect := srcRgnA^^.rgnBBox;
   scanline := srcRgnA^^.rgnSize; // Temp var usage.
   penstate := srcRgnB^^.rgnSize; // "
   workA := GetMem(scanline + 5 * sizeof(smallInt));
   workB := GetMem(penstate + 5 * sizeof(smallInt));
   if penstate > scanline then  // Buffer thrice the size of largest input.
      scanline := penstate;
   outbuf := GetMem(XF * scanline + 100); // See above.
   CopyWorkRgn(srcRgnA, workA);
   CopyWorkRgn(srcRgnB, workB);
   pA := @workA[2];             // Skip over room for dummy scanline.
   pB := @workB[2];
   SetRect(lasso, 32767, 32767, -32768, -32768); // No first coordinates.

   recta := @srcRgnA^^.rgnBBox;
   rectb := @srcRgnB^^.rgnBBox;
   if recta^.top > rectb^.top then begin // They don't start in the same place,
      dec(pA); pA^ := EndMarker;        // add a dummy scanline so they do.
      dec(pA); pA^ := rectb^.top;
   end;
   if rectb^.top > recta^.top then begin
      dec(pB); pB^ := EndMarker;
      dec(pB); pB^ := recta^.top;
   end;
   curlineA := pA;              // Drop the current input scanline anchors.
   curlineB := pB;

   scanline := urect.top;
   outptr := @RgnPtr(outbuf)^.rgnData;
   anchor := outptr;
   lastout := @RgnPtr(outbuf)^.rgnBBox.bottom;
   lastout[0] := scanline;      // Put out a dummy empty scanline so we can
   lastout[1] := EndMarker;     // compare 1st output scanline to something.
   while scanline < urect.bottom do begin
      penstate := AOFFBOFF;     // Neither region is 'ON'.
      holdingA := pA^ = EndMarker;
      if not holdingA then      // More of this region left?
         inc(pA);               // Skip to X coordinate pairs.
      holdingB := pB^ = EndMarker;
      if not holdingB then
         inc(pB);
      outptr^ := scanline; inc(outptr); // Output a Y coordinate.
      // Loop across scanline.
      start := lasso.left;      // (To avoid compiler warning; not necessary.)
      while (pA^ <> EndMarker) or (pB^ <> EndMarker) do begin
         Aval := pA^;
         if Aval < pB^ then begin // A's changing.
            if penstate = AOFFBOFF then begin // Result off & going on.
               start := Aval;
            end;
            if penstate = AONBOFF then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if Aval > lasso.right then
                  lasso.right := Aval;
               outptr^ := start; inc(outptr);
               outptr^ := Aval;  inc(outptr);
            end;
            penstate := penstate xor AONBOFF;
            inc(pA);
         end else if pB^ < Aval then begin // B's changing.
            if penstate = AONBON then begin // Result off & going on.
               start := pB^;
            end;
            if penstate = AONBOFF then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if pB^ > lasso.right then
                  lasso.right := pB^;
               outptr^ := start; inc(outptr);
               outptr^ := pB^;   inc(outptr);
            end;
            penstate := penstate xor AOFFBON;
            inc(pB);
         end else begin         // A & B both changing.
            if penstate = AOFFBON then begin // Result off & going on.
               start := Aval;
            end;
            if penstate = AONBOFF then begin // Result on & going off.
               if start < lasso.left then
                  lasso.left := start;
               if Aval > lasso.right then
                  lasso.right := Aval;
               outptr^ := start; inc(outptr);
               outptr^ := Aval;  inc(outptr);
            end;
            penstate := penstate xor AONBON;
            inc(pA);
            inc(pB);
         end;
      end;
      outptr^ := EndMarker; inc(outptr); // Mark end of X coordinate pairs.
      checkdups(anchor, lastout, outptr); // Check for dup line
      if not holdingA then
         inc(pA);               // Bump to the next Y coordinate in lists.
      if not holdingB then
         inc(pB);
      if pA^ <= pB^ then begin  // Current A line expired?
         curlineA := pA;        // Yes, use next A scanline.
         scanline := pA^;
      end;
      if pB^ <= pA^ then begin  // Current B line expired?
         curlineB := pB;        // Yes, use next B scanline.
         scanline := pB^;
      end;
      pA := curlineA;
      pB := curlineB;
   end;
   wrapup(lasso, outbuf, urect, outptr, dstRgn);
   FreeMem(outbuf);
   FreeMem(workB);
   FreeMem(workA);
end;

//
// XorRgn calculates the difference between the union and the intersection of
// of srcRgnA and srcRgnB and places the result in dstRgn.  This does _not_
// create the destination region; space must already have been allocated for
// it.  The destination region can be one of the source regions, if desired.
//
// If the regions are coincident, the destination is set to the empty region
// defined by the rectangle (0,0)(0,0).
//
procedure XorRgn(srcRgnA, srcRgnB, dstRgn : RgnHandle);
var
   urect, lasso : Rect;
   scanline, penstate, start : integer;
   pA, pB, outptr, curlineA, curlineB, lastout, anchor : ^smallInt;
   holdingA, holdingB : Boolean;
   outbuf, workA, workB : ^smallInt;
   recta, rectb : ^Rect;
   Aval : smallInt;
begin
   if EqualRgn(srcRgnA, srcRgnB) then begin // Sources the same?
      SetEmptyRgn(dstRgn);      // Yes, result is empty.
      exit;
   end;
   if EmptyRgn(srcRgnA) then begin // An empty source, result's the
      CopyRgn(srcRgnB, dstRgn); // other source.
      exit;
   end;
   if EmptyRgn(srcRgnB) then begin
      CopyRgn(srcRgnA, dstRgn);
      exit;
   end;
   UnionRect(srcRgnA^^.rgnBBox, srcRgnB^^.rgnBBox, urect);
   scanline := srcRgnA^^.rgnSize; // Temp var usage.
   penstate := srcRgnB^^.rgnSize; // "
   workA := GetMem(scanline + 5 * sizeof(smallInt));
   workB := GetMem(penstate + 5 * sizeof(smallInt));
   if penstate > scanline then  // Buffer thrice the size of largest input.
      scanline := penstate;
   outbuf := GetMem(XF * scanline + 100); // See above.
   CopyWorkRgn(srcRgnA, workA);
   CopyWorkRgn(srcRgnB, workB);
   pA := @workA[2];             // Skip over room for dummy scanline.
   pB := @workB[2];
   SetRect(lasso, 32767, 32767, -32768, -32768); // No first coordinates.

   recta := @srcRgnA^^.rgnBBox;
   rectb := @srcRgnB^^.rgnBBox;
   if recta^.top > rectb^.top then begin // They don't start in the same place,
      dec(pA); pA^ := EndMarker;        // add a dummy scanline so they do.
      dec(pA); pA^ := rectb^.top;
   end;
   if rectb^.top > recta^.top then begin
      dec(pB); pB^ := EndMarker;
      dec(pB); pB^ := recta^.top;
   end;
   curlineA := pA;              // Drop the current input scanline anchors.
   curlineB := pB;

   scanline := urect.top;
   outptr := @RgnPtr(outbuf)^.rgnData;
   anchor := outptr;
   lastout := @RgnPtr(outbuf)^.rgnBBox.bottom;
   lastout[0] := scanline;      // Put out a dummy empty scanline so we can
   lastout[1] := EndMarker;     // compare 1st output scanline to something.
   while scanline < urect.bottom do begin
      penstate := AOFFBOFF;     // Neither region is 'ON'.
      holdingA := pA^ = EndMarker;
      if not holdingA then      // More of this region left?
         inc(pA);               // Skip to X coordinate pairs.
      holdingB := pB^ = EndMarker;
      if not holdingB then
         inc(pB);
      outptr^ := scanline; inc(outptr); // Output a Y coordinate.
      // Loop across scanline.
      start := lasso.left;      // (To avoid compiler warning; not necessary.)
      while (pA^ <> EndMarker) or (pB^ <> EndMarker) do begin
         Aval := pA^;
         if Aval < pB^ then begin // A's changing.
            if (penstate = AOFFBOFF) or (penstate = AONBON) then begin
               start := Aval;   // Result off & going on.
            end;
            if (penstate = AONBOFF) or (penstate = AOFFBON) then begin
               if start < lasso.left then       // Result on & going off.
                  lasso.left := start;
               if Aval > lasso.right then
                  lasso.right := Aval;
               outptr^ := start; inc(outptr);
               outptr^ := Aval;  inc(outptr);
            end;
            penstate := penstate xor AONBOFF;
            inc(pA);
         end else if pB^ < Aval then begin // B's changing.
            if (penstate = AOFFBOFF) or (penstate = AONBON) then begin
               start := pB^;    // Result off & going on.
            end;
            if (penstate = AONBOFF) or (penstate = AOFFBON) then begin
               if start < lasso.left then       // Result on & going off.
                  lasso.left := start;
               if pB^ > lasso.right then
                  lasso.right := pB^;
               outptr^ := start; inc(outptr);
               outptr^ := pB^;   inc(outptr);
            end;
            penstate := penstate xor AOFFBON;
            inc(pB);
         end else begin         // A & B both changing.
            penstate := penstate xor AONBON;
            inc(pA);
            inc(pB);
         end;
      end;
      outptr^ := EndMarker; inc(outptr); // Mark end of X coordinate pairs.
      checkdups(anchor, lastout, outptr); // Check for dup line
      if not holdingA then
         inc(pA);               // Bump to the next Y coordinate in lists.
      if not holdingB then
         inc(pB);
      if pA^ <= pB^ then begin  // Current A line expired?
         curlineA := pA;        // Yes, use next A scanline.
         scanline := pA^;
      end;
      if pB^ <= pA^ then begin  // Current B line expired?
         curlineB := pB;        // Yes, use next B scanline.
         scanline := pB^;
      end;
      pA := curlineA;
      pB := curlineB;
   end;
   wrapup(lasso, outbuf, urect, outptr, dstRgn);
   FreeMem(outbuf);
   FreeMem(workB);
   FreeMem(workA);
end;

//
// InsetRgn shrinks or expands the region.  All points on the region boundary
// are moved inwards a distance of dv vertically and dh horizontally; if dh or
// dv is negative, the points are moved outwards in that direction.  InsetRgn
// leaves the region 'centered' at the same position, but moves the outline in
// (for positive values of dh and dv) or out (for negative values).  InsetRgn
// of a rectangular region works just like InsetRect.
//
// InsetRgn works by intersecting the region with itself shifted vertically
// over the range [-dv ... +dv] (by ones) for the vertical inset, and by
// shrinking the length of each horizontal segment by 2*dh.  For large
// vertical insets, this can be very slow.
//
procedure InsetRgn(rgn : RgnHandle; dh, dv : smallInt);
var
   rp : RgnPtr;
   p, q : ^smallInt;
   ii, jj, size : smallInt;
   copy : RgnPtr;
begin
   if (rgn <> nil) and (rgn^ <> nil) then begin
      rp := rgn^;
      size := rp^.rgnSize;
      if size = sizeof(RegionHdr) then begin
         InsetRect(rp^.rgnBBox, dh, dv);
         if EmptyRect(rp^.rgnBBox) then
            SetEmptyRgn(rgn);
      end else begin
         if dv <> 0 then begin          // Do vertical smear-intersection.
            copy := GetMem(size);
            system.Move(rp^, copy^, size); // Copy the region.
            OffsetRgn(@copy, 0, -dv);
            ii := 0;
            while ii < (dv * 2) do begin
               SectRgn(@copy, rgn, rgn);
               if ii <> (dv * 2 - 1) then begin
                  if ii = (dv - 1) then
                     jj := 2
                  else
                     jj := 1;
                  OffsetRgn(@copy, 0, jj);
               end;
               inc(ii);
            end;
            rp := rgn^;
            InsetRect(rp^.rgnBBox, dh, 0);
         end;
         rp := rgn^;
         size := rp^.rgnSize;
         if size <> sizeof(RegionHdr) then begin
            p := @rp^.rgnData;
            q := p;
            while p^ <> EndMarker do begin // Do each scanline of the region.
               q^ := p^;
               inc(q); inc(p);
               while p^ <> EndMarker do begin // Do each X pair of the line.
                  p[0] += dh;   // Squish the scanline segments.
                  p[1] -= dh;
                  if p[0] < p[1] then begin // Still valid scanline segment?
                     q^ := p^;  // Yes, copy it into destination.
                     inc(q); inc(p);
                     q^ := p^;
                     inc(q); inc(p);
                  end else begin // Drop it from the list.
                     size -= 4;
                     p += 2;
                  end;
               end;
               q^ := p^;        // Bump to next Y coordinate.
               inc(q); inc(p);
            end;
            q^ := p^;           // Copy over end tag..
            inc(q); inc(p);
            rgn^^.rgnSize := size; // Update size field of region.
         end;
      end;
   end;
end;

//
// FrameRgn draws an outline just inside the specified region, using the
// current grafPort's pen pattern, mode, and size.  The outline is as wide as
// the pen width and as tall as the pen height.  It's drawn with the pnPat,
// according to the pattern transfer mode specified by pnMode.  The outline
// will never go outside the region boundary.  The pen location is not changed
// by this procedure.
//
// Note:  FrameRgn actually does a CopyRgn, an InsetRgn, and a DiffRgn.
//
procedure FrameRgn(rgn : RgnHandle);
var
   tp : GrafPtr;
   temp : RgnHandle;
begin
   tp := thePort;
   if tp = nil then exit;
   if tp^.rgnSave <> nil then           // Region accumulation in progress?
      UnionRgn(rgn, tp^.rgnSave, tp^.rgnSave);
   if tp^.pnVis >= 0 then begin
      temp := NewRgn();
      if temp <> nil then begin
         CopyRgn(rgn, temp);
         InsetRgn(temp, tp^.pnSize.h, tp^.pnSize.v);
         DiffRgn(rgn, temp, temp);
         PaintRgn(temp);
         DisposeRgn(temp);
      end;
   end;
end;

// The BitMapToRegion function converts a given Bitmap record to a
// region.  You would generally use this region later for drawing
// operations. The region parameter must be a valid region handle
// created with the NewRgn function. The old region contents are lost.
//
function BitMapToRegion(rgn : RgnHandle; var sbm : BitMap) : OSErr;
var x, sx, scanline : integer;
    bm : BitMap;
    tp : GrafPtr;
    freeBm : Boolean;
    lastBit, nextBit : Boolean;
    outPtr, outBuf, lastout, anchor : ^smallInt;
    lasso : Rect;

   function srcBitIsSet(x, y : integer) : Boolean;
   var mask : byte;
       src : Pbyte;
   begin
      y -= bm.bounds.top;
      if (y < 0) or (y >= bm.bounds.bottom) then exit(False);
      x -= bm.bounds.left;
      if (x < 0) or (x >= bm.bounds.right) then exit(False);
      src := bm.baseAddr;   // Base address of source image.
      src += y * bm.rowBytes;
      src += x div 8;
      mask := 1 << (7 - (x and 7));
      srcBitIsSet := (src^ and mask) <> 0;
   end;

begin
   if (rgn = nil) or (rgn^ = nil) then exit(1);
   freeBm := False;
   bm := sbm;
   tp := thePort;
   if (tp <> nil) and (@sbm = @tp^.portBits) then begin
      if tp^.device = 1 then begin
         if not RotateBMapCW(sbm, bm) then exit(1);
         freeBm := True;
      end;
   end;
   SetRect(lasso, 32767, bm.bounds.top,
                 -32768, bm.bounds.bottom); // No first coordinates.

   // Calculate the worst-possible-case region size, for working storage.
   x := (bm.bounds.right - bm.bounds.left) div 2; // (Max scanline segments.)
   x *= 2 * sizeof(smallInt);             // (Size of scanline segment.)
   x += 2 * sizeof(smallInt);             // (Size of Y markers in scanline.)
   x *= bm.bounds.bottom - bm.bounds.top; // (Number of scanlines.)
   x += sizeof(word) + sizeof(RegionHdr); // (Region header.)
   x += 2 * sizeof(smallInt);             // (Dummy starting scanline.)
   outbuf := GetMem(x);                   // Probably very overlarge.

   scanline := bm.bounds.top;
   outptr := @RgnPtr(outbuf)^.rgnData;
   anchor := outptr;
   lastout := @RgnPtr(outbuf)^.rgnBBox.bottom;
   lastout[0] := scanline;      // Put out a dummy empty scanline so we can
   lastout[1] := EndMarker;     // compare 1st output scanline to something.
   while scanline < bm.bounds.bottom do begin
      lastBit := False;
      outPtr^ := scanline; inc(outPtr);
      sx := bm.bounds.left;
      for x := bm.bounds.left to bm.bounds.right + 1 do begin
         nextBit := srcBitIsSet(x, scanline);
         if nextBit <> lastBit then begin
            lastBit := nextBit;
            if not lastBit then begin // This WAS a pen-down segment.
               outPtr^ := sx; inc(outPtr);
               outPtr^ := x; inc(outPtr);
               if sx < lasso.left then
                  lasso.left := sx;
               if x > lasso.right then
                  lasso.right := x;
            end;
            sx := x;
         end;
      end;
      outptr^ := EndMarker; inc(outptr); // Mark end of X coordinate pairs.
      checkdups(anchor, lastout, outptr);
      inc(scanline);
   end;
   outPtr^ := EndMarker; inc(outPtr);
   wrapup(lasso, outbuf, bm.bounds, outptr, rgn);
   FreeMem(outbuf);
   if freeBm then
      FreeMem(bm.baseAddr);
   BitMapToRegion := 0;
end;

initialization

end.
