//===========================================================================
//
// Windowing part of the old ISC Clunkdraw routines.  Separate because you
// don't always want or need them.
//
//===========================================================================

unit clunkwinds;

{$mode objfpc}{$H+}

interface

   uses Clunkdraw, Clunktext;

// Window types/messages.
const
   documentProc  = 0;   // Regular window with all the trimmings.
   dBoxProc      = 1;   // No title bar, inset border.
   plainDBox     = 2;   // No title, no dropshadow.
   altDBoxProc   = 3;   // No title, dropshadow.
   noGrowDocProc = 4;   // Fixed size regular window.
   noBordDBox    = 5;   // No border/fill at all.
   iconWProc     = 6;   // Iconized window.

//
// Values for NewWindow's goAwayFlag field (window options).  These're for
// documentProc and noGrowDocProc types of windows only.
//
type
   GoAwayItem = (hasGoAway,     // Has go-away region (active window only).
                 hasZoom,       // Has zoom region (active window only).
                 hasSendToBack, // Has send-to-back rgn (active wdo only).
                 hasGrow,       // Has grow zone.
                 hasDrag,       // Has drag area.
                 isShy          // Is shy, must hit drag region to select.
   );
   GoAway = set of GoAwayItem;

//
// Values returned by FindWindow.
//
const
   inDesk       = 0;    // In desktop.
   inMenuBar    = 1;    // In Menu bar.
   inContent    = 3;    // In content rgn (except grow, if active).
   inDrag       = 4;    // In drag region.
   inGrow       = 5;    // In grow region (active window only).
   inGoAway     = 6;    // In go-away region (active window only).
   inZoom       = 7;    // In zoom region (active window only).
   inSendToBack = 8;    // In send-to-back rgn (active window only).

//
// Messages to the Window definition function.
//
const
   wDraw        = 0;    // Draw Window Frame.
   wHit         = 1;    // Tell what region mouse hit was in.
   wCalcRgns    = 2;    // Calculate strucRgn & contRgn.
   wNew         = 3;    // Do any additional window initialization.
   wDispose     = 4;    // Do any additional disposal actions.
   wGrow        = 5;    // Draw window's grow image.
   wDrawGIcon   = 6;    // Draw size box in content region.

//
// Returned by Window definition function's wHit routine.
//
const
   wNoHit        = 0;   // None of the following.
   wInContent    = 1;   // In content rgn (except grow, if active).
   wInDrag       = 2;   // In drag region.
   wInGrow       = 3;   // In grow region (active window only).
   wInGoAway     = 4;   // In go-away region (active window only).
   wInZoom       = 5;   // In zoom box.
   wInSendToBack = 6;   // In send-to-back box.

type
   WindowPtr = GrafPtr;
   WindowPeek = ^WindowRecord;

   WindowRecord = packed record
      port        : GrafPort;   // Window's GrafPort.
      windowKind  : smallInt;   // Window class.
      visible     : Boolean;    // TRUE if visible (_not_ same as non-overlapped)
      hilited     : Boolean;    // TRUE if highlighted.
      goAwayFlag  : GoAway;     // Window dingus options.
      spareFlag   : Boolean;    // not used.
      strucRgn    : RgnHandle;  // Structure Region. */
      contRgn     : RgnHandle;  // Content Region.
      updateRgn   : RgnHandle;  // Update Region.
      saveVisRgn  : RgnHandle;  // Saved visRgn during update.
      titleHandle : string;     // Window's title.
      titleWidth  : smallInt;   // Width of title in pixels.
      nextWindow  : WindowPeek; // Next window (behind) in window list.
      refCon      : PtrUInt;    // Window's reference value.
      pid         : smallInt;   // Process ID of window's creator.
      windowType  : smallInt;   // Window type (procID from newWindow).
   end;

// Following are the Window Manager routine declarations.

   procedure InitWindows;
   procedure EndWindows;
   procedure GetWMgrPort(var wPort : GrafPtr);
   function  NewWindow(boundsRect : Rect; title : string; visible : Boolean;
                       procID : smallInt; behind : WindowPtr;
                       goAwayFlag : GoAway; refCon : PtrUInt): WindowPtr;
   procedure WindowStyle(arg:smallInt);
   procedure DisposeWindow(theWindow    : WindowPtr);
   procedure SetWTitle(theWindow : WindowPtr; title : string);
   procedure GetWTitle(theWindow : WindowPtr; var title : string);
   procedure SelectWindow(theWindow : WindowPtr);
   procedure HideWindow(theWindow : WindowPtr);
   procedure ShowWindow(theWindow : WindowPtr);
   procedure ShowHide(theWindow : WindowPtr; showFlag : Boolean);
   procedure HiliteWindow(theWindow : WindowPtr; hilite : Boolean);
   procedure BringToFront(theWindow : WindowPtr);
   procedure SendBehind(theWindow, behindWindow : WindowPtr);
   function  FrontWindow : WindowPtr;
   procedure DrawGrowIcon(theWindow : WindowPtr);
   function  FindWindow(thePt : Point; var whichWindow : WindowPtr) : smallInt;
   function  TrackBox(theWindow : WindowPtr; thePt : Point; partCode : integer) : Boolean;
   procedure MoveWindow(theWindow : WindowPtr; hGlobal, vGlobal : smallInt; makefront : Boolean);
   function  DragWindow(theWindow : WindowPtr; delta : Point) : Boolean;
   function  GrowWindow(theWindow : WindowPtr; delta : Point) : Boolean;
   procedure SizeWindow(theWindow : WindowPtr; w, h : smallInt; fUpdate : Boolean);
   procedure ZoomWindow(theWindow : WindowPtr; newRect : Rect; makeFront : Boolean);
   procedure InvalRect(badRect : Rect);
   procedure InvalRgn(badRgn : RgnHandle);
   procedure ValidRect(goodRect : Rect);
   procedure ValidRgn(goodRgn : RgnHandle);
   procedure BeginUpdate(theWindow : WindowPtr);
   procedure BeginUpdateRect(theWindow : WindowPtr; uprect : Rect);
   procedure EndUpdate(theWindow : WindowPtr);
   procedure SetWRefCon(theWindow : WindowPtr; data : PtrUInt);
   function  GetWRefCon(theWindow : WindowPtr) : PtrUInt;
   procedure ChangeWindow(theWindow : WindowPeek; newtype : smallInt);
   function  CheckUpdate(window: WindowPtr) : WindowPtr;
   procedure ClipAbove(window : WindowPeek);
   procedure SaveOld(window : WindowPeek);
   procedure DrawNew(window : WindowPeek; update : Boolean);
   procedure PaintBehind(startWindow : WindowPeek; clobberedRgn : RgnHandle);
   procedure PaintOne(window : WindowPeek; clobberedRgn : RgnHandle);
   procedure CalcVis(theWindow : WindowPeek);
   procedure CalcVisBehind(startWindow : WindowPeek; clobberedRgn : RgnHandle);

   procedure ScrollRect(r : Rect; dh, dv : smallInt; updateRgn : RgnHandle);
//   procedure DumpWindows(s : string);
//
// Standard set of 38 patterns.
//
const
   syspats : array [0..37] of Pattern = (
      (bytes:($FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF)),
      (bytes:($DD, $FF, $77, $FF, $DD, $FF, $77, $FF)),
      (bytes:($DD, $77, $DD, $77, $DD, $77, $DD, $77)),
      (bytes:($AA, $55, $AA, $55, $AA, $55, $AA, $55)),
      (bytes:($55, $FF, $55, $FF, $55, $FF, $55, $FF)),
      (bytes:($AA, $AA, $AA, $AA, $AA, $AA, $AA, $AA)),
      (bytes:($EE, $DD, $BB, $77, $EE, $DD, $BB, $77)),
      (bytes:($88, $88, $88, $88, $88, $88, $88, $88)),
      (bytes:($B1, $30, $03, $1B, $D8, $C0, $0C, $8D)),
      (bytes:($80, $10, $02, $20, $01, $08, $40, $04)),
      (bytes:($FF, $88, $88, $88, $FF, $88, $88, $88)),
      (bytes:($FF, $80, $80, $80, $FF, $08, $08, $08)),
      (bytes:($80, $00, $00, $00, $00, $00, $00, $00)),
      (bytes:($80, $40, $20, $00, $02, $04, $08, $00)),
      (bytes:($82, $44, $39, $44, $82, $01, $01, $01)),
      (bytes:($F8, $74, $22, $47, $8F, $17, $22, $71)),
      (bytes:($55, $A0, $40, $40, $55, $0A, $04, $04)),
      (bytes:($20, $50, $88, $88, $88, $88, $05, $02)),
      (bytes:($BF, $00, $BF, $BF, $B0, $B0, $B0, $B0)),
      (bytes:($00, $00, $00, $00, $00, $00, $00, $00)),
      (bytes:($80, $00, $08, $00, $80, $00, $08, $00)),
      (bytes:($88, $00, $22, $00, $88, $00, $22, $00)),
      (bytes:($88, $22, $88, $22, $88, $22, $88, $22)),
      (bytes:($AA, $00, $AA, $00, $AA, $00, $AA, $00)),
      (bytes:($FF, $00, $FF, $00, $FF, $00, $FF, $00)),
      (bytes:($11, $22, $44, $88, $11, $22, $44, $88)),
      (bytes:($FF, $00, $00, $00, $FF, $00, $00, $00)),
      (bytes:($01, $02, $04, $08, $10, $20, $40, $80)),
      (bytes:($AA, $00, $80, $00, $88, $00, $80, $00)),
      (bytes:($FF, $80, $80, $80, $80, $80, $80, $80)),
      (bytes:($08, $1C, $22, $C1, $80, $01, $02, $04)),
      (bytes:($88, $14, $22, $41, $88, $00, $AA, $00)),
      (bytes:($40, $A0, $00, $00, $04, $0A, $00, $00)),
      (bytes:($03, $84, $48, $30, $0C, $02, $01, $01)),
      (bytes:($80, $80, $41, $3E, $08, $08, $14, $E3)),
      (bytes:($10, $20, $54, $AA, $FF, $02, $04, $08)),
      (bytes:($77, $89, $8F, $8F, $77, $98, $F8, $F8)),
      (bytes:($00, $08, $14, $2A, $55, $2A, $14, $08))
   );

implementation
   uses Clunkregions;

const
   EndMarker = $7FFF; // End-of-region/polygon-data marker value.

var
   weenieflag : smallInt = 3;   // Bit 0 = weenie title bar fill,
                                // Bit 1 = rectangular control boxes,
                                // Bit 2 = all-on-one-side rectangles.
                                // Bit 3 = no titles.
   titleheit : smallInt = 18;
   WindowList : WindowPeek;     // Start of Window List (-> frontmost).
   WMgrPort : GrafPort;         // Window Manager's GrafPort.
   DeskPattern : Pattern;       // Fill desktop with this.
   GrayRgn : RgnHandle;         // Desktop Region.
   PaintWhite : Boolean;        // White (erase) painting flag for updates.
   SaveUpdate : Boolean;
   DragFlag : Boolean;          // Flag TRUE when moving window around.
   menubar : smallInt;          // Amount of desktop rsvd for menu bar.

procedure WindowStyle(arg : smallInt);
begin
   weenieflag := arg;
   if arg > 15 then
      titleheit := arg >> 4;
end;

procedure calcwrects(var contbox, closebox, zoombox, backbox,
                         growbox, dragbox : Rect);
var ii : integer;
    flag1, flag2 : Boolean;
begin
   flag1 := (weenieflag and 2) <> 0;
   flag2 := (weenieflag and 4) <> 0;
   dragbox := contbox;
   ii := dragbox.top;
   dragbox.top := ii - titleheit;
   dragbox.bottom := ii + 1;

   backbox.top := ii - (titleheit - 3);
   closebox.top := backbox.top;
   backbox.bottom := ii - 4;
   closebox.bottom := backbox.bottom;
   closebox.left := dragbox.left + 5;
   if flag1 then begin
      closebox.right := closebox.left + titleheit - 3;
      if flag2 then
         backbox.right := closebox.right + (titleheit * 2) + 8
      else
         backbox.right := dragbox.right - 8;
   end else begin
      closebox.right := closebox.left + titleheit - 7;
      if flag2 then
         backbox.right := closebox.right + (titleheit * 2)
      else
         backbox.right := dragbox.right - 5;
   end;

   zoombox := backbox;
   if flag1 then
      OffsetRect(zoombox, -(titleheit + 4), 0)
   else
      OffsetRect(zoombox, -titleheit, 0);

   growbox := contbox;
   growbox.left := growbox.right - 17;
   growbox.top := growbox.bottom - 17;
end;

function prep(var r : Rect; param, varcode : Boolean) : Boolean;
begin
   InsetRect(r, 1, 1);
   if (param) then
      PenPat(black)
   else
      PenPat(white);
   FrameRect(r);
   PenPat(black);
   if not varcode then begin
      EraseRect(r);
      InsetRect(r, -1, -1);
      FrameRect(r);
      InsetRect(r, -1, -1);
      PenPat(white);
      FrameRect(r);
      PenPat(black);
   end;
   prep := not varcode;
end;

// Window def function.
function MyWindow(varcode   : integer;          // Window variation code (type).
                  theWindow : WindowPeek;       // Window to work on.
                  message,                      // What to do to it.
                  param     : PtrUInt) : PtrUint;
var
   scratch, scratch2 : Rect;
   closebox, zoombox, backbox, growbox, dragbox : Rect;
   hit : Point;
   arect : RgnHandle;
   saveport : GrafPtr;
   wid : integer;                       // String width.
begin
   scratch := theWindow^.port.portRect;
   GetPort(saveport);
   SetPort(@theWindow^.port);
   LocalToGlobal(scratch.topLeft);
   LocalToGlobal(scratch.botRight);
   SetPort(saveport);
   GlobalToLocal(scratch.topLeft);
   GlobalToLocal(scratch.botRight);
   case message of
      wDraw: begin              // Draw the window frame.
        if theWindow^.visible then begin
            case theWindow^.windowType of
              dBoxProc: begin
                PenSize(2, 2);
                InsetRect(scratch, -2, -2);
                FrameRect(scratch); // Frame it.
                PenPat(white);
                InsetRect(scratch, -2, -2);
                FrameRect(scratch); // Frame it.
                PenNormal;
                InsetRect(scratch, -1, -1);
                FrameRect(scratch); // Frame it.
              end;

              altDBoxProc: begin
                scratch2 := scratch; // Make a copy of globalized portRect.
                scratch2.top += 2;
                inc(scratch2.bottom);
                scratch2.left := scratch2.right + 1;
                scratch2.right += 3;
                PaintRect(scratch2); // Dropshadow the right side.

                scratch2 := scratch; // Make a copy
                scratch2.top := scratch2.bottom + 1;
                scratch2.left += 2;
                scratch2.right += 3;
                scratch2.bottom += 3;
                PaintRect(scratch2); // Dropshadow the bottom.
                InsetRect(scratch, -1, -1);
                FrameRect(scratch); // Frame it.
              end;

              plainDBox: begin
                InsetRect(scratch, -1, -1);
                FrameRect(scratch); // Frame it.
              end;

              documentProc: begin
                calcwrects(scratch, closebox, zoombox, backbox,growbox, dragbox);
                scratch2 := scratch; // Make copy of globalized portRect.
                if varcode = 0 then begin
                    InsetRect(scratch, -1, -1);
                    scratch.top -= titleheit;
                    FrameRect(scratch); // Frame the active region.
                    scratch.bottom := scratch.top + titleheit + 1;
                    InsetRect(scratch, 1, 1);
                    MoveTo(scratch.left, scratch.bottom);
                    LineTo(scratch.right, scratch.bottom);
                    EraseRect(scratch); // Title bar itself.
                    if theWindow^.hilited then begin // Put up highlight lines.
                        wid := 3;
                        while wid < (titleheit - 4) do begin
                            MoveTo(scratch.left+1, scratch.top + wid);
                            LineTo(scratch.right-2, scratch.top + wid);
                            wid += 2;
                        end;
                    end;
                    if (weenieflag and 8) = 0 then begin
                        wid := (scratch.left + scratch.right) div 2
                             - theWindow^.titleWidth div 2;
                        scratch.left := wid - 7;
                        scratch.right := wid + theWindow^.titleWidth + 7;
                        EraseRect(scratch);
///                        TextSize(0);
///                        TextFont(0);
                        MoveTo(wid, scratch.top +
                                    BEtoN(thePort^.fontPtr^.ascent) + 4);
                        DrawString(theWindow^.titleHandle);
                    end;
                end;
                if theWindow^.hilited then begin // Only do widgets if active.
                    if hasGoAway in theWindow^.goAwayFlag then begin
                        if (varcode = 0) or (varcode = inGoAway) then begin
                            if prep(closebox, Boolean(param), Boolean(varcode)) then begin
                                scratch := closebox;
                                InsetRect(scratch, 5, 5);
                                FrameRect(scratch);
                            end;
                        end;
                    end;
                    if hasZoom in theWindow^.goAwayFlag then begin
                        if (varcode = 0) or (varcode = inZoom) then begin
                            if prep(zoombox, Boolean(param), Boolean(varcode)) then begin
                                InsetRect(zoombox, 1, 1);
                                scratch := zoombox;
                                scratch.top += 7;
                                scratch.right -= 7;
                                FrameRect(scratch);
                                scratch.top -= 4;
                                scratch.right += 4;
                                FrameRect(scratch);
                            end;
                        end;
                    end;
                    if  hasSendToBack in theWindow^.goAwayFlag then begin
                        if (varcode = 0) or (varcode = inSendToBack) then begin
                            if prep(backbox, Boolean(param), Boolean(varcode)) then begin
                                InsetRect(backbox, 1, 1);
                                scratch := backbox;
                                scratch.bottom -= 3;
                                scratch.left += 3;
                                scratch.top += 2;
                                scratch.right -= 2;
                                FrameRect(scratch);
                                scratch.bottom += 1;
                                scratch.left -= 1;
                                scratch.top += 2;
                                scratch.right -= 2;
                                EraseRect(scratch);
                                FrameRect(scratch);
                            end;
                        end;
                    end;
                end;

                if varcode = 0 then begin       // Do dropshadows.
                    scratch := scratch2;
                    scratch.top -= titleheit;
                    inc(scratch.bottom);
                    scratch.left := scratch.right + 1;
                    scratch.right += 2;
                    PaintRect(scratch); // Dropshadow the right side.

                    scratch2.top := scratch2.bottom + 1;
                    scratch2.left += 1;
                    scratch2.right += 2;
                    scratch2.bottom += 2;
                    PaintRect(scratch2); // Dropshadow the bottom.
                end;
              end;

              noBordDBox: begin
              end;              // No frame drawing required.

              iconWProc: begin
                InsetRect(scratch, -1, -1);
                scratch.top -= 10;
                FrameRect(scratch); // Frame it.
                zoombox := scratch;
                zoombox.bottom := zoombox.top + 11;
                scratch := zoombox;
                InsetRect(scratch, 1, 1);
                zoombox.left := zoombox.right - 11;
                if varcode = 0 then begin       // Drawing whole frame?
                    MoveTo(scratch.left, scratch.bottom);
                    LineTo(scratch.right, scratch.bottom);
                    EraseRect(scratch); // Title bar itself.
                    if theWindow^.hilited then begin // Put up highlight lines.
                        wid := 2;
                        while wid < 8 do begin
                            MoveTo(scratch.left+1, scratch.top + wid);
                            LineTo(scratch.right-2, scratch.top + wid);
                            wid +=2;
                        end;
                    end;
                end;
                if theWindow^.hilited then begin // Only do widgets if active.
                    if hasZoom in theWindow^.goAwayFlag then begin
                        if (varcode = 0) or (varcode = inZoom) then begin
                            InsetRect(zoombox, 1, 1);
                            if param <> 0 then
                                PenPat(black)
                            else
                                PenPat(white);
                            FrameRect(zoombox);
                            PenPat(black);
                            if varcode = 0 then begin
                                zoombox.left -= 2;
                                EraseRect(zoombox);
                                zoombox.left += 2;
                                InsetRect(zoombox, -1, -1);
                                FrameRect(zoombox);
                            end;
                        end;
                    end;
                end;
              end;
            end;
        end;
      end;

      wHit: begin               // Find hit point within window.
        if theWindow^.visible then begin
            if param = 0 then begin
                MyWindow := wNoHit;
                exit;
            end;
            hit := Point(DWord(param));
            GlobalToLocal(hit);
            case theWindow^.windowType of
              dBoxProc,
              altDBoxProc,
              plainDBox,
              noBordDBox: begin
                if PtInRect(hit, scratch) then begin
                    MyWindow := wInContent;
                    exit;
                end;
              end;

              documentProc: begin
                calcwrects(scratch, closebox, zoombox, backbox,growbox, dragbox);
                if theWindow^.hilited then begin // Dinguses only when in front.
                    if hasGoAway in theWindow^.goAwayFlag then
                        if PtInRect(hit, closebox) then begin
                            myWindow := wInGoAway;
                            exit;
                        end;
                    if hasZoom in theWindow^.goAwayFlag then
                        if PtInRect(hit, zoombox) then begin
                            myWindow := wInZoom;
                            exit;
                        end;
                    if hasSendToBack in theWindow^.goAwayFlag then
                        if PtInRect(hit, backbox) then begin
                            myWindow := wInSendToBack;
                            exit;
                        end;
                    if hasGrow in theWindow^.goAwayFlag then
                        if PtInRect(hit, growbox) then begin
                            myWindow := wInGrow;
                            exit;
                        end;
                end;
                if PtInRect(hit, dragbox) then begin
                    myWindow := wInDrag;
                    exit;
                end;
                if PtInRect(hit, scratch) then begin
                    myWindow := wInContent;
                    exit;
                end;
              end;

              iconWProc: begin
                scratch.top -= 11;
                zoombox := scratch;
                zoombox.left := zoombox.right - 10;
                zoombox.bottom := zoombox.top + 11;
                if theWindow^.hilited then // Dinguses only when in front.
                    if hasZoom in theWindow^.goAwayFlag then
                        if PtInRect(hit, zoombox) then begin
                            myWindow := wInZoom;
                            exit;
                        end;
                if PtInRect(hit, scratch) then begin
                    myWindow := wInDrag;
                    exit;
                end;
              end;
            end;
        end;
        MyWindow := wNoHit;
        exit;
      end;

      wCalcRgns: begin          // Calculate window regions.
        arect := NewRgn;
        case theWindow^.windowType of
          dBoxProc: begin
            scratch2 := scratch; // Make a copy
            InsetRect(scratch2, -5, -5);
            RectRgn(theWindow^.strucRgn, scratch2);
          end;

          altDBoxProc: begin
            scratch2 := scratch; // Make a copy, do body.
            InsetRect(scratch2, -1, -1);
            RectRgn(theWindow^.strucRgn, scratch2); // Body

            scratch2 := scratch; // Make a copy, do right side strip.
            scratch2.top += 2;
            inc(scratch2.bottom);
            scratch2.left := scratch2.right + 1;
            scratch2.right += 3;
            RectRgn(arect, scratch2); // Right side.
            UnionRgn(arect, theWindow^.strucRgn, theWindow^.strucRgn);

            scratch2 := scratch; // Make a copy, do bottom strip.
            scratch2.top := scratch2.bottom + 1;
            scratch2.left += 2;
            scratch2.right += 3;
            scratch2.bottom += 3;
            RectRgn(arect, scratch2); // Bottom side.
            UnionRgn(arect, theWindow^.strucRgn, theWindow^.strucRgn);
          end;

          plainDBox: begin
            scratch2 := scratch; // Make a copy, do body.
            InsetRect(scratch2, -1, -1);
            RectRgn(theWindow^.strucRgn, scratch2);
          end;

          documentProc: begin
            scratch2 := scratch; // Make a copy, do body.
            InsetRect(scratch2, -1, -1);
            scratch2.top -= titleheit; // For title bar.
            RectRgn(theWindow^.strucRgn, scratch2); // Body
            scratch2 := scratch; // Make a copy, do right side strip.
            scratch2.top -= titleheit;
            inc(scratch2.bottom);
            scratch2.left := scratch2.right + 1;
            scratch2.right += 2;
            RectRgn(arect, scratch2); // Right side.
            UnionRgn(arect, theWindow^.strucRgn, theWindow^.strucRgn);

            scratch2 := scratch; // Make a copy, do bottom strip.
            scratch2.top := scratch2.bottom + 1;
            scratch2.left += 1;
            scratch2.right += 2;
            scratch2.bottom += 2;
            RectRgn(arect, scratch2); // Bottom side.
            UnionRgn(arect, theWindow^.strucRgn, theWindow^.strucRgn);
          end;

          noBordDBox: begin
            RectRgn(theWindow^.strucRgn, scratch);
          end;

          iconWProc: begin
            scratch2 := scratch; // Make a copy, do body.
            scratch2.top -= 10;
            InsetRect(scratch2, -1, -1);
            RectRgn(theWindow^.strucRgn, scratch2);
          end;

        end;
        RectRgn(theWindow^.contRgn, scratch);
        DisposeRgn(arect);
      end;

      wNew: begin
      end;

      wDispose: begin
      end;

      wGrow: begin
        if theWindow^.visible then begin
            case theWindow^.windowType of
              dBoxProc,
              altDBoxProc,
              plainDBox,
              noBordDBox,
              iconWProc: begin
                MyWindow := 0;
                exit;
              end;

              documentProc: begin
                scratch := theWindow^.strucRgn^^.rgnBBox;
                scratch.right += Point(DWord(param)).h;
                scratch.bottom += Point(DWord(param)).v;
                if (scratch.right < scratch.left + 60) or
                    (scratch.bottom < scratch.top + 40) then begin
                    MyWindow := 0; // Don't let it (window size) get too small!
                    exit;
                end;
                FrameRect(scratch);
                MoveTo(scratch.left + 1, scratch.top + titleheit);
                LineTo(scratch.right - 1, scratch.top + titleheit);
                if hasGrow in theWindow^.goAwayFlag then begin
                    wid := scratch.right - 18;
                    MoveTo(wid, scratch.top + titleheit + 1);
                    LineTo(wid, scratch.bottom - 1);
                    wid := scratch.bottom - 18;
                    MoveTo(scratch.left + 1, wid);
                    LineTo(scratch.right - 1, wid);
                end;
                MyWindow := 1;
                exit;
              end;
            end;
        end;
      end;

      wDrawGIcon: begin
        if theWindow^.visible then begin
            case theWindow^.windowType of
              dBoxProc,
              altDBoxProc,
              plainDBox,
              noBordDBox,
              iconWProc: begin
              end;

              documentProc: begin
                if hasGrow in theWindow^.goAwayFlag then begin // Grow zone.
                    calcwrects(scratch, closebox, zoombox, backbox,
                               growbox, dragbox);
                    OffsetRect(growbox, 1, 1); // Move over the edge.
                    FrameRect(growbox);
                    InsetRect(growbox, 1, 1);
                    EraseRect(growbox);
                    InsetRect(growbox, -1, -1);
                    if theWindow^.hilited then begin // Draw icon?.
                        growbox.bottom := growbox.top + 8;
                        growbox.right := growbox.left + 8;
                        FrameRect(growbox);
                        growbox.bottom += 4;
                        growbox.right += 4;
                        FrameRect(growbox);
                    end;
                end;
              end;
            end;
        end;
      end;
    end;
    MyWindow := 0;
end;

//
// InitWindows initializes the Window Manager.  It creates the Window Manager
// port; you can get a pointer to this port with the GetWMgrPort procedure.
// InitWindows draws the desktop.  Call this routine once before all other
// Window Manager routines.
//
procedure InitWindows;
var
   savePort : GrafPtr;
begin
   GetPort(savePort);           // Because of OpenPort.
   WindowList := nil;           // Init linked list of windows.
   OpenPort(@WMgrPort);         // Create the window mgr grafport.
   if WMgrPort.visRgn = nil then begin
      WMgrPort.visRgn := NewRgn;
      RectRgn(WMgrPort.visRgn, WMgrPort.portRect);
   end;
   if WMgrPort.clipRgn = nil then begin
      WMgrPort.clipRgn := NewRgn;
      RectRgn(WMgrPort.clipRgn, WMgrPort.portRect);
   end;
   DeskPattern := gray;         // Desktop pattern.
   GrayRgn := NewRgn;           // Desktop region.
   RectRgn(GrayRgn, WMgrPort.portRect);
   SetClip(GrayRgn);
   FillRect(WMgrPort.portRect, DeskPattern);    // Paint the desktop.
   SetPort(saveport);
   SaveUpdate := True;
   PaintWhite := True;
   menubar := 0;
end;

//
// EndWindows releases the resources used by the Window Manager.  It should
// only be called once when it is desired to terminate the Window Manager.
//
procedure EndWindows;
var
   savePort : GrafPtr;
begin
   WindowList := nil;
   GetPort(savePort);
   SetPort(@WMgrPort);
   RectRgn(GrayRgn, WMgrPort.portRect);
   SetClip(GrayRgn);
   EraseRect(WMgrPort.portRect); // Erase the desktop.
   DisposeRgn(GrayRgn);
   ClosePort(@WMgrPort);
   SetPort(savePort);
end;

//
// RsrvMenuBar reserves/unreserves the menu bar area.  Call it after
// InitWindows.  A zero argument unreserves the menu bar area.
//
procedure RsrvMenuBar(height : smallInt);
var
   scratch : Rect;
   savePort : GrafPtr;
begin
   GetPort(savePort);
   SetPort(@WMgrPort);
   scratch := WMgrPort.portRect;
   if height >= 0 then          // Reserving menu bar.
      menubar := height;
   scratch.top := menubar;
   if height <= 0 then          // Reserving 'status' line area (Chinese).
      scratch.bottom += height;
   RectRgn(GrayRgn, scratch);   // Desktop region.
   SetClip(GrayRgn);
   SetPort(saveport);
end;

//
// GetWMgrPort returns in wPort a pointer to the Window Manager port.
//
// WARNING:  Do not change any regions of the Window Manager port, or
//           overlapping windows may not be handled properly.
//
procedure GetWMgrPort(var wPort : GrafPtr);
begin
   wPort := @WMgrPort;
end;

procedure unhigh_n_deact(theWindow : WindowPtr);
begin
   HiliteWindow(theWindow, False); // Unhighlight the window.
end;

procedure insert_win(theWindow, behind : WindowPeek);
var
   p : WindowPeek;
begin
   p := WindowList;             // Search out home for us in window list.
   if (behind = WindowPeek(-1)) or (p = nil) then begin // Window goes in front.
      p := WindowPeek(FrontWindow); // Deactivate the last front
      if theWindow^.visible then begin
         if p <> nil then       // window (if any).
            unhigh_n_deact(WindowPtr(p));
         theWindow^.hilited := True;
      end;
      theWindow^.nextWindow := WindowList;
      WindowList := theWindow;
   end else begin               // Window goes behind something.
      while p^.nextWindow <> nil do begin // Find which one it's to go behind.
         if p = behind then
            break;              // We've found where to put it.
         p := p^.nextWindow;    // Not yet we ain't, go around again.
      end;
      theWindow^.nextWindow := p^.nextWindow; // Insert it in list.
      p^.nextWindow := theWindow;
   end;
end;


procedure newregions(theWindow : WindowPeek); // Calculate new window regions.
begin
   if theWindow^.visible then
      MyWindow(0, theWindow, wCalcRgns, 0)
   else begin                   // Invisible, no regions needed.
      SetEmptyRgn(theWindow^.port.visRgn); // No more drawable area.
      SetEmptyRgn(theWindow^.strucRgn);
      SetEmptyRgn(theWindow^.contRgn);
      if theWindow^.saveVisRgn <> nil then
         SetEmptyRgn(theWindow^.saveVisRgn);
   end;
end;

//
// Draws a window's frame.
//
procedure drawframe(theWindow : WindowPeek);
begin
   MyWindow(0, theWindow, wDraw, 0);
end;

//
// NewWindow creates a window as specified by its parameters, adds it to the
// window list, and returns a WindowPtr to the new window.  It allocates space
// for the window record itself, for the structure and content regions of the
// window and asks the window definition function to calculate those regions.
//
// BoundsRect, a rectangle given in global coordinates, determines the window's
// size and location, and becomes the portRect of the window's grafPort; note,
// however, that the portRect is in local coordinates.  NewWindow sets the top
// left corner of the portRect to (0, 0).  For the standard types of windows,
// the boundsRect defines the content region of the window.
//
// Note:  The bit map, pen pattern, and other characteristics of the window's
//        grafPort are the same as the default values set by the OpenPort
//        procedure in clunkdraw.  (NewWindow acturally calls OpenPort to
//        initialize the window's grafPort.)  Note, however, that the
//        coordinates of the grafPort's portBits.bounds and visRgn are changed
//        along with its portRect.
//
// The title parameter is the window's title.  If the title of a window is
// longer than will fit in the space allotted for it it will be truncated in
// some manner.  This assumes the window's type is such that a title is
// displayed at all.
//
// If the visible parameter is TRUE, NewWindow draws the window.  First it
// calls the window definition function to draw the window frame.  Then it
// generates an update event for the entire window contents.  You would
// normally only create an invisible window if its default characteristics
// were not suitable and needed to be changed before display (in particular
// the background pattern).
//
// ProcID is the window definition ID, which leads to the window definition
// function for this type of window.
//
// The behind parameter determines the window's plane.  The new window is
// inserted in back of the window pointed to by this parameter.  To put the
// new window behind all other windows use behind=NIL.  To place it in front
// of all other windows, use behind=WindowPtr(-1); in this case, NewWindow
// will unhighlight the previously active window, highlight the window being
// created, and generate appropriate activate events.
//
function NewWindow(boundsRect : Rect;   // Bounding rectangle (global coords)
                   title : string;      // Window's title.
                   visible : Boolean;   // TRUE if window isn't totally inactive.
                   procID : smallInt;   // Window type.
                   behind : WindowPtr;  // Window to put this one behind,
                                        // 0 == put in back, -1 == put in front.
                   goAwayFlag : GoAway; // Window option flags.
                   refCon : PtrUInt     // For use by application.
        ) : WindowPtr;
var
   theWindow : WindowPeek;
   savePort : GrafPtr;
begin
   GetPort(savePort);
   New(theWindow);
   NewWindow := WindowPtr(theWindow);
   if theWindow = nil then              // (Failed create?)
      exit;
   OpenPort(WindowPtr(theWindow));      // Make a new GrafPort.
   if WindowPtr(theWindow)^.visRgn = nil then  // We're using the windowing
      WindowPtr(theWindow)^.visRgn := NewRgn;  // library, we will actually need
   if WindowPtr(theWindow)^.clipRgn = nil then begin // these 2 regions.
      WindowPtr(theWindow)^.clipRgn := NewRgn; // Make them.
                                // This setting of the clipRgn is something
                                // of a hack.  We use 30000 so that the
                                // offsetting done later by the Window Mgr
                                // won't cause clipping.  In obscure cases
                                // of large, off-screen bitmaps this could
                                // happen anyway.  Tough shit.
      SetRectRgn(WindowPtr(theWindow)^.clipRgn, -30000, -30000, 30000, 30000);
   end;
   MovePortTo(boundsRect.left, boundsRect.top);
   PortSize(boundsRect.right - boundsRect.left,
            boundsRect.bottom - boundsRect.top);
   SetPort(@WMgrPort);          // Work in window manager's port.
   theWindow^.windowKind := 8;  // Application window.
   theWindow^.visible := visible;
   theWindow^.hilited := False;
   if procID = noGrowDocProc then begin
      goAwayFlag := goAwayFlag - [hasGrow];
      procID := documentProc;
   end;
   theWindow^.windowType := procID;
   theWindow^.goAwayFlag := goAwayFlag;
   theWindow^.spareFlag := False;
   theWindow^.strucRgn := NewRgn;
   theWindow^.contRgn := NewRgn;
   theWindow^.updateRgn := NewRgn;
   theWindow^.saveVisRgn := nil;
///   TextSize(0);
///   TextFont(0);
   theWindow^.titleWidth := StringWidth(title);
   theWindow^.refCon := refCon;
   theWindow^.titleHandle := title;
   MyWindow(0, theWindow, wNew, 0); // Extra initialization.

   newregions(theWindow); // Calculate window's new regions (unclipped yet).
   insert_win(theWindow, WindowPeek(behind)); // Put it in window list.

   SetPort(@theWindow^.port);   // Work in window's port.
   SetOrigin(0, 0);             // Default coordinate system.
   if visible then begin
      PaintOne(theWindow, theWindow^.strucRgn);
      CalcVisBehind(theWindow, theWindow^.strucRgn); // Fix up behind it. */
   end;
   SetPort(saveport);
end;

procedure unlink_win(theWindow : WindowPtr); // Remove a window from window list.
var
   p : WindowPeek;
begin
   p := WindowList;             // Search out this window in window list.
   if WindowPtr(p) = theWindow then // First window in list?
      WindowList := p^.nextWindow
   else begin
      while p <> nil do begin
         if WindowPtr(p^.nextWindow) = theWindow then
            break;
         p := p^.nextWindow;
      end;
      p^.nextWindow := p^.nextWindow^.nextWindow; // Delete link.
   end;
end;


//
// DisposeWindow removes the given window from the screen and deletes it from
// the window list.  It releases the memory occupied by all data structures
// associated with the window, and the memory occupied by the window record
// itself.  Call this procedure when you're done with a window.
//
// Any update events for the window are discarded.  If the window was the
// frontmost window and there was another window behind it, the latter window
// is highlighted and an appropriate activate event is generated.
//
procedure DisposeWindow(theWindow : WindowPtr);
var
   p : WindowPeek;
   savePort : GrafPtr;
begin
   GetPort(savePort);
   SetPort(theWindow);
   if thePort = saveport then
      savePort := nil;
   SetPort(@WMgrPort);          // Work in window manager's port.
   unlink_win(theWindow);       // Remove window from list.
   PaintBehind(WindowPeek(theWindow)^.nextWindow, // Re-do the windows
               WindowPeek(theWindow)^.strucRgn);  // behind us.
   CalcVisBehind(WindowPeek(theWindow)^.nextWindow,
                 WindowPeek(theWindow)^.strucRgn);
   DisposeRgn(WindowPeek(theWindow)^.strucRgn); // Dispose of regions.
   DisposeRgn(WindowPeek(theWindow)^.contRgn);
   DisposeRgn(WindowPeek(theWindow)^.updateRgn);
   if WindowPeek(theWindow)^.saveVisRgn <> nil then
      DisposeRgn(WindowPeek(theWindow)^.saveVisRgn);
//   DisposeHandle(WindowPeek(theWindow)^.titleHandle); // Kill title.
   ClosePort(theWindow);        // Close its grafport.
   Dispose(theWindow);          // Free its space.
   p := WindowPeek(FrontWindow);
   if p <> nil then             // If the front died, highlight
      HiliteWindow(WindowPtr(p), True); // the next in the heirarchy.
   SetPort(savePort);
end;

procedure doSelect(theWindow : WindowPtr);
var
   p : WindowPtr;
begin
   p := FrontWindow;
   BringToFront(theWindow);
   if p <> nil then
      if p <> theWindow then    // Former front not us?
         unhigh_n_deact(p);     // Deactivate it and activate us fer sure.
   HiliteWindow(theWindow, True); // In any event highlight ourself.
end;

//
// SetWTitle sets theWindow's title to the given string, performing any
// necessary redrawing of the window frame.
//
procedure SetWTitle(theWindow : WindowPtr; title : string);
var
   scratch : RgnHandle;
   savePort : GrafPtr;
begin
   GetPort(savePort);
   SetPort(@WMgrPort);          // Work in window manager's port.
   scratch := NewRgn;
   CopyRgn(WindowPeek(theWindow)^.strucRgn, scratch);
//   DisposeHandle(WindowPeek(theWindow)^.titleHandle);
   WindowPeek(theWindow)^.titleHandle := title;
///   TextSize(0);
///   TextFont(0);
   WindowPeek(theWindow)^.titleWidth := StringWidth(title);
   newregions(WindowPeek(theWindow));   // Frame area changed?
   if EqualRgn(scratch, WindowPeek(theWindow)^.strucRgn) then begin
      DiffRgn(scratch, WindowPeek(theWindow)^.contRgn, scratch);
      PaintOne(WindowPeek(theWindow), scratch); // No, only redraw it.
   end else begin               // Yes, redo all behind windows too.
      UnionRgn(WindowPeek(theWindow)^.strucRgn, scratch, scratch);
      DiffRgn(scratch, WindowPeek(theWindow)^.contRgn, scratch);
      PaintBehind(WindowPeek(theWindow), scratch);
      CalcVisBehind(WindowPeek(theWindow), scratch);
   end;
   DisposeRgn(scratch);
   SetPort(savePort);
end;

//
// GetWTitle returns theWindow's title into the given string.
//
procedure GetWTitle(theWindow : WindowPtr; var title : string);
begin
   title := WindowPeek(theWindow)^.titleHandle;
end;

//
// SelectWindow makes theWindow the active window as follows:  It unhighlights
// the previously active window, brings theWindow in front of all other
// windows, highlights theWindow, and generates the appropriate activate
// events.  Call this procedure if there's a mouse-down event in the content
// region of an inactive window.
//
procedure SelectWindow(theWindow : WindowPtr);
begin
   DragFlag := False;           // Not moving the windows around.
   doSelect(theWindow);
end;

//
// HideWindow makes theWindow invisible.  If theWindow is the frontmost window
// and there's a window behind it, HideWindow also unhighlights theWindow,
// brings the window behind it to the front, highlights that window, and
// generates appropriate activate events.  If theWindow is already invisible,
// HideWindow has no effect.
//
procedure HideWindow(theWindow : WindowPtr);
var
   p : WindowPtr;
begin
   p := FrontWindow;            // Find out the frontmost visible window.
   ShowHide(theWindow, False);  // Hide the given window.
   if theWindow = p then begin  // Hiding the frontmost?
      unhigh_n_deact(theWindow); // Unhighlight it.
      p := FrontWindow;
      if p <> nil then          // And activate the new frontmost window.
         SelectWindow(p);
   end;
end;

//
// ShowWindow makes theWindow visible.  It does not change the front-to-back
// ordering of the windows.  Remember that if you previously hid the frontmost
// window with HideWindow, HideWindow will have brought the window behind it
// to the front; so if you then do a ShowWindow of the window you hid, it will
// no longer be frontmost.  If theWindow is already visible, ShowWindow has
// no effect.
//
// Note:  Although it's inadvisable, you can create a situation where the
//        frontmost window is invisible.  If you do a ShowWindow of such a
//        window, it will highlight the window if it's not already highlighted
//        and will generate an activate event to force this window from
//        inactive to active.
//
procedure ShowWindow(theWindow : WindowPtr);
var
   p : WindowPeek;
begin
   p := WindowList;
   if (p = WindowPeek(theWindow)) and not p^.visible then
      SelectWindow(theWindow);
   ShowHide(theWindow, True);
end;

//
// If showFlag is TRUE, ShowHide makes theWindow visible if it's not already
// visible and has no effect if it's already visible.  If showFlag is FALSE,
// ShowHide makes theWindow invisible if it's not already invisible and has no
// effect if it is already invisible.  Unlike HideWindow and ShowWindow,
// ShowHide never changes the highlighting or front-to-back ordering of windows
// or generates activate events.
//
procedure ShowHide(theWindow : WindowPtr; showFlag : Boolean);
var
   savePort : GrafPtr;
begin
   if WindowPeek(theWindow)^.visible <> showFlag then begin
      GetPort(savePort);
      SetPort(@WMgrPort);       // Work in window manager's port.
      SaveOld(WindowPeek(theWindow));
      WindowPeek(theWindow)^.visible := showFlag;
      newregions(WindowPeek(theWindow));
      DrawNew(WindowPeek(theWindow), True);
      SetPort(saveport);
   end;
end;

//
// If hilite is TRUE, this procedure highlights theWindow if it's not already
// highlighted and has no effect if it is already highlighted.  If hilite is
// FALSE, HiliteWindow unhighlights theWindow if it is highlighted and has no
// effect if it's not highlighted.  The exact way a window is highlighted
// depends on the window definition function (the window type).
//
// Normally you won't have to call this procedure, since you should call
// SelectWindow to make a window active, and SelectWindow takes care of the
// necessary highlighting changes.  Highlighting a window that isn't the active
// window is contrary to good user interface design practices.
//
procedure HiliteWindow(theWindow : WindowPtr; hilite : Boolean);
var
   savePort : GrafPtr;
begin
   if hilite <> WindowPeek(theWindow)^.hilited then begin
      WindowPeek(theWindow)^.hilited := hilite;
      GetPort(savePort);
      SetPort(@WMgrPort);       // Work in window manager's port.
                // Clip drawing to the structure region of the window.
//      SetClip(WindowPeek(theWindow)^.strucRgn); // Deleted for speed.
// ClipRect(WMgrPort.portRect); // For speed, we draw unclipped into the
                                // Window Manager's port if we can.  This
                                // is possible because we trust the
                                // MyWindow function.
      ClipRect(WindowPeek(theWindow)^.strucRgn^^.rgnBBox); // "

                // Don't draw over parts of windows that obscure us.
      ClipAbove(WindowPeek(theWindow));
                // Redraw the window frame with appropriate highlighting.
      drawframe(WindowPeek(theWindow));
      SetPort(savePort);
   end;
end;

//
// BringToFront brings theWindow to the front of all other windows and redraws
// the window as necessary.  Normally you won't have to call this procedure,
// since you should call SelectWindow to make a window active, and SelectWindow
// takes care of bringing the window to the front.  If you do call
// BringToFront, however, remember to call HiliteWindow to make the necessary
// highlighting changes.
//
procedure BringToFront(theWindow : WindowPtr);
var
   p : WindowPeek;
   savePort : GrafPtr;
   scratch, r : RgnHandle;
   dx, dy : integer;
begin
   DragFlag := False;           // Not moving the windows around.
   p := WindowList;
    if p <> WindowPeek(theWindow) then begin // Already frontmost?
       GetPort(savePort);       // No, gotta go to work then.
       SetPort(@WMgrPort);      // Work in window manager's port.
       scratch := NewRgn;

       { Subtract the structure region of our window from the visible parts
         of all windows that 'were' in front of it.   While we're at it,
         also accumulate a region that's the union of all windows' structure
         regions that were in front of us.  That will be used by PaintOne
         to figure out how much of our window has been exposed. }

       while (p <> nil) and (p <> WindowPeek(theWindow)) do begin
          if p^.visible then begin
             if p^.port.device = 1 then begin // Handle e-paper rotated coords.
                dx := p^.port.portBits.bounds.top;
                dy := p^.port.portBits.bounds.left;
             end else begin
                dx := p^.port.portBits.bounds.left;
                dy := p^.port.portBits.bounds.top;
             end;
             r := p^.port.visRgn;
             OffsetRgn(r, -dx, -dy);
             DiffRgn(r, WindowPeek(theWindow)^.strucRgn, r);
             OffsetRgn(r, dx, dy);
             UnionRgn(p^.strucRgn, scratch, scratch);
             r := p^.saveVisRgn;
             if r <> nil then begin // Window's being updated?
                OffsetRgn(r, -dx, -dy);
                DiffRgn(r, WindowPeek(theWindow)^.strucRgn, r);
                OffsetRgn(r, dx, dy);
             end;
          end;
          p := p^.nextWindow;
       end;
       unlink_win(theWindow);   // Rearrange window list in new order.
       WindowPeek(theWindow)^.nextWindow := WindowList;
       WindowList := WindowPeek(theWindow);

       if not DragFlag then begin
          SectRgn(WindowPeek(theWindow)^.strucRgn, scratch, scratch);
          PaintOne(WindowPeek(theWindow), scratch);
       end;
       CalcVis(WindowPeek(theWindow));
       DisposeRgn(scratch);
       SetPort(savePort);
    end;
end;


procedure substructrgns(rgn : RgnHandle; window : WindowPeek);
var
   p : WindowPeek;
begin
   p := WindowList;
   while p <> window do begin
      if p^.visible then
         DiffRgn(rgn, p^.strucRgn, rgn);
      p := p^.nextWindow;
   end;
end;


//
// SendBehind sends theWindow behind behindWindow, redrawing any exposed
// windows.  If behindWindow is NULL, it sends theWindow behind all other
// windows.  If theWindow is the active window, it unhighlights theWindow,
// highlights the new active window, and generates the appropriate activate
// events.
//
// WARNING:  Do not use SendBehind to deactivate a previously active window.
//           Calling SelectWindow to make a window active takes care of
//           deactivating the previously active window.
//
// Note:  Unlike Apple's routine, this one properly handles the case when you
//        send a window farther forward than it was.
//
procedure SendBehind(theWindow, behindWindow : WindowPtr);
var
   saveport : GrafPtr;
   p : WindowPeek;
   scratch, sr : RgnHandle;
   movingUp : Boolean;
begin
   if (behindWindow <> nil) and // No effective change?  We're done.
      (WindowPeek(behindWindow)^.nextWindow = WindowPeek(theWindow)) then
      exit;
   GetPort(savePort);           // Work for now in Window Manager's port.
   SetPort(@WMgrPort);
   movingUp := False;           // By default, window's not moving forward.
   p := WindowPeek(behindWindow); // Find out if window's moving forward.
   if p <> nil then begin       // Search behind 'behindwindow' for it.
      p := p^.nextWindow;
      while p <> nil do begin
         if p = WindowPeek(theWindow) then begin // It's back there,
            movingup := True;   // therefore it's moving forward.
            break;
         end;
         p := p^.nextWindow;
      end;
   end;
   p := WindowPeek(theWindow);
   if WindowPeek(FrontWindow) = p then begin // Moving front window?
      p := p^.nextWindow;
      while p <> nil do begin // Yes, select the next visible window.
         if p^.visible then begin
            SelectWindow(WindowPtr(p));
            break;
         end;
         p := p^.nextWindow;
      end;
   end;
   p := WindowPeek(theWindow)^.nextWindow;
   unlink_win(theWindow);       // Remove theWindow from the window list.
   insert_win(WindowPeek(theWindow),
              WindowPeek(behindWindow)); // Put it in new place.
   scratch := NewRgn;           // Calculate how many windows to update...
   sr := WindowPeek(theWindow)^.strucRgn;
   CopyRgn(sr, scratch);        // Find intersection
   substructrgns(scratch, WindowPeek(theWindow)); // of window's region &
   DiffRgn(sr, scratch, scratch); // others'.
   PaintBehind(p, scratch);     // Now fix all windows that used to be
   CalcVisBehind(p, sr);        // behind us who
   DisposeRgn(scratch);         // intersect this spot.
   if movingUp then begin
      PaintOne(WindowPeek(theWindow), sr);
      CalcVis(WindowPeek(theWindow));
   end;
   SetPort(savePort);
end;


//
// FrontWindow returns a pointer to the first visible window in the window
// list (that is, the active window).  If there are no visible windows, it
// returns NULL.
//
function FrontWindow : WindowPtr;
var
   p : WindowPeek;
begin
   p := WindowList;             // Search out visible window in window list.
   while (p <> nil) and not p^.visible do
      p := p^.nextWindow;
   FrontWindow := WindowPtr(p);
end;

//
// Call DrawGrowIcon in response to an update or activate event involving a
// window that contains a size box in its content region.  If theWindow is
// active, DrawGrowIcon draws the size box; otherwise, it draws whatever is
// appropriate to show that the window temporarily cannot be sized.  The exact
// appearance and location of what's drawn depend on the window definition
// function.  For an active document window, DrawGrowIcon draws the size box
// icon in the bottom right corner of the portRect of the window's grafPort,
// along with the lines delimiting the size box itself.  It doesn't do anything
// with the scroll bar areas.  For an inactive document window DrawGrowIcon
// merely erases the size box icon.  The lines are still drawn around the icon.
//
procedure DrawGrowIcon(theWindow : WindowPtr);
var
   savePort : GrafPtr;
begin
   GetPort(savePort);           // Work for now in Window Manager's port.
   SetPort(@WMgrPort);
   ClipRect(WindowPeek(theWindow)^.strucRgn^^.rgnBBox); // Minimal
   ClipAbove(WindowPeek(theWindow)); // clipping because we're trusting.
   MyWindow(0, WindowPeek(theWindow), wDrawGIcon, 0);
   SetPort(savePort);
end;

//
// When a mouse-down event occurs, the system should call FindWindow with
// thePt equal to the point where the mouse button was pressed (in global
// coordinates).  FindWindow tells which part of which window, if any, the
// mouse button was pressed in.  If it was pressed in a window, the whichWindow
// parameter is set to the window pointer; otherwise, it's set to NULL.  The
// (short) integer returned by FindWindow is one of the following predefined
// constants:
//
//      inDesk = 0;
//      inMenuBar = 1;
//      inContent = 3;
//      inDrag = 4;
//      inGrow = 5;
//      inGoAway = 6;
//      inZoom = 7;
//      inSendtoBack = 8;
//
function FindWindow(thePt : Point; var whichWindow : WindowPtr) : smallInt;
var
   p : WindowPeek;
   savePort : GrafPtr;
begin
   whichWindow := nil;
   if GrayRgn^^.rgnBBox.top > thePt.v then // Check for menu bar hit.
      exit(inMenuBar);
   GetPort(savePort);           // Work for now in Window Manager's port.
   SetPort(@WMgrPort);
   FindWindow := inDesk;
   p := WindowList;             // Search out visible window in window list.
   while p <> nil do begin
      if p^.visible then begin
         FindWindow := MyWindow(0, p, wHit, longInt(thePt));
         if FindWindow <> 0 then begin
            whichWindow := WindowPtr(p);
            FindWindow += 2;    // Convert to proper returned values.
            break;
         end;
      end;
      p := p^.nextWindow;
   end;
   SetPort(savePort);
end;


procedure
move_Wcoords(theWindow : WindowPtr;       // Which window to move.
             hGlobal, vGlobal : smallInt; // Where to move window coordinates to.
             scratch : RgnHandle);        // Also moved with window if present.
var
   dh, dv : smallInt;
begin
   if theWindow^.device = 1 then begin // Handle weird e-paper rotated coords.
      dv := vGlobal - theWindow^.portRect.top  + theWindow^.portBits.bounds.left;
      dh := hGlobal - theWindow^.portRect.left + theWindow^.portBits.bounds.top;
      OffsetRect(theWindow^.portBits.bounds, -dv, -dh);
   end else begin
      dv := vGlobal - theWindow^.portRect.top  + theWindow^.portBits.bounds.top;
      dh := hGlobal - theWindow^.portRect.left + theWindow^.portBits.bounds.left;
      OffsetRect(theWindow^.portBits.bounds, -dh, -dv);
   end;
   OffsetRgn(WindowPeek(theWindow)^.strucRgn, dh, dv);
   OffsetRgn(WindowPeek(theWindow)^.contRgn, dh, dv);
   OffsetRgn(WindowPeek(theWindow)^.updateRgn, dh, dv);
   if scratch <> nil then
      OffsetRgn(scratch, dh, dv);
end;

//
// Whenever there's a mouse-down in one of the title-bar controls of theWindow
// (as reported by FindWindow), the application should repeatedly call TrackBox
// with thePt equal to the current mouse point (in global coordinates).  The
// partCode parameter should contain the value returned by FindWindow. TrackBox
// will take care of highlighting the proper part of the control, and will
// return TRUE or FALSE depending upon whether thePt is within the appropriate
// part of the title bar.  The application should stop calling TrackBox when
// the mouse button is released, and if the last returned value was TRUE (from
// the event containing the mouse-up) it should call TrackBox once more with an
// 'impossible' value for thePt.  This will clear the highlighting.
//
function TrackBox(theWindow : WindowPtr; thePt : Point; partCode : integer) : Boolean;
var
   savePort : GrafPtr;
   ii : integer;
begin
   GetPort(savePort);
   SetPort(theWindow);
   ii := MyWindow(0,WindowPeek(theWindow), wHit, longInt(thePt)) + 2;
   SetPort(@WMgrPort);
   SetClip(GrayRgn);
   ClipAbove(WindowPeek(theWindow));
   TrackBox := ii = partCode;
   MyWindow(partCode, WindowPeek(theWindow), wDraw, longInt(TrackBox));
   SetPort(saveport);
end;


//
// MoveWindow moves theWindow to another part of the screen, without affecting
// its size or plane.  The top left corner of the portRect of the window's
// grafPort is moved to the screen point indicated by the global coordinates
// hGlobal and vGlobal.  The local coordinates of the top left coordinate
// remain the same.  If the makefront parameter is TRUE and theWindow isn't
// the active window, MoveWindow makes it the active window by calling
// SelectWindow(theWindow).
//
procedure MoveWindow(theWindow : WindowPtr; hGlobal, vGlobal : smallInt; makefront : Boolean);
var
   savePort : GrafPtr;
   scratch, scratch2, sr : RgnHandle;
   dh, dv : smallInt;
begin
   if theWindow^.device = 1 then begin // Handle weird e-paper rotated coords.
      dv := vGlobal - theWindow^.portRect.top  + theWindow^.portBits.bounds.left;
      dh := hGlobal - theWindow^.portRect.left + theWindow^.portBits.bounds.top;
   end else begin
      dv := vGlobal - theWindow^.portRect.top  + theWindow^.portBits.bounds.top;
      dh := hGlobal - theWindow^.portRect.left + theWindow^.portBits.bounds.left;
   end;
   if (dh = 0) and (dv = 0) then // No movement at all?
      exit;                     // Quit then.
   GetPort(savePort);           // Work for now in Window Manager's port.
   SetPort(@WMgrPort);
   SetClip(GrayRgn);
   ClipAbove(WindowPeek(theWindow));
   scratch := NewRgn;
   sr := WindowPeek(theWindow)^.strucRgn;
   SectRgn(sr, WMgrPort.clipRgn, scratch);
   scratch2 := NewRgn;          // Make a copy of window's structure region.
   CopyRgn(sr, scratch2);
   move_Wcoords(theWindow, hGlobal, vGlobal, scratch);
   if makefront then
      SetClip(GrayRgn);
   CopyBits(WMgrPort.portBits, WMgrPort.portBits, // Move window data.
            scratch2^^.rgnBBox, sr^^.rgnBBox, srcCopy, scratch);
   DiffRgn(sr, scratch, scratch);
   PaintBehind(WindowPeek(theWindow)^.nextWindow, scratch2);
   if makefront then begin      // If making window frontmost then do so.
      DragFlag := True;         // It moved too.
      doSelect(theWindow);
   end;
   PaintOne(WindowPeek(theWindow), scratch);
   UnionRgn(scratch2, sr, scratch2);
   CalcVisBehind(WindowPeek(theWindow), scratch2);
   DisposeRgn(scratch);         // Clean up.
   DisposeRgn(scratch2);
   SetPort(saveport);
end;

//
// Call DragWindow to move a dotted outline of theWindow around.  Point delta
// tells how far to move it.  When moving, call DragWindow once with the old
// position and then once with the new position.  Call it once more to remove
// the outline altogether.  It returns TRUE if the window is being dragged to
// an acceptable place on the screen.
//
function DragWindow(theWindow : WindowPtr; delta : Point) : Boolean;
var
   savePort : GrafPtr;
   scratch : Rect;
begin
   GetPort(savePort);           // Work for now in Window Manager's port.
   SetPort(@WMgrPort);
   SetClip(GrayRgn);
   ClipAbove(WindowPeek(theWindow));
   PenPat(gray);
   PenMode(notSrcXor);
   scratch := WindowPeek(theWindow)^.strucRgn^^.rgnBBox;
   OffsetRect(scratch, delta.h, delta.v);
   DragWindow := True;          // Limit window's possible travel.
   if (scratch.left > WMgrPort.portRect.right - 4) or
      (scratch.right < WMgrPort.portRect.left + 4) or
      (scratch.top > WMgrPort.portRect.bottom - 4) or
      (scratch.top < WMgrPort.portRect.top + menubar - 4) then
      DragWindow := False;
   if DragWindow then           // It's OK, draw the frame.
      FrameRect(scratch);
   PenNormal;
   SetPort(savePort);
end;


//
// Call GrowWindow to move a dotted resizing outline of theWindow around.
// Point delta tells how far to move it.  When sizing, call GrowWindow once
// with the old size and then once with the new size.  Call it once more to
// remove the outline altogether.  It returns TRUE if the window is being
// grown to an acceptable size.
//
function GrowWindow(theWindow : WindowPtr; delta : Point) : Boolean;
var
   savePort : GrafPtr;
begin
   GetPort(savePort);           // Work for now in Window Manager's port.
   SetPort(@WMgrPort);
   SetClip(GrayRgn);
   ClipAbove(WindowPeek(theWindow));
   PenPat(gray);
   PenMode(notSrcXor);
   GrowWindow := Boolean(MyWindow(0, WindowPeek(theWindow), wGrow, longInt(delta)));
   PenNormal;
   SetPort(savePort);
end;


procedure redo_wind(theWindow : WindowPtr; fUpdate : Boolean);
begin
   newregions(WindowPeek(theWindow));
//    SetClip(WindowPeek(theWindow)^.strucRgn); // Deleted for speed.
//ClipRect(WMgrPort.portRect);  // For speed, we draw unclipped into the
                                // Window Manager's port if we can.  This
                                // is possible because we trust the
                                // MyWindow function.
   ClipRect(WindowPeek(theWindow)^.strucRgn^^.rgnBBox); // "
   ClipAbove(WindowPeek(theWindow));
   drawframe(WindowPeek(theWindow));
   DrawNew(WindowPeek(theWindow), fUpdate);
end;

//
// SizeWindow enlarges or shrinks the portRect of theWindow's grafPort to the
// width and height specified by w and h, or does nothing if w and h are 0.
// The window's position on the screen does not change.  The new window frame
// is drawn; if the width of a document window changes, the title is again
// centered in the title bar, or is truncated if it no longer fits.  If
// fUpdate is TRUE, SizeWindow accumulates any newly created area of the
// content region into the update region; normally this is what you'll want.
// If you pass FALSE for fUpdate, you're responsible for the update region
// maintenence yourself.  For more information, see InvalRect and ValidRect.
//
procedure SizeWindow(theWindow : WindowPtr; w, h : smallInt; fUpdate : Boolean);
var
   savePort : GrafPtr;
begin
   if (w <> 0) or (h <> 0) then begin
      if h <> 0 then
         theWindow^.portRect.bottom := theWindow^.portRect.top + h;
      if w <> 0 then
         theWindow^.portRect.right := theWindow^.portRect.left + w;
      if WindowPeek(theWindow)^.visible then begin
         GetPort(savePort);     // Work for now in Window Manager's port.
         SetPort(@WMgrPort);
         SaveOld(WindowPeek(theWindow));
         redo_wind(theWindow, fUpdate);
         SetPort(saveport);
      end;
   end;
end;


//
// ZoomWindow provides (in a single call) the action of both a MoveWindow and
// a SizeWindow.  A new rectangle (in global coordinates) is provided for it
// to change the window's content region to.  If the front parameter is TRUE,
// the window will be brought to the front; otherwise, the window is left where
// it is.  (This means a window can be zoomed without necessarily becoming the
// active window.
//
procedure ZoomWindow(theWindow : WindowPtr; newRect : Rect; makeFront : Boolean);
var
   saveport : GrafPtr;
begin
   GetPort(savePort);           // Work for now in Window Manager's port.
   SetPort(@WMgrPort);
   newregions(WindowPeek(theWindow));
   SaveOld(WindowPeek(theWindow));
   move_Wcoords(theWindow, newRect.left, newRect.top, nil);
   theWindow^.portRect.bottom := theWindow^.portRect.top +
                                 (newRect.bottom - newRect.top);
   theWindow^.portRect.right := theWindow^.portRect.left +
                                (newRect.right - newRect.left);
   redo_wind(theWindow, True);
   if makeFront then begin      // If making window frontmost then do so.
      DragFlag := True;         // It moved too.
      doSelect(theWindow);
   end;
   SetPort(theWindow);          // For InvalRect.
   InvalRect(theWindow^.portRect);
   SetPort(saveport);
end;

//
// CalcVis calculates the visRgn of the given window by starting with its
// content region and subtracting the structure region of each window in
// front of it.
//
procedure CalcVis(theWindow : WindowPeek);
var
   vr : RgnHandle;
begin
    if theWindow <> nil then begin
       vr := theWindow^.port.visRgn;
       if theWindow^.visible then begin
          SectRgn(GrayRgn, theWindow^.contRgn, vr);
          substructrgns(vr, theWindow);
          if theWindow^.port.device = 1 then begin // Handle e-paper rotated coords.
             OffsetRgn(vr,      // Move to local coordinates.
                       theWindow^.port.portBits.bounds.top,
                       theWindow^.port.portBits.bounds.left);
          end else begin
             OffsetRgn(vr,      // Move to local coordinates.
                       theWindow^.port.portBits.bounds.left,
                       theWindow^.port.portBits.bounds.top);
          end;
       end else
          SetEmptyRgn(vr);
       if theWindow^.saveVisRgn <> nil then // Window being updated?
          CopyRgn(vr, theWindow^.saveVisRgn);
    end;
end;


//
// CalcVisBehind calculates the visRgns of startWindow and all windows
// behind startWindow that intersect clobberedRgn.  It should be called
// after PaintBehind.
//
procedure CalcVisBehind(startWindow : WindowPeek; clobberedRgn : RgnHandle);
var
   p : WindowPeek;
   scratch : Rect;
begin
   p := startWindow;
    if p <> nil then begin      // Quit if at first window or no windows.
        while p <> nil do begin // For each window from here to the back...
           CalcVis(p);
           p := p^.nextWindow;
           while p <> nil do begin // Go further back in window list.
              if p^.visible then begin // Skip invisibles
                 if SectRect(p^.contRgn^^.rgnBBox,
                     clobberedRgn^^.rgnBBox,
                     scratch) then begin // and out-of-bounds windows.
                        break;  // Cause a CalcVis on this window.
                     end;
              end;
              p := p^.nextWindow;
           end;
        end;
    end;
end;


//
// Internal function for ValidRect, InvalRect, ValidRgn, and InvalRgn.  It
// does all the work.
//
procedure dovalinval(therect : PRect; theregion : RgnHandle; doDiff : Boolean);
var
   tp : GrafPtr;
   scratch : RgnHandle;
begin
   tp := thePort;
   scratch := NewRgn;
    if theregion <> nil then    // Do both rectangles and regions as req'd.
       CopyRgn(theregion, scratch)
    else
       RectRgn(scratch, therect^);
   if tp^.device = 1 then begin // Handle weird e-paper rotated coords.
      OffsetRgn(scratch,        // Move to global coordinates
                -tp^.portBits.bounds.top, // (the same ones the
                -tp^.portBits.bounds.left); // updateRgn is in).
   end else begin
      OffsetRgn(scratch,        // Move to global coordinates
                -tp^.portBits.bounds.left, // (the same ones the
                -tp^.portBits.bounds.top); // updateRgn is in).
   end;
   SectRgn(scratch, WindowPeek(tp)^.contRgn, scratch);
   if doDiff then
      DiffRgn(WindowPeek(tp)^.updateRgn, scratch,
              WindowPeek(tp)^.updateRgn)
   else
      UnionRgn(WindowPeek(tp)^.updateRgn, scratch,
               WindowPeek(tp)^.updateRgn);
   DisposeRgn(scratch);
end;


//
// InvalRect accumulates the given rectangle into the update region of the
// window whose grafPort is the current port.  This tells the Window Manager
// that the rectangle has changed and must be updated.  The rectangle is given
// in local coordinates and is clipped to the window's content region.
//
// For example, suppose your application scrolls up text in a document window
// and wants to show new text added at the bottom of the window.  You can cause
// the added text to be redrawn by accumulating that area into the update
// region with InvalRect.
//
procedure InvalRect(badRect : Rect);
begin
   dovalinval(@badRect, nil, False);
end;

//
// InvalRgn is the same as InvalRect but for a region that has changed rather
// than a rectangle.
//
procedure InvalRgn(badRgn : RgnHandle);
begin
    dovalinval(nil, badRgn, False);
end;


//
// ValidRect removes goodRect from the update region of the window whose
// grafPort is the current port.  This tells the window manager that the
// application has already drawn the rectangle and to cancel any updates
// accumulated for that area.  The rectangle is clipped to the window's
// content region and is given in local coordinates.  Using ValidRect
// results in better performance and less redundant redrawing in the window.
//
procedure ValidRect(goodRect : Rect);
begin
   dovalinval(@goodRect, nil, True);
end;


//
// ValidRgn is the same as ValidRect but for a region that has been drawn
// rather than a rectangle.
//
procedure ValidRgn(goodRgn : RgnHandle);
begin
    dovalinval(nil, goodRgn, True);
end;


//
// Call BeginUpdate when an update event occurs for theWindow.  BeginUpdate
// replaces the visRgn of the window's grafPort with the intersection of the
// visRgn and the update region and then sets the window's update region to an
// empty region.  The original visRgn is saved away.  You would then usually
// (re)draw the entire content region, although it suffices to draw only the
// visRgn; in either case, only the parts of the window that require updating
// will actually be drawn on the screen.  Every call to BeginUpdate must be
// balanced by a call to EndUpdate.
//
// Unlike in Quickdraw, BeginUpdate and EndUpdate calls may be nested, but not
// if they refer to the same window (that case is protected against so that
// nothing truly evil happens if you do, but it certainly won't work right).
//
procedure BeginUpdate(theWindow : WindowPtr);
var
   scratch : RgnHandle;
begin
   // Calculate intersection of vis & update.
   if WindowPeek(theWindow)^.saveVisRgn = nil then begin // Dennis-proofing!
      if theWindow^.device = 1 then begin // Handle weird e-paper rotated coords.
         OffsetRgn(WindowPeek(theWindow)^.updateRgn, // Move to lcl coords
                   theWindow^.portBits.bounds.top,   // (same ones the
                   theWindow^.portBits.bounds.left); //  visRgn is in).
      end else begin
         OffsetRgn(WindowPeek(theWindow)^.updateRgn, // Move to lcl coords
                   theWindow^.portBits.bounds.left,  // (same ones the
                   theWindow^.portBits.bounds.top);  //  visRgn is in).
      end;
      scratch := WindowPeek(theWindow)^.updateRgn;
      WindowPeek(theWindow)^.updateRgn := NewRgn;
      SectRgn(theWindow^.visRgn, scratch, scratch); // Intersect them.
      WindowPeek(theWindow)^.saveVisRgn := theWindow^.visRgn;
      theWindow^.visRgn := scratch; // Change the visRgn to intersection
   end;                             // of the dirty area and the drawable area.
end;

//
// BeginUpdateRect is like BeginUpdate except that you tell it what rectangle
// you will be updating.  The rectangle is in the local coordinates of the
// window.  This call is used to compensate for pipeline delay in the graphics
// system.  You need only issue BeginUpdateRect if you use the update
// rectangle passed in the update event to limit what you redraw.  If you
// always redraw the whole window you need not use BeginUpdateRect, but may
// use BeginUpdate instead.
//
// The offending scenario:
//
//  1)  Window is made dirty -- update event is placed in the pipeline to be
//      passed to the application.  Update rectangle is for this original
//      dirty area.
//
//  2)  Application issues a ScrollRect (for example) that causes more of the
//      window to become dirty.
//
//  3)  Application receives the original update event, issues a BeginUpdate
//      and sends only the data that falls into the original update rectangle.
//
//  4)  Application issues an EndUpdate, which fully clears out the update
//      region.
//
// The end result:  A dirty window (from step 2) that is marked as clean.
//
procedure BeginUpdateRect(theWindow : WindowPtr; uprect : Rect);
var
   scratch, updating, notupdated : RgnHandle;
   dh, dv : smallInt;
begin
    // Calculate intersection of vis & update.
   if WindowPeek(theWindow)^.saveVisRgn = nil then begin // Dennis-proofing!
      updating := NewRgn;
      notupdated := NewRgn;
      RectRgn(updating, uprect);
      if theWindow^.device = 1 then begin // Handle weird e-paper rotated coords.
         dh := theWindow^.portBits.bounds.top;
         dv := theWindow^.portBits.bounds.left;
      end else begin
         dh := theWindow^.portBits.bounds.left;
         dv := theWindow^.portBits.bounds.top;
      end;
      OffsetRgn(updating, -dh, -dv); // Move to global coords.
      DiffRgn(WindowPeek(theWindow)^.updateRgn, updating, notupdated);
      DisposeRgn(updating);
      OffsetRgn(WindowPeek(theWindow)^.updateRgn, // Move to local
                dh, dv);        // coords (the same ones the visRgn is in).
      scratch := WindowPeek(theWindow)^.updateRgn;
      WindowPeek(theWindow)^.updateRgn := notupdated;
      SectRgn(theWindow^.visRgn, scratch, scratch); // Intersect them.
      WindowPeek(theWindow)^.saveVisRgn := theWindow^.visRgn;
      theWindow^.visRgn := scratch; // Change the visRgn to intersection
   end;                             // of the dirty area and the drawable area.
end;

//
// Call EndUpdate to restore the normal visRgn of theWindow's grafPort, which
// was changed by BeginUpdate as described above.
//
procedure EndUpdate(theWindow : WindowPtr);
begin
   if WindowPeek(theWindow)^.saveVisRgn <> nil then begin // Dennis-proofing!
      DisposeRgn(theWindow^.visRgn);
      theWindow^.visRgn := WindowPeek(theWindow)^.saveVisRgn;
      WindowPeek(theWindow)^.saveVisRgn := nil;
   end;
end;

//
// SetWRefCon sets theWindow's reference value to the given data.
//
procedure SetWRefCon(theWindow : WindowPtr; data : PtrUInt);
begin
   WindowPeek(theWindow)^.refCon := data;
end;


//
// GetWRefCon returns theWindow's current reference value.
//
function GetWRefCon(theWindow : WindowPtr) : PtrUInt;
begin
   GetWRefCon :=  WindowPeek(theWindow)^.refCon;
end;

//
// ChangeWindow sets theWindow's type to the given kind.  For best (any)
// results the window should be hidden at the time you issue this.
//
procedure ChangeWindow(theWindow : WindowPeek; newtype : smallInt);
begin
   theWindow^.windowType := newtype;
end;


//
// CheckUpdate is called by the Event Manager.  From the front to the back in
// the window list, it looks for a visible window that needs updating (that is,
// whose update region is not empty).  If it finds one it returns the WindowPtr
// for that window.  If it never finds such a window it returns 0.
//
// It must be called once with the theEvent argument set to 0 to initialize
// the chain, there will be a return value for this call as well.  Each call
// thereafter returns any succeeding dirty window.
//
var cup : WindowPeek = nil;
function CheckUpdate(window: WindowPtr) : WindowPtr;
begin
   if window = nil then
      cup := WindowList;
   while cup <> nil do begin
      if cup^.visible then begin
         if not EmptyRgn(cup^.updateRgn) then begin
            window := WindowPtr(cup);
            cup := cup^.nextWindow;
            exit(window);
         end;
      end;
      cup := cup^.nextWindow;
   end;
   CheckUpdate := WindowPtr(cup);
end;

procedure paintdesk;
begin
   ClipAbove(nil);              // Only paint the desktop.
   FillRect(WMgrPort.portRect, DeskPattern); // Fill it with pattern.
end;


//
// ClipAbove sets the clipRgn of the Window Manager port to be the desktop
// intersected with the current clipRgn, minus the structure regions of all
// the windows in front of the given window.
//
// ClipAbove gets the desktop region from the variable GrayRgn.
//
procedure ClipAbove(window : WindowPeek);
begin
   SectRgn(WMgrPort.clipRgn, GrayRgn, WMgrPort.clipRgn);
   substructrgns(WMgrPort.clipRgn, window);
end;


//
// SaveOld saves the given window's current structure region and content region
// for the DrawNew operation (see below).  It must be balanced by a subsequent
// call to DrawNew.  It is _not_ nestable.
//

var
   OldStrucRgn : RgnHandle = nil;
   OldContRgn  : RgnHandle = nil;

procedure SaveOld(window : WindowPeek);
begin
   OldStrucRgn := NewRgn;
   CopyRgn(window^.strucRgn, OldStrucRgn);
   OldContRgn := NewRgn;
   CopyRgn(window^.contRgn, OldContRgn);
end;


//
// If the update parameter is TRUE, DrawNew updates the area
//
//      (OldStructure XOR NewStructure) UNION (OldContent XOR NewContent)
//
// where OldStructure and OldContent are the structure and content regions
// saved by the SaveOld procedure, and NewStructure and NewContent are the
// current structure and content regions.  It erases the area and adds it to
// the window's update region.  If update is FALSE, it only erases the area.
//
procedure DrawNew(window : WindowPeek; update : Boolean);
begin
   SaveUpdate := update;
   if window^.visible then begin
      XorRgn(window^.strucRgn, OldStrucRgn, OldStrucRgn);
      XorRgn(window^.contRgn, OldContRgn, OldContRgn);
   end;
   UnionRgn(OldContRgn, OldStrucRgn, OldStrucRgn);
   DisposeRgn(OldContRgn);
   PaintOne(window, OldStrucRgn);
   SaveUpdate := True;
   PaintBehind(window^.nextWindow, OldStrucRgn);
   CalcVisBehind(window, OldStrucRgn);
   DisposeRgn(OldStrucRgn);
end;


//
// PaintBehind calls PaintOne for startWindow and all the windows behind
// startWindow, clipped to clobberedRgn.
//
// PaintBehind clears the variable PaintWhite before calling PaintOne, so
// clobberedRgn isn't erased.  (PaintWhite is re-set after the call to
// PaintOne.)
//
procedure PaintBehind(startWindow : WindowPeek; clobberedRgn : RgnHandle);
var
   savePort : GrafPtr;
   scratch : RgnHandle;
   p : WindowPeek;
begin
   GetPort(savePort);           // Work for now in Window Manager's port.
   SetPort(@WMgrPort);

   CopyRgn(clobberedRgn, WMgrPort.clipRgn); // Re-paint the newly exposed
   ClipAbove(startWindow);      // desktop (if any).
   scratch := NewRgn;
   CopyRgn(WMgrPort.clipRgn, scratch);
   paintdesk;

   DiffRgn(scratch, WMgrPort.clipRgn, scratch);
   ClipRect(WMgrPort.portRect);
   EraseRgn(scratch);           // Blot out (re)exposed areas.
   DisposeRgn(scratch);

   p := startWindow;
   while p <> nil do begin      // While there are windows to do...
      if p^.visible then begin  // (Re-draw the insides if the bkPat's not white
         PaintWhite := CompareByte(p^.port.bkPat, white, sizeof(Pattern)) <> 0;
         PaintOne(p, clobberedRgn); // Re-paint the visible ones while
      end;
      p := p^.nextWindow;       // moving rearwards through the list.
    end;
   PaintWhite := True;          // Normal state.
   SetPort(savePort);
end;


//
// PaintOne "paints" the given window, clipped to clobberedRgn and all
// windows above it:  It draws the window frame and, if some content is
// exposed, erases the exposed area (paints it with the background pattern)
// and adds it to the window's update region.  If the window parameter is NIL,
// the window is the desktop and so is painted with the desktop pattern.
//
// The variables SaveUpdate and PaintWhite are flags used by PaintOne.
// Normally both flags are set.  Clearing SaveUpdate prevents clobberedRgn
// from being added to the window's update region.  Clearing PaintWhite
// prevents clobberedRgn from being erased before being added to the update
// region (this is useful, for example, if the background of the window isn't
// the background pattern).  The Window Manager sets both flags periodically,
// so you should clear the appropriate flag just before each situation you
// wish it to apply to.
//
procedure PaintOne(window : WindowPeek; clobberedRgn : RgnHandle);
var
   savePort : GrafPtr;
   scratch : RgnHandle;
   clipr : Rect;
begin
   GetPort(savePort);
   SetPort(@WMgrPort);
   if window <> nil then begin  // Window or desktop?
      if window^.visible then begin // Window, is it visible?
         SectRgn(clobberedRgn, window^.strucRgn, WMgrPort.clipRgn);
         if not EmptyRgn(WMgrPort.clipRgn) then begin // If window intersects
            ClipAbove(window); // the clobbered region then do work.
            if not EmptyRgn(WMgrPort.clipRgn) then begin
               scratch := NewRgn;
               CopyRgn(WMgrPort.clipRgn, scratch);
               if WindowList = window then begin // For speed.
                  clipr := window^.strucRgn^^.rgnBBox;
                  SectRect(clipr, GrayRgn^^.rgnBBox, clipr);
                  ClipRect(clipr);
               end;             // Frame it next.
               drawframe(window);
               if (PaintWhite) then
                  SectRgn(window^.contRgn, scratch, scratch);
               if PaintWhite then
                  FillRgn(scratch, &window^.port.bkPat); // No content
               if SaveUpdate then
                  UnionRgn(window^.updateRgn, scratch, window^.updateRgn);
               DisposeRgn(scratch);
            end;
         end;
      end;
   end else begin               // Desktop.
      SetClip(clobberedRgn);
      paintdesk;
   end;
   SetPort(savePort);
end;


//
// Rectangle scrolling stuff.
//
// ScrollRect shifts ("scrolls") the bits that are inside the intersection of
// the specified rectangle and the visRgn, clipRgn, portRect, and
// portBits.bounds of the current grafPort.  No other bits are affected.  The
// bits are shifted a distance of dh horizontally and dv vertically.  The
// positive directions are to the right and down.  Bits that are shifted out
// of the scroll area are lost -- they're neither placed outside the area nor
// saved.  The space created by the scroll is filled with the grafPort's
// background pattern (thePort.bkPat), and the updateRgn is changed to this
// filled area.  NOTE:  If the rectangle is partially obscured by another
// window then it is likely that more area is opened up than you might think.
// UpdateRgn will correctly reflect this case.
//
// If updateRgn is NIL, it will not be returned to you.  If it is -1, the
// region will be added to the update region of the current window.  In
// addition, if it's -1 ScrollRect will also scroll any already dirty area
// (the update region of the grafPort) by the amount of the scroll.  This will
// let cumulative ScrollRects get a proper update region (event) later.
//
// ScrollRect doesn't change the coordinate system of the grafPort, it simply
// moves the entire document to _different_ coordinates.  Notice that
// ScrollRect doesn't move the pen and the clipRgn.  However, since the
// document has moved, they're in a different position relative to the
// document.
//
// To restore the coordinates of the document to what they were before the
// ScrollRect, you can use the SetOrigin procedure.  For example, if before
// the ScrollRect the coordinates of the top left of your document were
// (100,100), and you did a ScrollRect(r, 10, 20, ...) the coordinates of the
// document are offset by (10, 20).  You could call SetOrigin(90, 80) to offset
// the coordinate system to compensate for the scroll.  The document itself
// doesn't move as a result of SetOrigin (ScrollRect did that), but the pen
// and ClipRgn move down and to the right, and thus are restored to their
// original positions relative to the document.  Notice that updateRgn will
// still need to be redrawn.
//
procedure ScrollRect(r : Rect; dh, dv : smallInt; updateRgn : RgnHandle);
var
   tp : GrafPtr;
   source, dest, work : Rect;
   upRgn, workRgn : RgnHandle;
   tempCopy, dirty : Boolean;
   ii, jj : smallInt;
   tempbits : BitMap; // on the stack for later copying
   p : PByte;         // to the real destination.
   size : longInt;
begin
   tp := thePort;
   if tp^.pnVis < 0 then
      exit;
   tempCopy := False;
   source := r;
   upRgn := NewRgn;             // Now create update region.
   workRgn := NewRgn;
   SectRgn(tp^.clipRgn, tp^.visRgn, upRgn);

   SectRect(source, upRgn^^.rgnBBox, source); // Clip to work
   SectRect(source, tp^.portRect, source);    // within the
   SectRect(source, tp^.portBits.bounds, source); // visible
   dest := source;                      // window (saves effort).

   if dh < 0 then
      ii := -dh
   else
      ii := dh;                 // If we're scrolling more than the
   if dv < 0 then
      jj := -dv
   else
      jj := dv;                 // size of the rectangle, skip the
   if (ii >= (source.right - source.left)) or // scroll itself.
      (jj >= (source.bottom - source.top)) then begin
      RectRgn(upRgn, source);
   end else begin               // We need to actually scroll some bits.
      if dv <= 0 then begin     // Clip off the appropriate edges of the
         source.top -= dv;      // src and dest rectangles so that
         dest.bottom += dv;     // CopyBits moves the right stuff.
      end else begin            // (Moving downwards.)
         source.bottom -= dv;
         dest.top += dv;
         tempCopy := True;      // Can't go fastest if moving down.
      end;
      if dh <= 0 then begin     // (Moving left.)
         source.left -= dh;
         dest.right += dh;
      end else begin            // (Moving right.)
         source.right -= dh;
         dest.left += dh;
         tempCopy := True;      // Can't go fastest if moving right.
      end;
                                // We've finished preparing for CopyBits.

      RectRgn(workRgn, source);
      SectRgn(upRgn, workRgn, upRgn);
      DiffRgn(workRgn, upRgn, upRgn); // Overlapped src region (if any).
      OffsetRgn(upRgn, dh, dv); // Move to dest coordinates.

      if dv > 0 then begin      // Wipe out 'opened' top or bottom strip.
         work.top := source.top;
         work.bottom := work.top + dv; // Wipe full width of
         work.right := r.right; // rect 1st, it's faster for
         work.left := r.left; // wider ones.  (The side'll
         RectRgn(workRgn, work); // be the shorter.)
         UnionRgn(workRgn, upRgn, upRgn);
      end else if dv < 0 then begin
         work.bottom := source.bottom;
         work.top := work.bottom + dv;
         work.right := r.right;
         work.left := r.left;
         RectRgn(workRgn, work);
         UnionRgn(workRgn, upRgn, upRgn);
      end;
      if dh > 0 then begin      // Wipe the opened up side strip.
         work.top := dest.top;
         work.bottom := dest.bottom;
         work.left := source.left;
         work.right := work.left + dh;
         RectRgn(workRgn, work);
         UnionRgn(workRgn, upRgn, upRgn);
      end else if dh < 0 then begin
         work.top := dest.top;
         work.bottom := dest.bottom;
         work.right := source.right;
         work.left := work.right + dh;
         RectRgn(workRgn, work);
         UnionRgn(workRgn, upRgn, upRgn);
      end;
      if tempCopy then begin    // Make a copy of the source bitmap
         size := source.bottom - source.top;
         SetRect(tempBits.bounds, tp^.portBits.bounds.left, 0,
                 tp^.portBits.bounds.right, size);
         tempBits.rowBytes := tp^.portBits.rowBytes;
         size := MulU(size, tempBits.rowBytes);
         tempBits.baseAddr := GetMem(size);
         p := tp^.portBits.baseAddr +
              MulU(source.top - tp^.portBits.bounds.top, tempbits.rowBytes);
         system.Move(p, tempbits.baseAddr, size);
         source.bottom -= source.top;
         source.top := 0;
         CopyBits(tempbits, tp^.portBits, source, dest, srcCopy, nil);
      end else begin
         RectRgn(workRgn, dest);        // Don't bother copying onto any area
         DiffRgn(workRgn, upRgn, workRgn); // that will be erased later.
         CopyBits(tp^.portBits, tp^.portBits, source, dest,
                  srcCopy, workRgn); // Move the rectangle's contents.
      end;
   end;

   EraseRgn(upRgn);             // Erase the no-good area, and

   if updateRgn <> nil then begin // perhaps do something with it,
      if updateRgn = RgnHandle(-1) then begin // maybe accumulate it into
         dirty := EmptyRgn(WindowPeek(tp)^.updateRgn);
         if not dirty then
            CopyRgn(WindowPeek(tp)^.updateRgn, workRgn);
         InvalRgn(upRgn);       // the update region...
         if not dirty then begin // (Window already dirty?)
            RectRgn(upRgn, r);
            OffsetRgn(workRgn, tp^.portBits.bounds.left + dh,
                               tp^.portBits.bounds.top + dv);
            SectRgn(workRgn, upRgn, workRgn);
            InvalRgn(workRgn); // (Scroll dirty part too [in effect].)
         end;
      end else begin
         CopyRgn(upRgn, updateRgn); // ...or maybe return it instead.
      end;
   end;
   DisposeRgn(workRgn);
   DisposeRgn(upRgn);
end;


{$IF 0}
//
// Window List dumper.
//
procedure DumpWindows(s : string);
var
   p : WindowPeek;
begin
   WriteLn('******* Window Dump ' + s);
   DumpRgn(GrayRgn, 'Gray (Desktop)');
   p := WindowList;
   while p <> nil do begin
      WriteLn('+++ Window ''', p^.titleHandle, ''' at $', HexStr(PtrUInt(p), 8));
      DumpPort(@p^.port);
      if p^.saveVisRgn <> nil then
         DumpRgn(p^.saveVisRgn, 'Saved Visible');
      DumpRgn(p^.strucRgn, 'Structure');
      DumpRgn(p^.contRgn, 'Content');
      DumpRgn(p^.updateRgn, 'Update');
      p := p^.nextWindow;
   end;
   WriteLn('Window Manager Port at $', HexStr(PtrUInt(@WMgrPort), 8));
   Write('Bitmap bounds ');
   DumpRect(WMgrPort.portBits.bounds, 'pbb');
   Write('Port  ');
   DumpRect(WMgrPort.portRect, 'pr');
   DumpRgn(WMgrPort.clipRgn, 'Clipping');
   DumpRgn(WMgrPort.visRgn, 'Visible');
   WriteLn('End of Window Dump.');
end;
{$ENDIF}

initialization

end.
