/*
** Font File Dumper.
*/

//#include <isc/clunkdraw.h>

typedef struct {
    unsigned short fontType;    /*  0: Font type, 1 = ISC 8-bit index, */
                                /*                2 = ISC 16-bit index, */
                                /*                others = Macintosh. */
    unsigned short firstChar;   /*  2: ASCII code of first character. */
    unsigned short lastChar;    /*  4: ASCII code of last character. */
    short widMax;               /*  6: Maximum character width. */
    short kernMax;              /*  8: Negative of max character kern. */
    short nDescent;             /* 10: Negative of descent value. */
    short fRectWidth;           /* 12: Width of font rectangle. */
    short fRectHeight;          /* 14: Height of font rectangle. */
    unsigned short owTLoc;      /* 16: Offset (word, from owTLoc) to */
                                /*     offset/width table. */
    short ascent;               /* 18: Height above baseline of font. */
    short descent;              /* 20: Height below baseline of font. */
    short leading;              /* 22: Row-to-row spacing of this font. */
    unsigned short rowWords;    /* 24: Offset (word) between rows of bitmap. */
                                /* 26: The bitmap, and both tables. Actually */
                                /*     is variable size, but we can't */
                                /*     declare that in C. */
} FontRec;

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <arpa/inet.h>

void
dumpMacimage(frp, loctab, owtab, chr)
FontRec *frp;
short *loctab, *owtab;
unsigned short chr;
{
    unsigned char *p, *p2, mask, maskreload;
    int nbits, reload, row, ii;

    p = (unsigned char *) &frp[1];
    nbits = ntohs(loctab[chr]);
    reload = ntohs(loctab[chr+1]) - nbits;
    if (!reload)                /* No image for character, return. */
        return;
    p += nbits >> 3;
    mask = maskreload = 1 << (7 - (nbits & 7));
    p2 = p;
    nbits = reload;
    for (row = ntohs(frp->fRectHeight); row--; ) {
        putchar('\t');
        putchar('\t');
        if (row == ntohs(frp->fRectHeight) - ntohs(frp->ascent))
            putchar('_');
        else
            putchar(' ');
        while (nbits--) {
            if (*p & mask) {
                putchar('X');   /* Character bit marker. */
            } else {
                if (ntohs(frp->kernMax)
                    && ((ii = (ntohs(frp->kernMax) + (ntohs(owtab[chr]) >> 8))) < 0)
                    && !(reload - nbits + ii)) {
                    putchar('|'); /* Kerning (left of cell) column marker. */
                } else {
                    if (row == ntohs(frp->fRectHeight) - ntohs(frp->ascent)) {
                        ii = ntohs(owtab[chr]);
                        if ((reload - nbits) == ((ii & 0xFF) + 1
                                                 - (ntohs(frp->kernMax)
                                                    + (ii >> 8)))) {
                            putchar('|'); /* Ending pen position marker. */
                        } else {
                            putchar('_'); /* Baseline row marker. */
                        }
                    } else {
                        putchar('.'); /* Regular no-bit marker. */
                    }
                }
            }
            mask >>= 1;
            if (!mask) {
                mask = 0x80;
                p++;
            }
        }
        if (row == ntohs(frp->fRectHeight) - ntohs(frp->ascent))
            putchar('_');
        else
            putchar(' ');
        nbits = reload;
        mask = maskreload;
        p = p2 = p2 + ntohs(frp->rowWords) * 2;
        putchar('\n');
    }
    putchar('\n');
}

void
dumpimage(frp, loctab, owtab, chr)
FontRec *frp;
short *loctab, *owtab;
unsigned short chr;
{
    unsigned char *p, *p2, mask, maskreload;
    int nbits, reload, row, ii;

    if (ntohs(frp->fontType) > 2)
        dumpMacimage(frp, loctab, owtab, chr);
    else {
        ii = (ntohs(frp->fRectWidth) > 8) ? 2 : 1; // Bytes per character row.
        p = (unsigned char *) &frp[1];
        p += ntohs(frp->fRectHeight) * ii * chr;
        nbits = reload = ntohs(frp->fRectWidth);
        mask = maskreload = 1 << (7 - (nbits & 7));
        p2 = p;
        nbits = reload;
        for (row = ntohs(frp->fRectHeight); row--; ) {
            putchar('\t');
            putchar('\t');
            if (row == ntohs(frp->fRectHeight) - ntohs(frp->ascent))
                putchar('_');
            else
                putchar(' ');
            while (nbits--) {
                if (*p & mask) {
                    putchar('X');       /* Character bit marker. */
                } else {
                    if (row == ntohs(frp->fRectHeight) - ntohs(frp->ascent)) {
                        ii = owtab ? ntohs(owtab[chr]) : frp->widMax;
                        if ((reload - nbits) == ((ii & 0xFF) + 1
                                                 - (ntohs(frp->kernMax)
                                                    + (ii >> 8)))) {
                            putchar('|'); /* Ending pen position marker. */
                        } else {
                            putchar('_'); /* Baseline row marker. */
                        }
                    } else {
                        putchar('.'); /* Regular no-bit marker. */
                    }
                }
                mask >>= 1;
                if (!mask) {
                    mask = 0x80;
                    p++;
                }
            }
            if (row == ntohs(frp->fRectHeight) - ntohs(frp->ascent))
                putchar('_');
            else
                putchar(' ');
            nbits = reload;
            mask = maskreload;
            p = p2 = p2 + ((ntohs(frp->fRectWidth) > 8) ? 2 : 1);
            putchar('\n');
        }
        putchar('\n');
    }
}

int
main(argc, argv)
int argc;
char *argv[];
{
  int ii, jj, fd, imageflag, dumpflag;
    long size, lseek();
    void *malloc();
    FontRec *frp;
    char *s;
    register short *owtab;
    register unsigned short *loctab;
    register unsigned char *bp;
    short nDescent;

    if (argc != 2 && argc != 3) {
        fprintf(stderr, "Usage: %s [-image] [-pascal] fontfile.\n", argv[0]);
        exit(1);
    }
    setvbuf(stdout, 0, _IOFBF, 0); /* Fully-buffered output. */
    ii = 1;
    imageflag = 1;
    dumpflag = 0;
    if (argc == 3 && !strncmp(argv[1], "-i", 2)) {
        ii = 2;
        imageflag = 0;
    }
    if (argc == 3 && !strncmp(argv[1], "-p", 2)) {
        ii = 2;
        dumpflag = 1;
    }
    if ((fd = open(argv[ii], O_RDONLY)) < 0) {
        fprintf(stderr, "%s: Error opening font file '%s'.\n",
                argv[0], argv[ii]);
        exit(2);
    }
    size = lseek(fd, 0L, 2);    /* Find font file size. */
    lseek(fd, 0L, 0);
    if ((frp = malloc(size)) <= 0) {
        fprintf(stderr, "%s: Error allocating %d bytes of memory.\n",
                argv[0], size);
        close(fd);
        exit(3);
    }
    if (read(fd, frp, size) <= 0) {
        fprintf(stderr, "%s: Error reading font file '%s'.\n",
                argv[0], argv[ii]);
        close(fd);
        exit(4);
    }
    close(fd);
    switch (ntohs(frp->fontType)) {
      case 1:
        s = "ISC format Roman";
        break;

      case 2:
        s = "ISC format Asian";
        break;

      case 0x9000:
      case 0x9001:
        s = "Macintosh format Roman proportional";
        break;

      case 0xB000:
        s = "Macintosh format Roman fixed-pitch";
        break;

      default:
        printf("Font file '%s' not a valid font format ($%04X).\n", argv[ii],
               frp->fontType);
        exit(5);
        break;
    }
    if (dumpflag) { // Dump out in Pascal source form.  For embedding in code.
        printf("_Table: array [0..%d] of Byte = (\n", size - 1);
        bp = (unsigned char *)frp;
        for (ii=jj=0; ii<size-1; ii++) {
            if (!jj)
                printf("{%04X} ", ii);
            printf("$%02X,", bp[ii]);
            if (++jj == 16) {
                jj = 0;
                putchar('\n');
            } else {
                if (!(jj & 1))
                  putchar(' ');
            }
        }
        printf("$%02X);\n", bp[size-1]);
        exit(0);
    }
    printf("Dump of font file '%s':\n\tFormat: %s font.\n", argv[ii], s);
    printf("\tSize:\t$%04X (%d) bytes.\n\n", size, size);
    printf("\tfontType\t$%04X\n", ntohs(frp->fontType));
    printf("\tfirstChar\t$%02X (%d)\n", ntohs(frp->firstChar), ntohs(frp->firstChar));
    printf("\tlastChar\t$%02X (%d)", ntohs(frp->lastChar), ntohs(frp->lastChar));
    printf(", %d character span\n", 1 + ntohs(frp->lastChar) - ntohs(frp->firstChar));
    printf("\twidMax\t\t%d pixels\n", ntohs(frp->widMax));
    printf("\tkernMax\t\t%d pixels\n", ntohs(frp->kernMax));
    nDescent = ntohs(frp->nDescent);
    printf("\tnDescent\t%d pixels", nDescent);
    if (nDescent != -ntohs(frp->descent))
        puts(" (Error?)");
    else
        puts("");
    printf("\tfRectWidth\t%d pixels\n", ntohs(frp->fRectWidth));
    printf("\tfRectHeight\t%d pixels\n", ntohs(frp->fRectHeight));
    printf("\tascent\t\t%d pixels\n", ntohs(frp->ascent));
    printf("\tdescent\t\t%d pixels\n", ntohs(frp->descent));
    printf("\tleading\t\t%d pixels (Total vertical pitch: %d pixels)\n",
           ntohs(frp->leading), ntohs(frp->ascent) + ntohs(frp->descent) + ntohs(frp->leading));
    printf("\trowWords\t%d words\n\n", ntohs(frp->rowWords));

    if (ntohs(frp->fontType) == 1) {
        owtab = (short *) (ntohs(frp->owTLoc) ? ((char *) frp + ntohs(frp->owTLoc)) : 0);
        loctab = 0;
    } else if (ntohs(frp->fontType) != 2) {
        owtab = (short *) &frp->owTLoc + ntohs(frp->owTLoc);
        loctab = (unsigned short *) ((long) &frp->rowWords +
                                     sizeof(unsigned short));
        loctab += (ntohs(frp->fRectHeight) * ntohs(frp->rowWords));
    }
    jj = 0;
    ii = (long) &frp[1] - (long) frp;
    printf("\tBit image at $%04X (%d) bytes into font.\n", ii, ii);
    if (owtab) {
        ii = (long) owtab - (long) frp;
        printf("\tOffset/Width table at $%04X (%d) bytes into font.\n", ii, ii);
        if (loctab) {
            ii = (long) loctab - (long) frp;
            printf("\tLocation table at $%04X (%d) bytes into font.\n", ii, ii);
        }
        for (ii=ntohs(frp->firstChar); ii<(ntohs(frp->lastChar) + 2); ii++) {
            switch (ntohs(frp->fontType)) {
              case 1:
                printf("\tChar %02X (%d):\tWidth: %3d", ii, ii, ntohs(owtab[jj]));
                break;

              case 0x9000:
              case 0x9001:
              case 0xB000:
                if (ntohs(owtab[jj]) == -1)
                    printf("\tChar %02X (%d):\tLocation %4d\tOff/Wid: %3d",
                           ii, ii, ntohs(loctab[jj]), -1);
                else
                    printf("\tChar %02X (%d):\tLocation %4d\tOffset:  %3d  Width: %3d",
                           ii, ii, ntohs(loctab[jj]), ntohs(owtab[jj]) >> 8, ntohs(owtab[jj]) & 0xFF);
                break;
            }
            if (ntohs(owtab[jj]) == -1)
                printf(" (Missing)");
            if (ii >= ' ' && ii <= 126)
                printf("  '%c'", ii);
            else if (ii == (ntohs(frp->lastChar) + 1))
                printf("  Missing Symbol");
            puts("");
            if (imageflag)
                dumpimage(frp, loctab, owtab, jj);
            jj++;
        }
    } else {
        for (ii=ntohs(frp->firstChar); ii<(ntohs(frp->lastChar) + 2); ii++) {
            switch (ntohs(frp->fontType)) {
              case 1:
                printf("\tChar %02X (%d):\tWidth: %3d", ii, ii, ntohs(frp->widMax));
                break;

              case 0x9000:
              case 0x9001:
              case 0xB000:
                break;
            }
            if (ii >= ' ' && ii <= 126)
                printf("  '%c'", ii);
            else if (ii == (ntohs(frp->lastChar) + 1))
                printf("  Missing Symbol");
            puts("");
            if (imageflag)
                dumpimage(frp, loctab, owtab, jj);
            jj++;
        }
    }
    if (ntohs(frp->fontType) == 2)
        printf("\tAsian font image file: '%s'\n", &frp[1]);
}
