//===========================================================================
//
// Beginning of 3.7" Waveshare e-Paper display (EPD) support code.
// Taken from their C demo library, and ported to Pascal.
//
//===========================================================================

unit epaper;

{$mode objfpc}{$H+}

interface

type

   spi_ioc_transfer = packed record
      tx_buf, rx_buf : qword; // U64
      len, speed_hz  : dword; // U32
      delay_usecs    : word;  // U16
      bits_per_word,
      cs_change,
      tx_nbits,
      rx_nbits       : byte;  // U8
      pad            : word;  // U16
end;

const EPD_3IN7_WIDTH = 280;
      EPD_3IN7_HEIGHT = 480;
// Constants from /usr/include/linux/spi/spidev.h
   SPI_CPHA = 1;
   SPI_CPOL = 2;

   // SPI mode register bits
   SPIM_MODE_0 = (0 or 0);
   SPIM_MODE_1 = (0 or SPI_CPHA);
   SPIM_MODE_2 = (SPI_CPOL or 0);
   SPIM_MODE_3 = (SPI_CPOL or SPI_CPHA);
   SPIM_CS_HIGH = $04;
   SPIM_LSB_FIRST = $08;
   SPIM_3WIRE = $10;
   SPIM_LOOP = $20;
   SPIM_NO_CS = $40;
   SPIM_READY = $80;
   SPIM_TX_DUAL = $100;
   SPIM_TX_QUAD = $200;
   SPIM_RX_DUAL = $400;
   SPIM_RX_QUAD = $800;

   SPI_IOC_MAGIC = 'k';

   SPI_CS_Mode_LOW = 0;     // Chip Select 0
   SPI_CS_Mode_HIGH = 1;    // Chip Select 1
   SPI_CS_Mode_NONE = 3;    // No CS, control it yourself
   SPI_BIT_ORDER_LSBFIRST = 0;  // LSB First
   SPI_BIT_ORDER_MSBFIRST = 1;  // MSB First

   // In these ioctl codes the 6B is the 'k' magic from above.
   SPI_IOC_RD_MODE = $80016B01;          // 1-byte ioctl read, virtual reg #1
   SPI_IOC_WR_MODE = $40016B01;          // 1-byte ioctl write, virtual reg #1
   SPI_IOC_RD_LSB_FIRST = $80016B02;     // 1-byte ioctl read, virtual reg #2
   SPI_IOC_WR_LSB_FIRST = $40016B02;     // 1-byte ioctl write, virtual reg #2
   SPI_IOC_RD_BITS_PER_WORD = $80016B03; // 1-byte ioctl read, virtual reg #3
   SPI_IOC_WR_BITS_PER_WORD = $40016B03; // 1-byte ioctl write, virtual reg #3
   SPI_IOC_RD_MAX_SPEED_HZ = $80046B04;  // 4-byte ioctl read, virtual reg #4
   SPI_IOC_WR_MAX_SPEED_HZ = $40046B04;  // 4-byte ioctl write, virtual reg #4
   SPI_IOC_MESSAGE = $40206B00;    // 32-byte ioctl write/read, virtual reg #0

var
   EPD_RST_PIN : integer; // Low is reset.
   EPD_DC_PIN : integer; // Data/Command selection pin, low is command.
   EPD_CS_PIN : integer; // SPI chip select, active low.
   EPD_BUSY_PIN : integer; // Low is busy.

   bits : byte = 8;

   tr : spi_ioc_transfer;
   spifd : integer;
   spiMode : byte;

   procedure EPD_SPI_Init;
   procedure EPD_SPI_Exit;
   procedure EPD_3IN7_Test;
   procedure EPD_3IN7_4Gray_Init;
   procedure EPD_3IN7_1Gray_Init;
   procedure EPD_3IN7_4Gray_Clear;
   procedure EPD_3IN7_1Gray_Clear;
   procedure EPD_3IN7_1Gray_Display(Image : PByte);
   procedure EPD_3IN7_1Gray_Display_Part(Image : PByte; Xstart, Ystart, Xend, Yend : integer);
   procedure EPD_3IN7_4Gray_Display(Image : PByte);
   procedure EPD_Delay_ms(xms : integer);
   procedure EPD_3IN7_Sleep;

implementation
   uses
{$IFDEF Unix}
      BaseUnix, Unix,
{$ENDIF}
   strings, sysutils, Math, Clunkdraw;

//////////////////////////////////////////////////////////////////////////////
//// Beginning of code duplicated from main program.  Can't quite figure out
//// how to get access to that code in this separate unit.

// Program a GPIO pin to be an output.  We need elevated privileges to do this.

Procedure gpioPinOut(pin : integer);
var s : string;
begin
   s := 'echo ' + IntToStr(pin) + ' | sudo tee /sys/class/gpio/export';
   fpSystem(s + ' >/dev/null 2>&1');
   s := 'echo out | sudo tee /sys/class/gpio/gpio'+IntToStr(pin) + '/direction';
   fpSystem(s + ' >/dev/null 2>&1');
end;

// Program a GPIO pin to be an input.  We need elevated privileges to do this.

Procedure gpioPinIn(pin : integer);
var s : string;
begin
   s := 'echo ' + IntToStr(pin) + ' | sudo tee /sys/class/gpio/export';
   fpSystem(s + ' >/dev/null 2>&1');
   s := 'echo in | sudo tee /sys/class/gpio/gpio'+IntToStr(pin) + '/direction';
   fpSystem(s + ' >/dev/null 2>&1');
end;

// Write value to selected GPIO control file.  This way we can wiggle
// I/O pins directly without using additional drivers (like I2C or SSI).
// For our needs nothing faster is necessary, and this is simpler.

Procedure gpioW(fName : string; value : string);
var writeTo : Text;
    f       : string;
begin
   f := '/sys/class/gpio/' + fName;
   try
      Assign(writeTo, f);
      Rewrite(writeTo);
      Write(writeTo, value);
      Close(writeTo);
   except
      exit;
   end;
end;

// Read GPIO pin's value.

Function gpioR(fName : string) : Boolean;
var readFrom : Text;
    f, value : string;
begin
   f := '/sys/class/gpio/' + fName;
   try
      Assign(readFrom, f);
      Reset(readFrom);
      Read(readFrom, value);
      Close(readFrom);
   except
      exit;
   end;
   result := value[1] = '1';
end;

// Turn the selected LED on or off.

Procedure setPin(pin : integer; lit : Boolean);
var s : string;
begin
   if (pin <= 0) then exit;
   s := 'gpio' + IntToStr(pin) + '/value';
   if lit then
      gpioW(s, '1')
   else
      gpioW(s, '0');
end;

// Return the selected LED on/off value.  Used (as IPC) in rare conditions
// where there might be multiple processes wiggling pins.

Function getPin(pin : integer) : Boolean;
var s : string;
begin
   result := False;
   if (pin <= 0) then exit;
   s := 'gpio' + IntToStr(pin) + '/value';
   result := gpioR(s);
end;

//// End of code duplicated from main program.  Can't quite figure out
//// how to share that code in this separate unit.
//////////////////////////////////////////////////////////////////////////////

procedure EPD_Delay_ms(xms : integer);
var
   Req, Rem : TimeSpec;
begin
   with Req do begin
      tv_sec := xms div 1000;
      tv_nsec := (xms mod 1000) * 1000;
   end;
   fpNanoSleep(@Req, @rem);
end;

procedure EPD_GPIO_Init;
begin
   EPD_RST_PIN     := 17;
   EPD_DC_PIN      := 25;
   EPD_CS_PIN      := 8;
   EPD_BUSY_PIN    := 24;
   gpioPinOut(EPD_RST_PIN);
   gpioPinOut(EPD_DC_PIN);
   gpioPinOut(EPD_CS_PIN);
   gpioPinIn(EPD_BUSY_PIN);
   setPin(EPD_CS_PIN, True);
end;

function EPD_SPI_setSpeed(speed : dword) : integer;
var ret : integer;
begin
   result := 1;
   ret := fpIOCtl(spiFd, SPI_IOC_WR_MAX_SPEED_HZ, @speed);
   if ret < 0 then
      result := ret
   else
      tr.speed_hz := speed;
end;

function EPD_SPI_Mode(mode : integer) : integer;
var ret : integer;
begin
   result := 1;
   spiMode := spiMode and $FC; // Clear low 2 bits.
   spiMode := spiMode or mode;
   ret := fpIOCtl(spiFd, SPI_IOC_WR_MAX_SPEED_HZ, @spiMode);
   if ret < 0 then
      result := ret;
end;

function EPD_SPI_ChipSelect(CS_Mode : integer) : integer;
var ret : integer;
begin
   if CS_Mode = SPI_CS_Mode_HIGH then begin
      spiMode := spiMode or SPIM_CS_HIGH;
      spiMode := spiMode and not SPIM_NO_CS;
   end else if CS_Mode = SPI_CS_Mode_LOW then begin
      spiMode := spiMode and not SPIM_CS_HIGH;
      spiMode := spiMode and not SPIM_NO_CS;
   end else if CS_Mode = SPI_CS_Mode_NONE then begin
      spiMode := spiMode or SPIM_NO_CS;
   end;
   result := 1;
   ret := fpIOCtl(spiFd, SPI_IOC_WR_MODE, @spiMode);
   if ret < 0 then
      result := ret;
end;

function EPD_SPI_SetBitOrder(Order : integer) : integer;
var ret : integer;
begin
   result := 1;
   if Order = SPI_BIT_ORDER_LSBFIRST then begin
      spiMode := spiMode or SPIM_LSB_FIRST;
   end else if Order = SPI_BIT_ORDER_MSBFIRST then begin
      spiMode := spiMode and not SPIM_LSB_FIRST;
   end else begin
      exit;
   end;
   ret := fpIOCtl(spiFd, SPI_IOC_WR_MODE, @spiMode);
   if ret < 0 then
      result := ret;
end;

procedure EPD_SPI_SetDataInterval(us : dword);
begin
   tr.delay_usecs := us;
end;

procedure EPD_SPI_begin(SPI_device : string);
var ret :  integer;
begin
   ret := 0;
   spiFd := fpOpen(SPI_device, O_RDWR);
   if spifd < 0 then begin
      writeLn('Failed to open SPI device');
      halt(1);
   end;
   spiMode := 0;

   ret := fpIOCtl(spiFd, SPI_IOC_WR_BITS_PER_WORD, @bits);
   if ret < 0 then
      WriteLn('can''t set bits per word');

   tr.bits_per_word := bits;

   EPD_SPI_Mode(SPIM_MODE_0);
   EPD_SPI_ChipSelect(SPI_CS_Mode_LOW);
   EPD_SPI_SetBitOrder(SPI_BIT_ORDER_LSBFIRST);
   EPD_SPI_setSpeed(20000000);
   EPD_SPI_SetDataInterval(0);
end;

procedure EPD_SPI_end;
var ret : integer;
begin
   spiMode := 0;
   ret := fpClose(spiFd);
   spiFd := -1;
   if ret <> 0 then
      WriteLn('Failed to close SPI device');
end;

function EPD_SPI_TransferByte(buf : PByte) : byte;
var rbuf : byte;
begin
   tr.len := 1;
   tr.tx_buf := QWord(PtrUInt(buf));
   tr.rx_buf := QWord(PtrUint(@rbuf));
   fpIOCtl(spiFd, SPI_IOC_MESSAGE, @tr);
   result := rbuf;
end;

procedure EPD_SPI_WriteByte(value : byte);
begin
   EPD_SPI_TransferByte(@value);
end;

procedure EPD_SPI_Init;
begin
   EPD_GPIO_Init;
   EPD_SPI_begin('/dev/spidev0.0');
   EPD_SPI_setSpeed(10000000);
end;

{Module exits, closes SPI}
procedure EPD_SPI_Exit;
begin
   setPin(EPD_CS_PIN, False);
   setPin(EPD_DC_PIN, False);
   setPin(EPD_RST_PIN, False);
end;

const LutSize = 104; // Zero-based, 105 total cells.
lut_4Gray_GC: array [0..LutSize] of Byte = (
   $2A,$06,$15,$00,$00,$00,$00,$00,$00,$00, // 1
   $28,$06,$14,$00,$00,$00,$00,$00,$00,$00, // 2
   $20,$06,$10,$00,$00,$00,$00,$00,$00,$00, // 3
   $14,$06,$28,$00,$00,$00,$00,$00,$00,$00, // 4
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 5
   $00,$02,$02,$0A,$00,$00,$00,$08,$08,$02, // 6
   $00,$02,$02,$0A,$00,$00,$00,$00,$00,$00, // 7
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 8
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 9
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, //10
   $22,$22,$22,$22,$22
);

lut_1Gray_GC: array [0..LutSize] of Byte = (
   $2A,$05,$00,$00,$00,$00,$00,$00,$00,$00, // 1
   $05,$2A,$00,$00,$00,$00,$00,$00,$00,$00, // 2
   $2A,$15,$00,$00,$00,$00,$00,$00,$00,$00, // 3
   $05,$0A,$00,$00,$00,$00,$00,$00,$00,$00, // 4
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 5
   $00,$02,$03,$0A,$00,$02,$06,$0A,$05,$00, // 6
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 7
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 8
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 9
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, //10
   $22,$22,$22,$22,$22
);

lut_1Gray_DU: array [0..LutSize] of Byte = (
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 1
   $01,$2A,$00,$00,$00,$00,$00,$00,$00,$00, // 2
   $0A,$55,$00,$00,$00,$00,$00,$00,$00,$00, // 3
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 4
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 5
   $00,$00,$05,$05,$00,$05,$03,$05,$05,$00, // 6
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 7
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 8
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 9
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, //10
   $22,$22,$22,$22,$22
);

lut_1Gray_A2: array [0..LutSize] of Byte = (
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 1
   $0A,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 2
   $05,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 3
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 4
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 5
   $00,$00,$03,$05,$00,$00,$00,$00,$00,$00, // 6
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 7
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 8
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, // 9
   $00,$00,$00,$00,$00,$00,$00,$00,$00,$00, //10
   $22,$22,$22,$22,$22
);

(************************ Font (C) COPYRIGHT STMicroelectronics **)
Font24_Table: array [0..3*24*(127-32)-1] of Byte = (
        // @0 ' ' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @72 '!' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $01, $00, $00, //        #
        $01, $00, $00, //        #
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @144 '"' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $0E, $70, $00, //     ###  ###
        $0E, $70, $00, //     ###  ###
        $0E, $70, $00, //     ###  ###
        $04, $20, $00, //      #    #
        $04, $20, $00, //      #    #
        $04, $20, $00, //      #    #
        $04, $20, $00, //      #    #
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @216 '#' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $06, $60, $00, //      ##  ##
        $06, $60, $00, //      ##  ##
        $06, $60, $00, //      ##  ##
        $06, $60, $00, //      ##  ##
        $06, $60, $00, //      ##  ##
        $3F, $F8, $00, //   ###########
        $3F, $F8, $00, //   ###########
        $06, $60, $00, //      ##  ##
        $0C, $C0, $00, //     ##  ##
        $3F, $F8, $00, //   ###########
        $3F, $F8, $00, //   ###########
        $0C, $C0, $00, //     ##  ##
        $0C, $C0, $00, //     ##  ##
        $0C, $C0, $00, //     ##  ##
        $0C, $C0, $00, //     ##  ##
        $0C, $C0, $00, //     ##  ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @288 '$' (17 pixels wide)
        $00, $00, $00, //
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $07, $B0, $00, //      #### ##
        $0F, $F0, $00, //     ########
        $18, $70, $00, //    ##    ###
        $18, $70, $00, //    ##    ###
        $1C, $00, $00, //    ###
        $0F, $80, $00, //     #####
        $07, $E0, $00, //      ######
        $00, $F0, $00, //         ####
        $18, $30, $00, //    ##     ##
        $1C, $30, $00, //    ###    ##
        $1C, $70, $00, //    ###   ###
        $1F, $E0, $00, //    ########
        $1B, $C0, $00, //    ## ####
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @360 '%' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $80, $00, //      ####
        $0F, $C0, $00, //     ######
        $1C, $E0, $00, //    ###  ###
        $18, $60, $00, //    ##    ##
        $18, $60, $00, //    ##    ##
        $1C, $E0, $00, //    ###  ###
        $0F, $F8, $00, //     #########
        $07, $E0, $00, //      ######
        $1F, $F0, $00, //    #########
        $07, $38, $00, //      ###  ###
        $06, $18, $00, //      ##    ##
        $06, $18, $00, //      ##    ##
        $07, $38, $00, //      ###  ###
        $03, $F0, $00, //       ######
        $01, $E0, $00, //        ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @432 '&' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $F0, $00, //       ######
        $07, $F0, $00, //      #######
        $0C, $60, $00, //     ##   ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $06, $00, $00, //      ##
        $07, $00, $00, //      ###
        $0F, $9C, $00, //     #####  ###
        $1D, $FC, $00, //    ### #######
        $18, $F0, $00, //    ##   ####
        $18, $70, $00, //    ##    ###
        $0F, $FC, $00, //     ##########
        $07, $DC, $00, //      ##### ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @504 ''' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $01, $00, $00, //        #
        $01, $00, $00, //        #
        $01, $00, $00, //        #
        $01, $00, $00, //        #
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @576 '(' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $18, $00, //            ##
        $00, $38, $00, //           ###
        $00, $70, $00, //          ###
        $00, $F0, $00, //         ####
        $00, $E0, $00, //         ###
        $00, $E0, $00, //         ###
        $01, $C0, $00, //        ###
        $01, $C0, $00, //        ###
        $01, $C0, $00, //        ###
        $01, $C0, $00, //        ###
        $01, $C0, $00, //        ###
        $01, $C0, $00, //        ###
        $00, $E0, $00, //         ###
        $00, $E0, $00, //         ###
        $00, $70, $00, //          ###
        $00, $70, $00, //          ###
        $00, $38, $00, //           ###
        $00, $18, $00, //            ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @648 ')' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $18, $00, $00, //    ##
        $1C, $00, $00, //    ###
        $0E, $00, $00, //     ###
        $0E, $00, $00, //     ###
        $07, $00, $00, //      ###
        $07, $00, $00, //      ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $03, $80, $00, //       ###
        $07, $00, $00, //      ###
        $07, $00, $00, //      ###
        $0F, $00, $00, //     ####
        $0E, $00, $00, //     ###
        $1C, $00, $00, //    ###
        $18, $00, $00, //    ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @720 '*' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $1D, $B8, $00, //    ### ## ###
        $1F, $F8, $00, //    ##########
        $07, $E0, $00, //      ######
        $03, $C0, $00, //       ####
        $03, $C0, $00, //       ####
        $06, $60, $00, //      ##  ##
        $06, $60, $00, //      ##  ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @792 '+' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $3F, $FC, $00, //   ############
        $3F, $FC, $00, //   ############
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @864 ',' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $E0, $00, //         ###
        $00, $C0, $00, //         ##
        $01, $C0, $00, //        ###
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $03, $00, $00, //       ##
        $03, $00, $00, //       ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @936 '-' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1008 '.' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $C0, $00, //       ####
        $03, $C0, $00, //       ####
        $03, $C0, $00, //       ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1080 '/' (17 pixels wide)
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $00, $38, $00, //           ###
        $00, $30, $00, //           ##
        $00, $70, $00, //          ###
        $00, $60, $00, //          ##
        $00, $60, $00, //          ##
        $00, $C0, $00, //         ##
        $00, $C0, $00, //         ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $03, $00, $00, //       ##
        $03, $00, $00, //       ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $0E, $00, $00, //     ###
        $0C, $00, $00, //     ##
        $1C, $00, $00, //    ###
        $18, $00, $00, //    ##
        $18, $00, $00, //    ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1152 '0' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $C0, $00, //       ####
        $07, $E0, $00, //      ######
        $0C, $30, $00, //     ##    ##
        $0C, $30, $00, //     ##    ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $0C, $30, $00, //     ##    ##
        $0C, $30, $00, //     ##    ##
        $07, $E0, $00, //      ######
        $03, $C0, $00, //       ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1224 '1' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $80, $00, //         #
        $07, $80, $00, //      ####
        $1F, $80, $00, //    ######
        $1D, $80, $00, //    ### ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1296 '2' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $C0, $00, //      #####
        $1F, $F0, $00, //    #########
        $38, $30, $00, //   ###     ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $00, $18, $00, //            ##
        $00, $30, $00, //           ##
        $00, $60, $00, //          ##
        $01, $C0, $00, //        ###
        $03, $80, $00, //       ###
        $06, $00, $00, //      ##
        $0C, $00, $00, //     ##
        $18, $00, $00, //    ##
        $3F, $F8, $00, //   ###########
        $3F, $F8, $00, //   ###########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1368 '3' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $C0, $00, //       ####
        $0F, $E0, $00, //     #######
        $0C, $70, $00, //     ##   ###
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $60, $00, //          ##
        $03, $C0, $00, //       ####
        $03, $E0, $00, //       #####
        $00, $70, $00, //          ###
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $18, $38, $00, //    ##     ###
        $1F, $F0, $00, //    #########
        $0F, $C0, $00, //     ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1440 '4' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $E0, $00, //         ###
        $01, $E0, $00, //        ####
        $01, $E0, $00, //        ####
        $03, $60, $00, //       ## ##
        $06, $60, $00, //      ##  ##
        $06, $60, $00, //      ##  ##
        $0C, $60, $00, //     ##   ##
        $0C, $60, $00, //     ##   ##
        $18, $60, $00, //    ##    ##
        $30, $60, $00, //   ##     ##
        $3F, $F8, $00, //   ###########
        $3F, $F8, $00, //   ###########
        $00, $60, $00, //          ##
        $03, $F8, $00, //       #######
        $03, $F8, $00, //       #######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1512 '5' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $F0, $00, //    #########
        $1F, $F0, $00, //    #########
        $18, $00, $00, //    ##
        $18, $00, $00, //    ##
        $18, $00, $00, //    ##
        $1B, $C0, $00, //    ## ####
        $1F, $F0, $00, //    #########
        $1C, $30, $00, //    ###    ##
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $30, $30, $00, //   ##      ##
        $3F, $F0, $00, //   ##########
        $0F, $C0, $00, //     ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1584 '6' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $F8, $00, //         #####
        $03, $F8, $00, //       #######
        $07, $00, $00, //      ###
        $0E, $00, $00, //     ###
        $0C, $00, $00, //     ##
        $18, $00, $00, //    ##
        $1B, $C0, $00, //    ## ####
        $1F, $F0, $00, //    #########
        $1C, $30, $00, //    ###    ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $0C, $38, $00, //     ##    ###
        $0F, $F0, $00, //     ########
        $03, $E0, $00, //       #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1656 '7' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $18, $18, $00, //    ##      ##
        $18, $38, $00, //    ##     ###
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $70, $00, //          ###
        $00, $60, $00, //          ##
        $00, $60, $00, //          ##
        $00, $E0, $00, //         ###
        $00, $C0, $00, //         ##
        $00, $C0, $00, //         ##
        $01, $C0, $00, //        ###
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1728 '8' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $E0, $00, //      ######
        $0F, $F0, $00, //     ########
        $1C, $38, $00, //    ###    ###
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $0C, $30, $00, //     ##    ##
        $07, $E0, $00, //      ######
        $07, $E0, $00, //      ######
        $0C, $30, $00, //     ##    ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $1C, $38, $00, //    ###    ###
        $0F, $F0, $00, //     ########
        $07, $E0, $00, //      ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1800 '9' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $C0, $00, //      #####
        $0F, $F0, $00, //     ########
        $1C, $30, $00, //    ###    ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $0C, $38, $00, //     ##    ###
        $0F, $F8, $00, //     #########
        $03, $D8, $00, //       #### ##
        $00, $18, $00, //            ##
        $00, $30, $00, //           ##
        $00, $70, $00, //          ###
        $00, $E0, $00, //         ###
        $1F, $C0, $00, //    #######
        $1F, $00, $00, //    #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1872 ':' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $C0, $00, //       ####
        $03, $C0, $00, //       ####
        $03, $C0, $00, //       ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $C0, $00, //       ####
        $03, $C0, $00, //       ####
        $03, $C0, $00, //       ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @1944 ';' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $F0, $00, //         ####
        $00, $F0, $00, //         ####
        $00, $F0, $00, //         ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $E0, $00, //         ###
        $01, $C0, $00, //        ###
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $03, $00, $00, //       ##
        $02, $00, $00, //       #
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2016 '<' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $1C, $00, //            ###
        $00, $3C, $00, //           ####
        $00, $F0, $00, //         ####
        $03, $C0, $00, //       ####
        $0F, $00, $00, //     ####
        $3C, $00, $00, //   ####
        $F0, $00, $00, // ####
        $3C, $00, $00, //   ####
        $0F, $00, $00, //     ####
        $03, $C0, $00, //       ####
        $00, $F0, $00, //         ####
        $00, $3C, $00, //           ####
        $00, $1C, $00, //            ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2088 '=' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $FC, $00, //  #############
        $7F, $FC, $00, //  #############
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $FC, $00, //  #############
        $7F, $FC, $00, //  #############
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2160 '>' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $70, $00, $00, //  ###
        $78, $00, $00, //  ####
        $1E, $00, $00, //    ####
        $07, $80, $00, //      ####
        $01, $E0, $00, //        ####
        $00, $78, $00, //          ####
        $00, $1E, $00, //            ####
        $00, $78, $00, //          ####
        $01, $E0, $00, //        ####
        $07, $80, $00, //      ####
        $1E, $00, $00, //    ####
        $78, $00, $00, //  ####
        $70, $00, $00, //  ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2232 '?' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $C0, $00, //      #####
        $0F, $E0, $00, //     #######
        $18, $70, $00, //    ##    ###
        $18, $30, $00, //    ##     ##
        $18, $30, $00, //    ##     ##
        $00, $70, $00, //          ###
        $00, $E0, $00, //         ###
        $03, $C0, $00, //       ####
        $03, $80, $00, //       ###
        $03, $00, $00, //       ##
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $00, $00, //      ###
        $07, $00, $00, //      ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2304 '@' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $E0, $00, //       #####
        $07, $F0, $00, //      #######
        $0E, $38, $00, //     ###   ###
        $0C, $18, $00, //     ##     ##
        $18, $78, $00, //    ##    ####
        $18, $F8, $00, //    ##   #####
        $19, $D8, $00, //    ##  ### ##
        $19, $98, $00, //    ##  ##  ##
        $19, $98, $00, //    ##  ##  ##
        $19, $98, $00, //    ##  ##  ##
        $18, $F8, $00, //    ##   #####
        $18, $78, $00, //    ##    ####
        $18, $00, $00, //    ##
        $0C, $00, $00, //     ##
        $0E, $18, $00, //     ###    ##
        $07, $F8, $00, //      ########
        $03, $E0, $00, //       #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2376 'A' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $80, $00, //    ######
        $1F, $C0, $00, //    #######
        $01, $C0, $00, //        ###
        $03, $60, $00, //       ## ##
        $03, $60, $00, //       ## ##
        $06, $30, $00, //      ##   ##
        $06, $30, $00, //      ##   ##
        $0C, $30, $00, //     ##    ##
        $0F, $F8, $00, //     #########
        $1F, $F8, $00, //    ##########
        $18, $0C, $00, //    ##       ##
        $30, $0C, $00, //   ##        ##
        $FC, $7F, $00, // ######   #######
        $FC, $7F, $00, // ######   #######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2448 'B' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $E0, $00, //  ##########
        $7F, $F0, $00, //  ###########
        $18, $38, $00, //    ##     ###
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $38, $00, //    ##     ###
        $1F, $F0, $00, //    #########
        $1F, $F8, $00, //    ##########
        $18, $1C, $00, //    ##      ###
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $7F, $F8, $00, //  ############
        $7F, $F0, $00, //  ###########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2520 'C' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $EC, $00, //       ##### ##
        $0F, $FC, $00, //     ##########
        $1C, $1C, $00, //    ###     ###
        $18, $0C, $00, //    ##       ##
        $30, $0C, $00, //   ##        ##
        $30, $00, $00, //   ##
        $30, $00, $00, //   ##
        $30, $00, $00, //   ##
        $30, $00, $00, //   ##
        $30, $00, $00, //   ##
        $18, $0C, $00, //    ##       ##
        $1C, $1C, $00, //    ###     ###
        $0F, $F8, $00, //     #########
        $03, $F0, $00, //       ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2592 'D' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $C0, $00, //  #########
        $7F, $F0, $00, //  ###########
        $18, $38, $00, //    ##     ###
        $18, $18, $00, //    ##      ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $18, $00, //    ##      ##
        $18, $38, $00, //    ##     ###
        $7F, $F0, $00, //  ###########
        $7F, $E0, $00, //  ##########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2664 'E' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $F8, $00, //  ############
        $7F, $F8, $00, //  ############
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $19, $98, $00, //    ##  ##  ##
        $19, $80, $00, //    ##  ##
        $1F, $80, $00, //    ######
        $1F, $80, $00, //    ######
        $19, $80, $00, //    ##  ##
        $19, $98, $00, //    ##  ##  ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $7F, $F8, $00, //  ############
        $7F, $F8, $00, //  ############
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2736 'F' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $3F, $FC, $00, //   ############
        $3F, $FC, $00, //   ############
        $0C, $0C, $00, //     ##      ##
        $0C, $0C, $00, //     ##      ##
        $0C, $CC, $00, //     ##  ##  ##
        $0C, $C0, $00, //     ##  ##
        $0F, $C0, $00, //     ######
        $0F, $C0, $00, //     ######
        $0C, $C0, $00, //     ##  ##
        $0C, $C0, $00, //     ##  ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $3F, $C0, $00, //   ########
        $3F, $C0, $00, //   ########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2808 'G' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $EC, $00, //       ##### ##
        $0F, $FC, $00, //     ##########
        $1C, $1C, $00, //    ###     ###
        $18, $0C, $00, //    ##       ##
        $30, $0C, $00, //   ##        ##
        $30, $00, $00, //   ##
        $30, $00, $00, //   ##
        $30, $FE, $00, //   ##    #######
        $30, $FE, $00, //   ##    #######
        $30, $0C, $00, //   ##        ##
        $38, $0C, $00, //   ###       ##
        $1C, $1C, $00, //    ###     ###
        $0F, $FC, $00, //     ##########
        $03, $F0, $00, //       ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2880 'H' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7E, $7E, $00, //  ######  ######
        $7E, $7E, $00, //  ######  ######
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $7E, $7E, $00, //  ######  ######
        $7E, $7E, $00, //  ######  ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @2952 'I' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3024 'J' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $FE, $00, //      ##########
        $07, $FE, $00, //      ##########
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $30, $30, $00, //   ##      ##
        $30, $30, $00, //   ##      ##
        $30, $30, $00, //   ##      ##
        $30, $30, $00, //   ##      ##
        $30, $60, $00, //   ##     ##
        $3F, $E0, $00, //   #########
        $0F, $80, $00, //     #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3096 'K' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $3E, $00, //  #######  #####
        $7F, $3E, $00, //  #######  #####
        $18, $30, $00, //    ##     ##
        $18, $60, $00, //    ##    ##
        $18, $C0, $00, //    ##   ##
        $19, $80, $00, //    ##  ##
        $1B, $80, $00, //    ## ###
        $1F, $C0, $00, //    #######
        $1C, $E0, $00, //    ###  ###
        $18, $70, $00, //    ##    ###
        $18, $30, $00, //    ##     ##
        $18, $38, $00, //    ##     ###
        $7F, $1F, $00, //  #######   #####
        $7F, $1F, $00, //  #######   #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3168 'L' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $80, $00, //  ########
        $7F, $80, $00, //  ########
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $0C, $00, //     ##      ##
        $0C, $0C, $00, //     ##      ##
        $0C, $0C, $00, //     ##      ##
        $0C, $0C, $00, //     ##      ##
        $7F, $FC, $00, //  #############
        $7F, $FC, $00, //  #############
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3240 'M' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $F0, $0F, $00, // ####        ####
        $F8, $1F, $00, // #####      #####
        $38, $1C, $00, //   ###      ###
        $3C, $3C, $00, //   ####    ####
        $3C, $3C, $00, //   ####    ####
        $36, $6C, $00, //   ## ##  ## ##
        $36, $6C, $00, //   ## ##  ## ##
        $33, $CC, $00, //   ##  ####  ##
        $33, $CC, $00, //   ##  ####  ##
        $31, $8C, $00, //   ##   ##   ##
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $FE, $7F, $00, // #######  #######
        $FE, $7F, $00, // #######  #######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3312 'N' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $78, $FE, $00, //  ####   #######
        $78, $FE, $00, //  ####   #######
        $1C, $18, $00, //    ###     ##
        $1E, $18, $00, //    ####    ##
        $1F, $18, $00, //    #####   ##
        $1B, $18, $00, //    ## ##   ##
        $1B, $98, $00, //    ## ###  ##
        $19, $D8, $00, //    ##  ### ##
        $18, $D8, $00, //    ##   ## ##
        $18, $F8, $00, //    ##   #####
        $18, $78, $00, //    ##    ####
        $18, $38, $00, //    ##     ###
        $7F, $18, $00, //  #######   ##
        $7F, $18, $00, //  #######   ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3384 'O' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $C0, $00, //       ####
        $0F, $F0, $00, //     ########
        $1C, $38, $00, //    ###    ###
        $18, $18, $00, //    ##      ##
        $38, $1C, $00, //   ###      ###
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $38, $1C, $00, //   ###      ###
        $18, $18, $00, //    ##      ##
        $1C, $38, $00, //    ###    ###
        $0F, $F0, $00, //     ########
        $03, $C0, $00, //       ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3456 'P' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $3F, $F0, $00, //   ##########
        $3F, $F8, $00, //   ###########
        $0C, $1C, $00, //     ##     ###
        $0C, $0C, $00, //     ##      ##
        $0C, $0C, $00, //     ##      ##
        $0C, $0C, $00, //     ##      ##
        $0C, $18, $00, //     ##     ##
        $0F, $F8, $00, //     #########
        $0F, $E0, $00, //     #######
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $3F, $C0, $00, //   ########
        $3F, $C0, $00, //   ########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3528 'Q' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $C0, $00, //       ####
        $0F, $F0, $00, //     ########
        $1C, $38, $00, //    ###    ###
        $18, $18, $00, //    ##      ##
        $38, $1C, $00, //   ###      ###
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $38, $1C, $00, //   ###      ###
        $18, $18, $00, //    ##      ##
        $1C, $38, $00, //    ###    ###
        $0F, $F0, $00, //     ########
        $07, $C0, $00, //      #####
        $07, $CC, $00, //      #####  ##
        $0F, $FC, $00, //     ##########
        $0C, $38, $00, //     ##    ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3600 'R' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $E0, $00, //  ##########
        $7F, $F0, $00, //  ###########
        $18, $38, $00, //    ##     ###
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $38, $00, //    ##     ###
        $1F, $F0, $00, //    #########
        $1F, $C0, $00, //    #######
        $18, $E0, $00, //    ##   ###
        $18, $70, $00, //    ##    ###
        $18, $30, $00, //    ##     ##
        $18, $38, $00, //    ##     ###
        $7F, $1E, $00, //  #######   ####
        $7F, $0E, $00, //  #######    ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3672 'S' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $D8, $00, //      ##### ##
        $0F, $F8, $00, //     #########
        $1C, $38, $00, //    ###    ###
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $1E, $00, $00, //    ####
        $0F, $C0, $00, //     ######
        $03, $F0, $00, //       ######
        $00, $78, $00, //          ####
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $1C, $38, $00, //    ###    ###
        $1F, $F0, $00, //    #########
        $1B, $E0, $00, //    ## #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3744 'T' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $3F, $FC, $00, //   ############
        $3F, $FC, $00, //   ############
        $31, $8C, $00, //   ##   ##   ##
        $31, $8C, $00, //   ##   ##   ##
        $31, $8C, $00, //   ##   ##   ##
        $31, $8C, $00, //   ##   ##   ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $0F, $F0, $00, //     ########
        $0F, $F0, $00, //     ########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3816 'U' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7E, $7E, $00, //  ######  ######
        $7E, $7E, $00, //  ######  ######
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $0C, $30, $00, //     ##    ##
        $0F, $F0, $00, //     ########
        $03, $C0, $00, //       ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3888 'V' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7F, $7F, $00, //  ####### #######
        $7F, $7F, $00, //  ####### #######
        $18, $0C, $00, //    ##       ##
        $0C, $18, $00, //     ##     ##
        $0C, $18, $00, //     ##     ##
        $0C, $18, $00, //     ##     ##
        $06, $30, $00, //      ##   ##
        $06, $30, $00, //      ##   ##
        $03, $60, $00, //       ## ##
        $03, $60, $00, //       ## ##
        $03, $60, $00, //       ## ##
        $01, $C0, $00, //        ###
        $01, $C0, $00, //        ###
        $00, $80, $00, //         #
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @3960 'W' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $FE, $3F, $80, // #######   #######
        $FE, $3F, $80, // #######   #######
        $30, $06, $00, //   ##         ##
        $30, $06, $00, //   ##         ##
        $30, $86, $00, //   ##    #    ##
        $19, $CC, $00, //    ##  ###  ##
        $19, $CC, $00, //    ##  ###  ##
        $1B, $6C, $00, //    ## ## ## ##
        $1B, $6C, $00, //    ## ## ## ##
        $1E, $7C, $00, //    ####  #####
        $0E, $38, $00, //     ###   ###
        $0E, $38, $00, //     ###   ###
        $0C, $18, $00, //     ##     ##
        $0C, $18, $00, //     ##     ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4032 'X' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7E, $7E, $00, //  ######  ######
        $7E, $7E, $00, //  ######  ######
        $18, $18, $00, //    ##      ##
        $0C, $30, $00, //     ##    ##
        $06, $60, $00, //      ##  ##
        $03, $C0, $00, //       ####
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $03, $C0, $00, //       ####
        $06, $60, $00, //      ##  ##
        $0C, $30, $00, //     ##    ##
        $18, $18, $00, //    ##      ##
        $7E, $7E, $00, //  ######  ######
        $7E, $7E, $00, //  ######  ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4104 'Y' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7C, $7E, $00, //  #####   ######
        $7C, $7E, $00, //  #####   ######
        $18, $18, $00, //    ##      ##
        $0C, $30, $00, //     ##    ##
        $06, $60, $00, //      ##  ##
        $06, $60, $00, //      ##  ##
        $03, $C0, $00, //       ####
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $0F, $F0, $00, //     ########
        $0F, $F0, $00, //     ########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4176 'Z' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $18, $18, $00, //    ##      ##
        $18, $30, $00, //    ##     ##
        $18, $60, $00, //    ##    ##
        $18, $C0, $00, //    ##   ##
        $01, $80, $00, //        ##
        $03, $00, $00, //       ##
        $06, $18, $00, //      ##    ##
        $0C, $18, $00, //     ##     ##
        $18, $18, $00, //    ##      ##
        $30, $18, $00, //   ##       ##
        $3F, $F8, $00, //   ###########
        $3F, $F8, $00, //   ###########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4248 '[' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $01, $F0, $00, //        #####
        $01, $F0, $00, //        #####
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $F0, $00, //        #####
        $01, $F0, $00, //        #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4320 '\' (17 pixels wide)
        $18, $00, $00, //    ##
        $18, $00, $00, //    ##
        $1C, $00, $00, //    ###
        $0C, $00, $00, //     ##
        $0E, $00, $00, //     ###
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $03, $00, $00, //       ##
        $03, $00, $00, //       ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $00, $C0, $00, //         ##
        $00, $C0, $00, //         ##
        $00, $60, $00, //          ##
        $00, $60, $00, //          ##
        $00, $70, $00, //          ###
        $00, $30, $00, //           ##
        $00, $38, $00, //           ###
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4392 ']' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $0F, $80, $00, //     #####
        $0F, $80, $00, //     #####
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $0F, $80, $00, //     #####
        $0F, $80, $00, //     #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4464 '^' (17 pixels wide)
        $00, $00, $00, //
        $00, $80, $00, //         #
        $01, $C0, $00, //        ###
        $03, $E0, $00, //       #####
        $07, $70, $00, //      ### ###
        $06, $30, $00, //      ##   ##
        $0C, $18, $00, //     ##     ##
        $18, $0C, $00, //    ##       ##
        $10, $04, $00, //    #         #
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4536 '_' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $FF, $FF, $00, // ################
        $FF, $FF, $00, // ################

        // @4608 '`' (17 pixels wide)
        $00, $00, $00, //
        $03, $00, $00, //       ##
        $03, $80, $00, //       ###
        $00, $E0, $00, //         ###
        $00, $60, $00, //          ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4680 'a' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $0F, $C0, $00, //     ######
        $1F, $E0, $00, //    ########
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $07, $F0, $00, //      #######
        $1F, $F0, $00, //    #########
        $38, $30, $00, //   ###     ##
        $30, $30, $00, //   ##      ##
        $30, $70, $00, //   ##     ###
        $1F, $FC, $00, //    ###########
        $0F, $BC, $00, //     ##### ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4752 'b' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $78, $00, $00, //  ####
        $78, $00, $00, //  ####
        $18, $00, $00, //    ##
        $18, $00, $00, //    ##
        $1B, $E0, $00, //    ## #####
        $1F, $F8, $00, //    ##########
        $1C, $18, $00, //    ###     ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $1C, $18, $00, //    ###     ##
        $7F, $F8, $00, //  ############
        $7B, $E0, $00, //  #### #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4824 'c' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $EC, $00, //       ##### ##
        $0F, $FC, $00, //     ##########
        $1C, $1C, $00, //    ###     ###
        $38, $0C, $00, //   ###       ##
        $30, $0C, $00, //   ##        ##
        $30, $00, $00, //   ##
        $30, $00, $00, //   ##
        $38, $0C, $00, //   ###       ##
        $1C, $1C, $00, //    ###     ###
        $0F, $F8, $00, //     #########
        $03, $F0, $00, //       ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4896 'd' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $78, $00, //          ####
        $00, $78, $00, //          ####
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $07, $D8, $00, //      ##### ##
        $1F, $F8, $00, //    ##########
        $18, $38, $00, //    ##     ###
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $18, $38, $00, //    ##     ###
        $1F, $FE, $00, //    ############
        $07, $DE, $00, //      ##### ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @4968 'e' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $E0, $00, //      ######
        $1F, $F8, $00, //    ##########
        $18, $18, $00, //    ##      ##
        $30, $0C, $00, //   ##        ##
        $3F, $FC, $00, //   ############
        $3F, $FC, $00, //   ############
        $30, $00, $00, //   ##
        $30, $00, $00, //   ##
        $18, $0C, $00, //    ##       ##
        $1F, $FC, $00, //    ###########
        $07, $F0, $00, //      #######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5040 'f' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $01, $FC, $00, //        #######
        $03, $FC, $00, //       ########
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $3F, $F8, $00, //   ###########
        $3F, $F8, $00, //   ###########
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $3F, $F0, $00, //   ##########
        $3F, $F0, $00, //   ##########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5112 'g' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $DE, $00, //      ##### ####
        $1F, $FE, $00, //    ############
        $18, $38, $00, //    ##     ###
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $18, $38, $00, //    ##     ###
        $1F, $F8, $00, //    ##########
        $07, $D8, $00, //      ##### ##
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $00, $38, $00, //           ###
        $0F, $F0, $00, //     ########
        $0F, $C0, $00, //     ######
        $00, $00, $00, //
        $00, $00, $00, //

        // @5184 'h' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $78, $00, $00, //  ####
        $78, $00, $00, //  ####
        $18, $00, $00, //    ##
        $18, $00, $00, //    ##
        $1B, $E0, $00, //    ## #####
        $1F, $F0, $00, //    #########
        $1C, $38, $00, //    ###    ###
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $7E, $7E, $00, //  ######  ######
        $7E, $7E, $00, //  ######  ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5256 'i' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $80, $00, //    ######
        $1F, $80, $00, //    ######
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $3F, $FC, $00, //   ############
        $3F, $FC, $00, //   ############
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5328 'j' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $C0, $00, //         ##
        $00, $C0, $00, //         ##
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $F0, $00, //    #########
        $1F, $F0, $00, //    #########
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $30, $00, //           ##
        $00, $70, $00, //          ###
        $1F, $E0, $00, //    ########
        $1F, $80, $00, //    ######
        $00, $00, $00, //
        $00, $00, $00, //

        // @5400 'k' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $3C, $00, $00, //   ####
        $3C, $00, $00, //   ####
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $F8, $00, //     ##  #####
        $0C, $F8, $00, //     ##  #####
        $0C, $C0, $00, //     ##  ##
        $0D, $80, $00, //     ## ##
        $0F, $80, $00, //     #####
        $0F, $00, $00, //     ####
        $0F, $80, $00, //     #####
        $0D, $C0, $00, //     ## ###
        $0C, $E0, $00, //     ##  ###
        $3C, $7C, $00, //   ####   #####
        $3C, $7C, $00, //   ####   #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5472 'l' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $80, $00, //    ######
        $1F, $80, $00, //    ######
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $3F, $FC, $00, //   ############
        $3F, $FC, $00, //   ############
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5544 'm' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $F7, $78, $00, // #### ### ####
        $FF, $FC, $00, // ##############
        $39, $CC, $00, //   ###  ###  ##
        $31, $8C, $00, //   ##   ##   ##
        $31, $8C, $00, //   ##   ##   ##
        $31, $8C, $00, //   ##   ##   ##
        $31, $8C, $00, //   ##   ##   ##
        $31, $8C, $00, //   ##   ##   ##
        $31, $8C, $00, //   ##   ##   ##
        $FD, $EF, $00, // ###### #### ####
        $FD, $EF, $00, // ###### #### ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5616 'n' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7B, $E0, $00, //  #### #####
        $7F, $F0, $00, //  ###########
        $1C, $38, $00, //    ###    ###
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $7E, $7E, $00, //  ######  ######
        $7E, $7E, $00, //  ######  ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5688 'o' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $03, $C0, $00, //       ####
        $0F, $F0, $00, //     ########
        $1C, $38, $00, //    ###    ###
        $38, $1C, $00, //   ###      ###
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $30, $0C, $00, //   ##        ##
        $38, $1C, $00, //   ###      ###
        $1C, $38, $00, //    ###    ###
        $0F, $F0, $00, //     ########
        $03, $C0, $00, //       ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5760 'p' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7B, $E0, $00, //  #### #####
        $7F, $F8, $00, //  ############
        $1C, $18, $00, //    ###     ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $18, $0C, $00, //    ##       ##
        $1C, $18, $00, //    ###     ##
        $1F, $F8, $00, //    ##########
        $1B, $E0, $00, //    ## #####
        $18, $00, $00, //    ##
        $18, $00, $00, //    ##
        $18, $00, $00, //    ##
        $7F, $00, $00, //  #######
        $7F, $00, $00, //  #######
        $00, $00, $00, //
        $00, $00, $00, //

        // @5832 'q' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $DE, $00, //      ##### ####
        $1F, $FE, $00, //    ############
        $18, $38, $00, //    ##     ###
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $30, $18, $00, //   ##       ##
        $18, $38, $00, //    ##     ###
        $1F, $F8, $00, //    ##########
        $07, $D8, $00, //      ##### ##
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $00, $18, $00, //            ##
        $00, $FE, $00, //         #######
        $00, $FE, $00, //         #######
        $00, $00, $00, //
        $00, $00, $00, //

        // @5904 'r' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $3E, $78, $00, //   #####  ####
        $3E, $FC, $00, //   ##### ######
        $07, $CC, $00, //      #####  ##
        $07, $00, $00, //      ###
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $06, $00, $00, //      ##
        $3F, $F0, $00, //   ##########
        $3F, $F0, $00, //   ##########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @5976 's' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $F8, $00, //      ########
        $0F, $F8, $00, //     #########
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $1F, $80, $00, //    ######
        $0F, $F0, $00, //     ########
        $00, $F8, $00, //         #####
        $18, $18, $00, //    ##      ##
        $18, $38, $00, //    ##     ###
        $1F, $F0, $00, //    #########
        $1F, $E0, $00, //    ########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6048 't' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $3F, $F0, $00, //   ##########
        $3F, $F0, $00, //   ##########
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $00, $00, //     ##
        $0C, $1C, $00, //     ##     ###
        $07, $FC, $00, //      #########
        $03, $F0, $00, //       ######
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6120 'u' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $78, $78, $00, //  ####    ####
        $78, $78, $00, //  ####    ####
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $18, $38, $00, //    ##     ###
        $0F, $FE, $00, //     ###########
        $07, $DE, $00, //      ##### ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6192 'v' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7C, $3E, $00, //  #####    #####
        $7C, $3E, $00, //  #####    #####
        $18, $18, $00, //    ##      ##
        $18, $18, $00, //    ##      ##
        $0C, $30, $00, //     ##    ##
        $0C, $30, $00, //     ##    ##
        $06, $60, $00, //      ##  ##
        $06, $60, $00, //      ##  ##
        $07, $E0, $00, //      ######
        $03, $C0, $00, //       ####
        $03, $C0, $00, //       ####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6264 'w' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $78, $3C, $00, //  ####     ####
        $78, $3C, $00, //  ####     ####
        $31, $18, $00, //   ##   #   ##
        $33, $98, $00, //   ##  ###  ##
        $33, $98, $00, //   ##  ###  ##
        $1A, $B0, $00, //    ## # # ##
        $1E, $F0, $00, //    #### ####
        $1E, $F0, $00, //    #### ####
        $1C, $60, $00, //    ###   ##
        $0C, $60, $00, //     ##   ##
        $0C, $60, $00, //     ##   ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6336 'x' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $3E, $7C, $00, //   #####  #####
        $3E, $7C, $00, //   #####  #####
        $0C, $30, $00, //     ##    ##
        $06, $60, $00, //      ##  ##
        $03, $C0, $00, //       ####
        $01, $80, $00, //        ##
        $03, $C0, $00, //       ####
        $06, $60, $00, //      ##  ##
        $0C, $30, $00, //     ##    ##
        $3E, $7C, $00, //   #####  #####
        $3E, $7C, $00, //   #####  #####
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6408 'y' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $7E, $1F, $00, //  ######    #####
        $7E, $1F, $00, //  ######    #####
        $18, $0C, $00, //    ##       ##
        $0C, $18, $00, //     ##     ##
        $0C, $18, $00, //     ##     ##
        $06, $30, $00, //      ##   ##
        $06, $30, $00, //      ##   ##
        $03, $60, $00, //       ## ##
        $03, $E0, $00, //       #####
        $01, $C0, $00, //        ###
        $00, $C0, $00, //         ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $03, $00, $00, //       ##
        $3F, $C0, $00, //   ########
        $3F, $C0, $00, //   ########
        $00, $00, $00, //
        $00, $00, $00, //

        // @6480 'z' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $18, $30, $00, //    ##     ##
        $18, $60, $00, //    ##    ##
        $00, $C0, $00, //         ##
        $01, $80, $00, //        ##
        $03, $00, $00, //       ##
        $06, $18, $00, //      ##    ##
        $0C, $18, $00, //     ##     ##
        $1F, $F8, $00, //    ##########
        $1F, $F8, $00, //    ##########
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6552 '{' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $E0, $00, //         ###
        $01, $E0, $00, //        ####
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $03, $80, $00, //       ###
        $07, $00, $00, //      ###
        $03, $80, $00, //       ###
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $E0, $00, //        ####
        $00, $E0, $00, //         ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6624 '|' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6696 '}' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $07, $00, $00, //      ###
        $07, $80, $00, //      ####
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $C0, $00, //        ###
        $00, $E0, $00, //         ###
        $01, $C0, $00, //        ###
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $01, $80, $00, //        ##
        $07, $80, $00, //      ####
        $07, $00, $00, //      ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //

        // @6768 '~' (17 pixels wide)
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $0E, $00, $00, //     ###
        $1F, $18, $00, //    #####   ##
        $3B, $B8, $00, //   ### ### ###
        $31, $F0, $00, //   ##   #####
        $00, $E0, $00, //         ###
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00, //
        $00, $00, $00 //
);

(******************************************************************************
function :      Software reset
******************************************************************************)
procedure EPD_3IN7_Reset;
begin
   setPin(EPD_RST_PIN, True);
   EPD_Delay_ms(30);
   setPin(EPD_RST_PIN, False);
   EPD_Delay_ms(3);
   setPin(EPD_RST_PIN, True);
   EPD_Delay_ms(30);
end;

(******************************************************************************
function :      send command
******************************************************************************)
procedure EPD_3IN7_SendCommand(Reg: Byte);
begin
   setPin(EPD_DC_PIN, False);
   setPin(EPD_CS_PIN, False);
   EPD_SPI_WriteByte(Reg);
   setPin(EPD_CS_PIN, True);
end;

(******************************************************************************
function :      send data
******************************************************************************)
procedure EPD_3IN7_SendData(Data: Byte);
begin
   setPin(EPD_DC_PIN, True);
   setPin(EPD_CS_PIN, False);
   EPD_SPI_WriteByte(Data);
   setPin(EPD_CS_PIN, True);
end;

Procedure EPD_3IN7_ReadBusy_HIGH;
var busy : Boolean;
begin
   repeat
      busy := getPin(EPD_BUSY_PIN);
   until not busy;
   EPD_Delay_ms(200);
end;

(******************************************************************************
function :      set the look-up tables
******************************************************************************)
Procedure EPD_3IN7_Load_LUT(lut: Byte);
var ii : integer;
begin
   EPD_3IN7_SendCommand($32);
   for ii := 0 to LutSize do
      case lut of
        0: EPD_3IN7_SendData(lut_4Gray_GC[ii]);
        1: EPD_3IN7_SendData(lut_1Gray_GC[ii]);
        2: EPD_3IN7_SendData(lut_1Gray_DU[ii]);
        3: EPD_3IN7_SendData(lut_1Gray_A2[ii]);
      otherwise
        WriteLn('There is no such LUT');
      end;
end;

(******************************************************************************
function :      Initialize the e-Paper register
******************************************************************************)
procedure EPD_3IN7_4Gray_Init;
begin
   EPD_3IN7_Reset;

   EPD_3IN7_SendCommand($12);
   EPD_Delay_ms(300);

   EPD_3IN7_SendCommand($46);
   EPD_3IN7_SendData($F7);
   EPD_3IN7_ReadBusy_HIGH;
   EPD_3IN7_SendCommand($47);
   EPD_3IN7_SendData($F7);
   EPD_3IN7_ReadBusy_HIGH;

   EPD_3IN7_SendCommand($01); // setting gate number
   EPD_3IN7_SendData($DF);
   EPD_3IN7_SendData($01);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($03); // set gate voltage
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($04); // set source voltage
   EPD_3IN7_SendData($41);
   EPD_3IN7_SendData($A8);
   EPD_3IN7_SendData($32);

   EPD_3IN7_SendCommand($11); // set data entry sequence
   EPD_3IN7_SendData($03);

   EPD_3IN7_SendCommand($3C); // set border
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($0C); // set booster strength
   EPD_3IN7_SendData($AE);
   EPD_3IN7_SendData($C7);
   EPD_3IN7_SendData($C3);
   EPD_3IN7_SendData($C0);
   EPD_3IN7_SendData($C0);

   EPD_3IN7_SendCommand($18); // set internal sensor on
   EPD_3IN7_SendData($80);

   EPD_3IN7_SendCommand($2C); // set vcom value
   EPD_3IN7_SendData($44);

   EPD_3IN7_SendCommand($37); // set display option, these setting turn on previous function
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($44); // setting X direction start/end position of RAM
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($17);
   EPD_3IN7_SendData($01);

   EPD_3IN7_SendCommand($45); // setting Y direction start/end position of RAM
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($DF);
   EPD_3IN7_SendData($01);

   EPD_3IN7_SendCommand($22); // Display Update Control 2
   EPD_3IN7_SendData($CF);
end;

(******************************************************************************
function :  Initialize the e-Paper register
******************************************************************************)
procedure EPD_3IN7_1Gray_Init;
begin
   EPD_3IN7_Reset;

   EPD_3IN7_SendCommand($12);
   EPD_Delay_ms(300);

   EPD_3IN7_SendCommand($46);
   EPD_3IN7_SendData($F7);
   EPD_3IN7_ReadBusy_HIGH;
   EPD_3IN7_SendCommand($47);
   EPD_3IN7_SendData($F7);
   EPD_3IN7_ReadBusy_HIGH;

   EPD_3IN7_SendCommand($01); // setting gate number
   EPD_3IN7_SendData($DF);
   EPD_3IN7_SendData($01);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($03); // set gate voltage
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($04); // set source voltage
   EPD_3IN7_SendData($41);
   EPD_3IN7_SendData($A8);
   EPD_3IN7_SendData($32);

   EPD_3IN7_SendCommand($11); // set data entry sequence
   EPD_3IN7_SendData($03);

   EPD_3IN7_SendCommand($3C); // set border
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($0C); // set booster strength
   EPD_3IN7_SendData($AE);
   EPD_3IN7_SendData($C7);
   EPD_3IN7_SendData($C3);
   EPD_3IN7_SendData($C0);
   EPD_3IN7_SendData($C0);

   EPD_3IN7_SendCommand($18); // set internal sensor on
   EPD_3IN7_SendData($80);

   EPD_3IN7_SendCommand($2C); // set vcom value
   EPD_3IN7_SendData($44);

   EPD_3IN7_SendCommand($37); // set display option, these setting turn on previous function
   EPD_3IN7_SendData($00);     //can switch 1 gray or 4 gray
   EPD_3IN7_SendData($FF);
   EPD_3IN7_SendData($FF);
   EPD_3IN7_SendData($FF);
   EPD_3IN7_SendData($FF);
   EPD_3IN7_SendData($4F);
   EPD_3IN7_SendData($FF);
   EPD_3IN7_SendData($FF);
   EPD_3IN7_SendData($FF);
   EPD_3IN7_SendData($FF);

   EPD_3IN7_SendCommand($44); // setting X direction start/end position of RAM
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($17);
   EPD_3IN7_SendData($01);

   EPD_3IN7_SendCommand($45); // setting Y direction start/end position of RAM
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($DF);
   EPD_3IN7_SendData($01);

   EPD_3IN7_SendCommand($22); // Display Update Control 2
   EPD_3IN7_SendData($CF);
end;

(******************************************************************************
function :      Clear screen
******************************************************************************)
procedure EPD_3IN7_4Gray_Clear;
var Width, Height, ii, jj : integer;
begin
   Width := (EPD_3IN7_WIDTH + 7) div 8;
   Height := EPD_3IN7_HEIGHT;

   EPD_3IN7_SendCommand($49);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendCommand($4E);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendCommand($4F);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($24);
   for jj := 1 to Height do
      for ii := 1 to Width do
         EPD_3IN7_SendData($FF);

   EPD_3IN7_SendCommand($4E);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($4F);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($26);
   for jj := 1 to Height do
      for ii := 1 to Width do
         EPD_3IN7_SendData($FF);

   EPD_3IN7_Load_LUT(0);
   EPD_3IN7_SendCommand($22);
   EPD_3IN7_SendData($C7);

   EPD_3IN7_SendCommand($20);
   EPD_3IN7_ReadBusy_HIGH;
end;

(******************************************************************************
function :  Clear screen
******************************************************************************)
procedure EPD_3IN7_1Gray_Clear;
var ii, jj : integer;
begin
   jj := (EPD_3IN7_WIDTH * EPD_3IN7_HEIGHT) div 8;

   EPD_3IN7_SendCommand($4E);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendCommand($4F);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($24);
   for ii := 1 to jj do
      EPD_3IN7_SendData($FF);

   EPD_3IN7_Load_LUT(2);

   EPD_3IN7_SendCommand($20);
   EPD_3IN7_ReadBusy_HIGH;
end;

(******************************************************************************
function :      Sends the image buffer in RAM to e-Paper and displays
******************************************************************************)
procedure EPD_3IN7_4Gray_Display(Image : PByte);
var ii, jj, kk: integer;
    temp1, temp2, temp3, temp4: Byte;
begin
   EPD_3IN7_SendCommand($49);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($4E);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($4F);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($24);
   for ii := 0 to 16800 - 1 do begin
      temp3 := 0;
      for jj := 0 to 1 do begin
         temp1 := Image[ii*2 + jj];
         for kk := 0 to 1 do begin
            temp2 := temp1 and $C0;
            case temp2 of
              $C0: temp4 := $01; //white
              $00: temp4 := $00; //black
              $80: temp4 := $00; //gray1
            else // $40
                   temp4 := $01; //gray2
            end;
            temp3 := (temp3 or temp4) << 1;

            temp1 := temp1 << 2;
            temp2 := temp1 and $C0;
            case temp2 of
              $C0: temp4 := $01; //white
              $00: temp4 := $00; //black
              $80: temp4 := $00; //gray1
            else // $40
                   temp4 := $01; //gray2
            end;
            temp3 := temp3 or temp4;
            if (jj<>1) or (kk<>1) then
               temp3 := temp3 << 1;

            temp1 := temp1 << 2;
         end;
      end;
      EPD_3IN7_SendData(temp3);
   end;

   // new  data
   EPD_3IN7_SendCommand($4E);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($4F);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($26);
   for ii := 0 to 16800 - 1 do begin
      temp3 := 0;
      for jj := 0 to 1 do begin
         temp1 := Image[ii*2 + jj];
         for kk := 0 to 1 do begin
            temp2 := temp1 and $C0;
            case temp2 of
              $C0: temp4 := $01; //white
              $00: temp4 := $00; //black
              $80: temp4 := $01; //gray1
            else // $40
                   temp4 := $00; //gray2
            end;
            temp3 := (temp3 or temp4) << 1;

            temp1 := temp1 << 2;
            temp2 := temp1 and $C0;
            case temp2 of
              $C0: temp4 := $01; //white
              $00: temp4 := $00; //black
              $80: temp4 := $01; //gray1
            else // $40
                   temp4 := $00; //gray2
            end;
            temp3 := temp3 or temp4;
            if (jj<>1) or (kk<>1) then
               temp3 := temp3 << 1;

            temp1 := temp1 << 2;
         end;
      end;
      EPD_3IN7_SendData(temp3);
   end;

   EPD_3IN7_Load_LUT(0);

   EPD_3IN7_SendCommand($22);
   EPD_3IN7_SendData($C7);

   EPD_3IN7_SendCommand($20);

   EPD_3IN7_ReadBusy_HIGH;
end;

(******************************************************************************
function :  Sends the image buffer in RAM to e-Paper and displays
******************************************************************************)
procedure EPD_3IN7_1Gray_Display(Image : PByte);
var ii, jj : integer;
begin
   jj := ((EPD_3IN7_WIDTH + 7) div 8) * EPD_3IN7_HEIGHT;

   EPD_3IN7_SendCommand($4E);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendCommand($4F);
   EPD_3IN7_SendData($00);
   EPD_3IN7_SendData($00);

   EPD_3IN7_SendCommand($24);
   for ii := 0 to jj - 1 do
      EPD_3IN7_SendData(not Image[ii]);

   EPD_3IN7_Load_LUT(2);
   EPD_3IN7_SendCommand($20);
   EPD_3IN7_ReadBusy_HIGH;
end;

(******************************************************************************
function :  Sends part of the image buffer in RAM to e-Paper and displays
notes:
    You can send a part of data to e-Paper,But this function is not recommended
    1.Xsize must be as big as EPD_3IN7_WIDTH
    2.Ypointer must be start at 0
******************************************************************************)
procedure EPD_3IN7_1Gray_Display_Part(Image : PByte; Xstart, Ystart, Xend, Yend : integer);
var ii, jj : integer;
begin
   jj := (((Xend - Xstart) + 7) div 8) * (Yend - Ystart);

   EPD_3IN7_SendCommand($44);
   EPD_3IN7_SendData(Xstart and $FF);
   EPD_3IN7_SendData((Xstart>>8) and $03);
   EPD_3IN7_SendData(Xend and $FF);
   EPD_3IN7_SendData((Xend>>8) and $03);
   EPD_3IN7_SendCommand($45);
   EPD_3IN7_SendData(Ystart and $FF);
   EPD_3IN7_SendData((Ystart>>8) and $03);
   EPD_3IN7_SendData(Yend and $FF);
   EPD_3IN7_SendData((Yend>>8) and $03);

   EPD_3IN7_SendCommand($24);
   for ii := 0 to jj - 1 do
      EPD_3IN7_SendData(Image[ii]);

   EPD_3IN7_Load_LUT(2);
   EPD_3IN7_SendCommand($20);
   EPD_3IN7_ReadBusy_HIGH;
end;

(******************************************************************************
function :      Enter sleep mode
******************************************************************************)
procedure EPD_3IN7_Sleep;
begin
   EPD_3IN7_SendCommand($50);
   EPD_3IN7_SendData($F7);
   EPD_3IN7_SendCommand($02);   //power off
   EPD_3IN7_SendCommand($07);   //deep sleep
   EPD_3IN7_SendData($A5);
end;

// Draw a character using the 24-high fixed pitch STMicroelectronics font.
procedure DrawChar24(c : Char);
var s, d : Pbyte;
   x, y, ii : integer;
begin
   if (c >= ' ') and (c <= '~') then begin
      ii := (integer(c) - 32) * 3 * 24;
      s := Font24_Table;
      s += ii;
      for y := 0 to 23 do begin
         ii := (y + thePort^.pnLoc.v) * (EPD_3IN7_WIDTH div 8);
         d := thePort^.portBits.baseAddr + ii + (thePort^.pnLoc.h div 8);
         for x := 0 to 2 do begin
            d^ := d^ or s^;
            s += 1;
            d += 1;
         end;
      end;
   end;
   thePort^.pnLoc.h += 24;
end;

procedure EPD_3in7_test;
begin
   WriteLn('EPD_3IN7_test Demo');
   EPD_SPI_Init;
   WriteLn('e-Paper Init and Clear...');
   EPD_3IN7_4Gray_Init;
   EPD_3IN7_4Gray_Clear;
   EPD_Delay_ms(250);

    //Create a new image cache
   InitGraf;

{
   Paint_NewImage(BlackImage, EPD_3IN7_WIDTH, EPD_3IN7_HEIGHT, 270, WHITE);
   Paint_SetScale(4);
   Paint_Clear(WHITE);

   //1.Select Image
   WriteLn('SelectImage:BlackImage');
   Paint_SelectImage(BlackImage);
   Paint_SetScale(4);
   Paint_Clear(WHITE);

   // 2.Drawing on the image
   WriteLn('Drawing:BlackImage');
   Paint_DrawPoint(10, 80, BLACK, DOT_PIXEL_1X1, DOT_STYLE_DFT);
   Paint_DrawPoint(10, 90, BLACK, DOT_PIXEL_2X2, DOT_STYLE_DFT);
   Paint_DrawPoint(10, 100, BLACK, DOT_PIXEL_3X3, DOT_STYLE_DFT);
   Paint_DrawLine(20, 70, 70, 120, BLACK, DOT_PIXEL_1X1, LINE_STYLE_SOLID);
   Paint_DrawLine(70, 70, 20, 120, BLACK, DOT_PIXEL_1X1, LINE_STYLE_SOLID);
   Paint_DrawRectangle(20, 70, 70, 120, BLACK, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
   Paint_DrawRectangle(80, 70, 130, 120, BLACK, DOT_PIXEL_1X1, DRAW_FILL_FULL);
   Paint_DrawCircle(45, 95, 20, BLACK, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
   Paint_DrawCircle(105, 95, 20, WHITE, DOT_PIXEL_1X1, DRAW_FILL_FULL);
   Paint_DrawLine(85, 95, 125, 95, BLACK, DOT_PIXEL_1X1, LINE_STYLE_DOTTED);
   Paint_DrawLine(105, 75, 105, 115, BLACK, DOT_PIXEL_1X1, LINE_STYLE_DOTTED);
   Paint_DrawString_EN(10, 0, 'waveshare', Font16, BLACK, WHITE);
   Paint_DrawString_EN(10, 20, 'hello world', Font12, WHITE, BLACK);
   Paint_DrawNum(10, 33, 123456789, &Font12, BLACK, WHITE);
   Paint_DrawNum(10, 50, 987654321, &Font16, WHITE, BLACK);
   Paint_DrawString_EN(10, 150, 'GRAY1 with black background', Font24, BLACK, GRAY1);
   Paint_DrawString_EN(10, 175, 'GRAY2 with white background', Font24, WHITE, GRAY2);
   Paint_DrawString_EN(10, 200, 'GRAY3 with white background', Font24, WHITE, GRAY3);
   Paint_DrawString_EN(10, 225, 'GRAY4 with white background', Font24, WHITE, GRAY4);
}
   EndGraf;

   // Sleep & close 5V
   EPD_3IN7_Sleep;

   EPD_Delay_ms(2000); //important, at least 2s
   EPD_SPI_Exit;
end;

initialization

end.
